/* 3dmap.c
 * Functions for drawing the 3d map in the map requestor
 */
#include "sim.h"
#include "client.h"
#include "graphics.h"

static Pixmap bmp;		/* pixmap for drawing map into */
static Pixmap pmp;		/* pixmap for map with player's location */
static GC mgc;			/* gc for drawing map */
static int mapw, maph;		/* size of map */

static unsigned long bg,wl;	/* background and wall colours */
static unsigned long pt,pl;	/* path and player colours */


/* init_map
 * Create map pixmap of given size.
 */
void init_map(Display *display, int w, int h)
{
/* Create pixmap and GC
 */
mapw = w; maph = h;
bmp = XCreatePixmap(display,DefaultRootWindow(display),mapw,maph,
		    DefaultDepth(display,DefaultScreen(display)));
pmp = XCreatePixmap(display,DefaultRootWindow(display),mapw,maph,
		    DefaultDepth(display,DefaultScreen(display)));
mgc = XCreateGC(display,bmp,0,NULL);

/* Allocate colours
 */
bg = best_colour(200,200,200);
wl = best_colour(0,0,0);
pt = best_colour(0,0,255);
pl = best_colour(255,0,0);
}


/* get_map_pixmap
 * Returns the pixmap with the map and player's location drawn into it
 */
Pixmap get_map_pixmap(void)
{
return pmp;
}


/* Info on current map scaling
 */
static double minx, miny;
static double maxx, maxy;
static double scale;

/* create_base_map
 * Create the pixmap with nodes and paths
 */
void create_base_map(Display *display)
{
double xscale, yscale;
node *n;
path *p;

/* Clear pixmap
 */
XSetForeground(display,mgc,bg);
XFillRectangle(display,bmp,mgc,0,0,mapw,maph);
XFlush(display);

/* Find extent of map
 */
minx = 1000000;  miny = 1000000;
maxx = -1000000; maxy = -1000000;
reset_node();
while((n = next_node())) {
	minx = min(minx,n->x); miny = min(miny,n->y);
	maxx = max(maxx,n->x); maxy = max(maxy,n->y);
	}
xscale = (double)mapw / (double)(maxx - minx);
yscale = (double)maph / (double)(maxy - miny);
scale  = min(xscale,yscale);
scale *= .9;			/* scale down so fit isn't so tight */
minx -= (maxx-minx)*.05;
miny -= (maxy-miny)*.05;

/* Draw all paths with circles at nodes
 */
XSetForeground(display,mgc,pt);
reset_path();
while((p = next_path())) {
	double x1,y1,x2,y2;

	x1 = (p->na->x - minx)*scale;
	y1 = (p->na->y - miny)*scale;
	x2 = (p->nb->x - minx)*scale;
	y2 = (p->nb->y - miny)*scale;

	XDrawLine(display,bmp,mgc,x1,y1,x2,y2);
	XDrawArc(display,bmp,mgc,x1-2,y1-2,4,4,0,360*64);
	XDrawArc(display,bmp,mgc,x2-2,y2-2,4,4,0,360*64);
	}
}


/* create_player_map
 * Create a map with nodes, paths and the player's location
 */
void create_player_map(Display *display, plview *plv)
{
double vx,vy,vz;	/* position in world space */
int x,y;		/* position on map */
int x1,y1,x2,y2,x3,y3;	/* vertices of direction arrow */

XCopyArea(display,bmp,pmp,mgc,0,0,mapw,maph,0,0);
current_position(plv,&vx,&vy,&vz);
x = (vx - minx)*scale;
y = (vy - miny)*scale;

x1 = x + cos(dtor(plv->ang + 110))*4; y1 = y + sin(dtor(plv->ang + 110))*4;
x2 = x + cos(dtor(plv->ang))*8;       y2 = y + sin(dtor(plv->ang))*8;
x3 = x + cos(dtor(plv->ang - 110))*4; y3 = y + sin(dtor(plv->ang - 110))*4;
XSetForeground(display,mgc,pl);
XDrawLine(display,pmp,mgc,x1,y1,x2,y2);
XDrawLine(display,pmp,mgc,x2,y2,x3,y3);
XFlush(display);
}

