/* client.h
 *
 * Function prototypes, macros and data structures used only by sim clients.
 */
#if !defined(CLIENT_H)
#define CLIENT_H

/* plview
 * Structure storing information about 3d view position and direction.
 */
#define PLAYER_HEIGHT 1.8	/* height of view pos above floor */
typedef struct {
	double x1,y1,z1;	/* position moving from */
	double x2,y2,z2;	/* position moving to (node actually at) */
	double dst;		/* distance moved so far */
	double range;		/* distance between start and end */
	double ang;		/* angle in x/y plane */
	double tilt;		/* tilt above horizontal */
	double tar_ang;		/* angle turning to */
	int move;		/* 0=still, 1=moving, 2=turning left, 3=right */
	} plview;


/* userprefs
 * User's preferences
 */
typedef struct {
	int default_3d;		/* graphics_level if none given */
	int recved_data;	/* 0=save, 1=ask, 2=display, 3=ignore */
	double move_speed;	/* smooth movement speed in 3d clients */
	char password[256];	/* password to allow access to the server */
	int display_transfer;	/* 1=display transfer status, 0=dont */
	int transfer_block;	/* size of transfer block */

	int x,y,w,h;		/* size and position of window (not saved) */
	} userprefs;

/* userservrefs
 * User's prefs for default server
 */
typedef struct {
	char server[256];	/* default server to use */
	int port;		/* default port to use */
	} userservprefs;


/* shapeattribs
 * Position / scale / rotation of a shape being parsed
 */
typedef struct {
	double cx,cy,cz;	/* centre postition */
	double sx,sy,sz;	/* scale factors */
	double rx,ry,rz;	/* rotation vector */
	double rang;		/* rotation angle */
	} shapeattribs;

/* From netclient.c
 */
int connect_server(char *name, player **p, int ilvl,
		   int (*shapeparser)(int fd));
int reconnect_server(char *name, player **p, int ilvl,
		     int (*shapeparser)(int fd));
void client_request(int fd, int req, ...);
void recv_display_data(char *servhost, char *type, int len, int port, int act,
		       char *fn);

/* From client.c
 */
int parse_client_args(int argc, char **argv, char **name);
int identify_command(char *cmd);
int validate_command(int opc, char **vec, int c);
void list_commands(void);
void command_help(char *cmd);
double *path_angles(node *pos);
double *sorted_path_angles(node *pos);
int next_angle(double *angs, int angc, double a);
int upload_data(int fd, char *file);
void rotate_left(person *pr, plview *plv);
void rotate_right(person *pr, plview *plv);

/* From simui.c
 */
int init_ui(void);
void create_base_ui_map(void);
void position_ui(plview *plv);
int interface_fd(void);
void check_ui(int fd, player *pl, person *pr, plview *plv);
void check_3d_ui(int fd, player *pl, person *pr, plview *plv);
void draw_frame(player *p, plview *plv);
double last_frame_len(void);
void display_message(char *fmt, ...);
void display_messagebox(char *fmt, ...);
void fill_here_list(person *pr);
void fill_inventory_list(person *pr);
void fill_hereinv_list(person *pr);
void fill_do_requestor(char *oname, char *acts[], int actc);
void fill_send_requestor(char **dats, int datc);
void normal_cursor(void);
void waiting_cursor(void);
void move_along(int fd, person *pr, plview *plv, int fwd);
void download_data(char *mtype, int port, int len);

/* From mimeclient.c
 */
void display_mime_data(char *fn, char *content_type, int content_length);
void run_mime_viewer(char *cmd, char *file);

/* From user.c
 */
void current_position(plview *plv, double *x, double *y, double *z);
userprefs *get_prefs(userprefs *p);
void put_prefs(userprefs *p);
int save_prefs(void);
userservprefs *get_sprefs(userservprefs *p);
void put_sprefs(userservprefs *p);
int save_sprefs(void);


/* Command numbers used by identify_command() and validate_command()
 */
#define cmd_unknown	-1
#define cmd_go		1
#define cmd_say		2
#define cmd_yell	3
#define cmd_get		4
#define cmd_look	5
#define cmd_do		6
#define cmd_exam	7
#define cmd_help	8
#define cmd_quit	9
#define cmd_drop	10
#define cmd_inv		11
#define cmd_face	12
#define cmd_give	13
#define cmd_upload	14
#define cmd_send	15
#define cmd_ls		16
#define cmd_rm		17
#define cmd_what	18
#define cmd_set		19
#define cmd_prefs	20
#define cmd_suicide	21
#define cmd_avatar	22
#define cmd_email	23
#define cmd_fullname	24
#define cmd_about	25

/* Request numbers from cgi program to webinv
 */
#define cgi_noop	0
#define cgi_image	1
#define cgi_left	2
#define cgi_right	3
#define cgi_forward	4
#define cgi_back	5
#define cgi_up		6
#define cgi_down	7
#define cgi_inv		8
#define cgi_quit	9
#define cgi_suicide	10
#define cgi_say		11

#endif /* !defined(CLIENT_H) */

