#include "jx.h"

/* Find the gadget on the gadget list and -
 * 1) unlink from list
 * 2) free it's children
 * 3) destroy it's window
 * 4) free it's memory
 */
void JXFreeGadget(Display *display, JXgadget *g)
{
JXgadget **prev;
JXtext *tmp;
int i;

if (g == NULL)
	return;

/* Find the pointer to this gadget */
for(prev = &glist; *prev && *prev != g; prev = &((*prev)->next))
	;
if (*prev == NULL) {
	printf("JX : Attempted to free non-existant gadget\n");
	exit(5);
	}

/* unlink from list */
*prev = g->next;

/* Free children, if this gadget has any */
switch(g->type) {
case slist:
	JXFreeGadget(display,g->jxgslist.slg);
	break;
case scroller:
	JXFreeGadget(display,g->jxgscroller.dir1);
	JXFreeGadget(display,g->jxgscroller.dir2);
	JXFreeGadget(display,g->jxgscroller.slg);
	break;
case prscroller:
	JXFreeGadget(display,g->jxgprscroller.dir1);
	JXFreeGadget(display,g->jxgprscroller.dir2);
	JXFreeGadget(display,g->jxgprscroller.slg);
	break;
case file:
	JXFreeGadget(display,g->jxgfile.ls);
	JXFreeGadget(display,g->jxgfile.fname);
	JXFreeGadget(display,g->jxgfile.path);
	JXFreeGadget(display,g->jxgfile.par);
	JXFreeGadget(display,g->jxgfile.but);
	JXFreeGadget(display,g->jxgfile.can);
	break;
case choice:
	JXFreeGadget(display,g->jxgchoice.head);
	for(i=0; i<g->jxgchoice.bcount; i++)
		JXFreeGadget(display,g->jxgchoice.buts[i]);
	break;
case combo:
	JXFreeGadget(display,g->jxgcombo.str);
	JXFreeGadget(display,g->jxgcombo.but);
	if (g->jxgcombo.open) XDestroyWindow(display,g->jxgcombo.mwin);
	break;
case twoword:
	JXFreeGadget(display,g->jxgtwoword.str1);
	JXFreeGadget(display,g->jxgtwoword.str2);
	JXFreeGadget(display,g->jxgtwoword.pr1);
	JXFreeGadget(display,g->jxgtwoword.pr2);
	JXFreeGadget(display,g->jxgtwoword.ok);
	JXFreeGadget(display,g->jxgtwoword.can);
	break;
case window:
	for(i=0; i<g->jxgwindow.gcount; i++)
		JXFreeGadget(display,g->jxgwindow.gl[i]);
	break;
case scrollpane:
	JXFreeGadget(display,g->jxgscrollpane.pn);
	JXFreeGadget(display,g->jxgscrollpane.hz);
	JXFreeGadget(display,g->jxgscrollpane.vt);
	break;
case message:
	for(i=0; i<g->jxgmessage.lnc; i++)
		JXFreeGadget(display,g->jxgmessage.ln[i]);
	JXFreeGadget(display,g->jxgmessage.ok);
	break;
default:
	/* no other gadgets have any children */
	break;
	}

/* destroy the gadget's window */
XDestroyWindow(display,g->window);
XFlush(display);

/* free extra data structures */
switch(g->type) {
case menu:
	tmp = g->jxgmenu.buf;
	while(tmp) {
		JXtext *tmp2;
		tmp2 = tmp->next;
		free(tmp);
		tmp = tmp2;
		}
	break;
case slist:
	tmp = g->jxgslist.buf;
	while(tmp) {
		JXtext *tmp2;
		tmp2 = tmp->next;
		free(tmp);
		tmp = tmp2;
		}
	break;
case choice:
	free(g->jxgchoice.buts);
	break;
case window:
	free(g->jxgwindow.gl);
	break;
case message:
	free(g->jxgmessage.ln);
default:
	break;
	/* no other gadgets have dangling data structures */
	}

/* free the gadget's memory */
if (g->label)
	free(g->label);
free(g);
}

