#include "jx.h"

/* Check if the event passed in is for one of the gadgets in the gadget
 * list, and if so act on it. This may involve re-drawing the gadget,
 * sending messages back to the parent gadget or other actions.
 *
 * display		- connection to the server
 * event		- the event to check
 *
 * Returns 1 if the event was for a gadget, 0 otherwise.
 */
int JXGadgetEvent(Display *display, XEvent *event)
{
JXgadget *pos;
JXgadget *clicked=NULL;
Window ewin;
char c;
KeySym keysym;
int key;
XComposeStatus compose;
int x,y,xy;
float slid;

/* find which window this event is for */
switch(event->type) {
case KeyPress :
        ewin=event->xkey.window;
        break;
case ButtonPress :
case ButtonRelease :
        ewin=event->xbutton.window;
        break;
case LeaveNotify :
case EnterNotify:
        ewin=event->xcrossing.window;
        break;
case Expose :
        ewin=event->xexpose.window;
        break;
case MotionNotify :
	ewin=event->xmotion.window;
	break;
case ClientMessage :
	ewin=event->xclient.window;
	break;
case ConfigureNotify :
	ewin=event->xconfigure.window;
	break;
default :
        return 0;
        }

/* find the gadget corresponding to that window */
for(pos=glist;pos != NULL;pos=pos->next) {
        if (pos->window == ewin)
                clicked = pos;
	/* allow for a click in the menu pane of a menu gadget, or in
	 * the menu pane of a combo gadget.
	 */
	if (pos->type == menu && pos->jxgmenu.mwin == ewin)
		clicked = pos;
	if (pos->type == combo && pos->jxgcombo.mwin == ewin)
		clicked = pos;
	/* since the string gadget has a sub-window, events for that window
	 * should go to the corresponding gadget.
	 */
	if (pos->type == string && pos->jxgstring.twin == ewin)
		clicked = pos;
	}
if (!clicked)
        return 0;

/* Ignore user events to inactive gadgets
 */
if (!clicked->active && (event->type == KeyPress ||
			 event->type == ButtonPress ||
			 event->type == ButtonRelease ||
			 event->type == LeaveNotify ||
			 event->type == EnterNotify ||
			 event->type == MotionNotify )) {
	return 1;
	}

switch (clicked->type) {
case boolean :
case pboolean :
        switch (event->type) {
        case ButtonPress :
		/* A button has been pressed in the window, but
		 * don't trigger an event until it is released.
		 */
                clicked->jxgboolean.state=1;
                JXRedrawGadget(display,clicked);
                break;
        case ButtonRelease :
		/* The mouse button has been released in the window,
		 * after being pressed. Trigger an event to the
		 * parent window.
		 */
                if (event->xbutton.x > 0 && event->xbutton.y >0 &&
                 event->xbutton.x < clicked->w && event->xbutton.y <
                 clicked->h && clicked->jxgboolean.state) {
                        clicked->jxgboolean.state=0;
			JXNotify(display,clicked->parent,clicked->number,
			 ClickEvent);
                        JXRedrawGadget(display,clicked);
                        }
                break;
        case LeaveNotify :
		/* The mouse has been moved out of the window. If the 
		 * gadget is in the pressed state, return it to normal.
		 */
                clicked->jxgboolean.state=0;
		clicked->in = False;
                JXRedrawGadget(display,clicked);
                break;
	case EnterNotify:
		clicked->in = True;
                JXRedrawGadget(display,clicked);
		break;
        case Expose :
                JXRedrawGadget(display,clicked);
                break;
	case ConfigureNotify :
		JXRedrawGadget(display,clicked);
                break;
                }
        break;
case toggle :
case ptoggle :
        switch (event->type) {
        case ButtonPress :
                clicked->jxgtoggle.state = !clicked->jxgtoggle.state;
                JXRedrawGadget(display,clicked);
		JXNotify(display,clicked->parent,clicked->number,ClickEvent);
                break;
        case Expose :
                JXRedrawGadget(display,clicked);
                break;
	case LeaveNotify:
		clicked->in = False;
                JXRedrawGadget(display,clicked);
		break;
	case EnterNotify:
		clicked->in = True;
                JXRedrawGadget(display,clicked);
		break;
                }
        break;
case string :
        switch (event->type) {
        case KeyPress :
		if (!clicked->jxgstring.editable)
			break;	/* can't edit this string */
                XLookupString(&(event->xkey),&c,1,&keysym,&compose);
                key=(int)keysym;
                if (key==XK_Delete || key==XK_BackSpace) {
			if (clicked->jxgstring.cur) {
				int i = clicked->jxgstring.cur - 1;

				while(i<clicked->jxgstring.len) {
					clicked->label[i]=clicked->label[i+1];
					i++;
					}
				clicked->jxgstring.cur--;
                                clicked->jxgstring.len--;
                                }
                        }
		else if (key == XK_Left) {
			if (clicked->jxgstring.cur)
				clicked->jxgstring.cur--;
			}
		else if (key == XK_Right) {
			if (clicked->jxgstring.cur != clicked->jxgstring.len)
				clicked->jxgstring.cur++;
			}
		else if (key == XK_Return) {
			if (clicked->jxgstring.grab) {
				/* Release grab.
				 */
				XUngrabKeyboard(display,CurrentTime);
				clicked->jxgstring.grab = False;
				}
			/* send message to parent window.
			 */
			JXNotify(display,clicked->parent,clicked->number,
			 ClickEvent);
			}
                else if (key < 256 && clicked->jxgstring.len <
				      clicked->jxgstring.maxlen-1) {
			int i = clicked->jxgstring.len+1;

			while(i>clicked->jxgstring.cur) {
				clicked->label[i] = clicked->label[i-1];
				i--;
				}
			clicked->label[clicked->jxgstring.cur] = c;
			clicked->jxgstring.cur++;
			clicked->jxgstring.len++;
                        }
                JXRedrawGadget(display,clicked);
                break;
        case Expose :
                JXRedrawGadget(display,clicked);
                break;
	case LeaveNotify:
		clicked->in = False;
                JXRedrawGadget(display,clicked);
		break;
	case EnterNotify:
		clicked->in = True;
                JXRedrawGadget(display,clicked);
		break;
	case ButtonPress:
		/* If the user clicks in this gadget, then it will
		 * grab all keyboard events until return is hit, or
		 * the mouse button is clicked again.
		 */
		if (!clicked->jxgstring.editable)
			break;	/* can't edit this string */
		if (event->xbutton.button == 1) {
			if (clicked->jxgstring.grab) {
				/* already grabbing... let go */
				XUngrabKeyboard(display,CurrentTime);
				clicked->jxgstring.grab = False;
				}
			else {
				/* snarf all keyboard events */
				XGrabKeyboard(display,clicked->window,False,
				 GrabModeAsync,GrabModeAsync,CurrentTime);
				clicked->jxgstring.grab = True;
				}
			JXRedrawGadget(display,clicked);
			}
		else if (event->xbutton.button == 2) {
			/* Paste in from cut buffer.
			 */
			char *paste;
			int nbytes;

			paste = XFetchBytes(display,&nbytes);
			if (paste) {
				strncpy(clicked->label,paste,nbytes);
				clicked->jxgstring.len = nbytes;
				JXRedrawGadget(display,clicked);
				}
			}
		break;
                }
        break;

case slider :
	switch (event->type) {
	case ButtonPress :
		clicked->jxgslider.xyrel = -1;
		if (clicked->jxgslider.horiz)
			xy=event->xbutton.x;
		else
			xy=event->xbutton.y;
		if (xy > clicked->jxgslider.pos+clicked-> jxgslider.slsize)
			clicked->jxgslider.pos += clicked->jxgslider.
			 slsize;
		else if (xy < clicked->jxgslider.pos)
			clicked->jxgslider.pos -= clicked->jxgslider.
			 slsize;
		else
			clicked->jxgslider.xyrel=xy-clicked->
			 jxgslider.pos;
		JXRedrawGadget(display,clicked);
		break;
	case Expose :
		JXRedrawGadget(display,clicked);
		break;
	case ButtonRelease :
		JXRedrawGadget(display,clicked);
		break;
	case MotionNotify :
		if (clicked->jxgslider.xyrel == -1)
			break;
		if (clicked->jxgslider.horiz)
			xy=event->xmotion.x;
		else
			xy=event->xmotion.y;
		clicked->jxgslider.pos=xy-clicked->jxgslider.xyrel;
		break;
	case LeaveNotify:
		clicked->in = False;
                JXRedrawGadget(display,clicked);
		break;
	case EnterNotify:
		clicked->in = True;
                JXRedrawGadget(display,clicked);
		break;
		}

	if (clicked->jxgslider.horiz)
		xy=clicked->w;
	else
		xy=clicked->h;
	if (clicked->jxgslider.pos < 0)
		clicked->jxgslider.pos=0;
	else if (clicked->jxgslider.pos > xy-clicked->jxgslider.slsize-4)
		clicked->jxgslider.pos=xy-clicked->jxgslider.slsize-4;
	clicked->jxgslider.npos=(float)(clicked->jxgslider.pos)/
	 (xy-clicked->jxgslider.slsize-4);
	if (event->type == ButtonRelease || event->type == MotionNotify) {
		/* send move event */
		if (event->type == ButtonRelease) {
			/* only when the button is released on the slider
			 * is the NewSliderPos message sent.
			 */
			JXNotify(display,clicked->parent,clicked->number,
			 NewSliderPos);
			}
		else if (clicked->jxgslider.report) {
			/* the ClickEvent message is sent every time
			 * the slider is moved, even while the mouse
			 * button is still down.
			 */
			JXNotify(display,clicked->parent,clicked->number,
			 ClickEvent);
			if (clicked->jxgslider.smooth)
				JXRedrawGadget(display,clicked);
			}
		}
	break;

case slist :
	switch(event->type) {
	case ClientMessage :			/* message from child */
		JXRedrawGadget(display,clicked);
		break;
	case Expose :
		clicked->jxgslist.cline = -1;	/* force re-draw */
		JXRedrawGadget(display,clicked);
		break;
	case ButtonPress:
		/* A click somewhere in the list of items
		 */
		y = event->xbutton.y;
		if (y < 3 || y > clicked->h-3)
			break;
		y = (clicked->h - y) / (font_info->ascent+font_info->descent);
		JXNotify(display,clicked->parent,clicked->number,
		 y + clicked->jxgslist.cline);
		break;
		}
	break;

case scroller :
	switch(event->type) {
	case ClientMessage :
		slid=clicked->jxgscroller.slg->jxgslider.npos;
		if (JXGMatch(clicked->jxgscroller.dir1,event)) {
			/* up arrow clicked.
			 */
			JXMoveSlider(display,clicked->jxgscroller.
			 slg,slid-clicked->jxgscroller.step);
			JXNotify(display,clicked->parent,clicked->number,
			 NewSliderPos);
			}
		if (JXGMatch(clicked->jxgscroller.dir2,event)) {
			/* down arrow clicked
			 */
			JXMoveSlider(display,clicked->jxgscroller.
			 slg,slid+clicked->jxgscroller.step);
			JXNotify(display,clicked->parent,clicked->number,
			 NewSliderPos);
			}
		if (JXGMatch(clicked->jxgscroller.slg,event)) {
			/* slider moved. Send the slider movement mesg
			 * on to this gadget's parent.
			 */
			JXNotify(display,clicked->parent,clicked->number,
			 DataFromEvent(event));
			}
		break;
		}
	break;

case menu: 
	switch(event->type) {
	case ButtonPress: {
		/* The open menu button has been pressed.
		 */
		int w,h,x,y,i,fh;
		JXtext *tx;

		if (clicked->jxgmenu.open)
			break;
		clicked->jxgmenu.open = True;
		/* find width of menu, allowing for 2
		 * pixels of space on each side of the
		 * menu item.
		 */
		w = clicked->w;
		for(tx=clicked->jxgmenu.buf; tx; tx=tx->next)
			if (tx->width+4 > w)
				w=tx->width+4;

		/* find height of menu, allowing for some 
		 * space between menu items.
		 */
		fh = font_info->ascent + font_info->descent + 5;
		h = fh * clicked->jxgmenu.lines;

		clicked->jxgmenu.w = w;
		clicked->jxgmenu.h = h;

		/* Find position of menu.
		 */
		x = clicked->x;
		y = clicked->y + clicked->h;

		/* create the menu window.
		 */
		clicked->jxgmenu.mwin = XCreateSimpleWindow(display,
					clicked->parent,x,y,w,h,0,
					clicked->botcolour,clicked->backcolour);
		XSelectInput(display,clicked->jxgmenu.mwin,ButtonPressMask |
			     ExposureMask);
		/*
		XSetTransientForHint(display,clicked->jxgmenu.mwin,
				     clicked->parent);
		*/
		XMapWindow(display,clicked->jxgmenu.mwin);
		XFlush(display);

		/* draw the menu items, and re-draw the gadget.
		 */
		JXRedrawMenu(display,clicked);
		JXRedrawGadget(display,clicked);
		break;
		}
	case ButtonRelease: {
		/* The mouse button has been released, possibly over
		 * a menu item. Close the menu, and send a message to the
		 * parent.
		 */
		int item,fontht;

		if (!clicked->jxgmenu.open)
			break;		/* menu isn't open?! */
		/* Close the menu.
		 */
		clicked->jxgmenu.open = False;
		XUnmapWindow(display,clicked->jxgmenu.mwin);
		XDestroyWindow(display,clicked->jxgmenu.mwin);
		JXRedrawGadget(display,clicked);

		/* Check if the mouse button was released over a menu
		 * item.
		 */
		if (event->xbutton.x < 0 || event->xbutton.x > clicked->
		    jxgmenu.w || event->xbutton.y < clicked->h) {
			/* Button was released outside the menu.
			 */
			break;
			}

		/* notify the parent */
		fontht = font_info->ascent + font_info->descent + 5;
		item = (event->xbutton.y - clicked->h) / fontht;
		if (item >= 0 && item <clicked->jxgmenu.lines)
			JXNotify(display,clicked->parent,clicked->number,item);

		break;
		}

	case Expose: {
		if (ewin == clicked->window) {
			/* expose is on menu button */
			JXRedrawGadget(display,clicked);
			}
		else {
			if (clicked->jxgmenu.open) {
				/* re-draw list of menu items */
				JXRedrawMenu(display,clicked);
				}
			}
		break;
		}

	case EnterNotify:
		clicked->in = True;
		JXRedrawGadget(display,clicked);
		break;

	case LeaveNotify:
		clicked->in = False;
		JXRedrawGadget(display,clicked);
		break;

	default:
		break;
		}
	break;

case stext:
	JXRedrawGadget(display,clicked);
	break;

case prslider :
	switch (event->type) {
	case ButtonPress :
		/* a button has been pressed somewhere in the gadget.
		 */
		clicked->jxgprslider.xyrel = -1;
		/* use the x or y part of the button event, depending
		 * on whether the gadget is horizontal or vertical.
		 */
		if (clicked->jxgprslider.horiz)
			xy = event->xbutton.x;
		else
			xy = event->xbutton.y;

		/* if the button click is outside the slider, step it along
		 * in the direction of the point clicked. otherwise, 
		 * remember where in the slider the click was made.
		 */
		if (xy > clicked->jxgprslider.pos+clicked->jxgprslider.lsize)
			clicked->jxgprslider.pos += clicked->jxgprslider.
			 lsize;
		else if (xy < clicked->jxgprslider.pos)
			clicked->jxgprslider.pos -= clicked->jxgprslider.
			 lsize;
		else
			clicked->jxgprslider.xyrel=xy-clicked->
			 jxgprslider.pos;
		break;
	case MotionNotify :
		/* The pointer is being moved gadget, with the button
		 * down. Move the slider part as well.
		 */
		if (clicked->jxgprslider.xyrel != -1) {
			if (clicked->jxgprslider.horiz)
				xy=event->xmotion.x;
			else
				xy=event->xmotion.y;
			clicked->jxgprslider.pos=xy-clicked->jxgprslider.xyrel;
			}
		break;
	case LeaveNotify:
		clicked->in = False;
                JXRedrawGadget(display,clicked);
		break;
	case EnterNotify:
		clicked->in = True;
                JXRedrawGadget(display,clicked);
		break;
		}

	if (clicked->jxgprslider.horiz)
		xy=clicked->w;
	else
		xy=clicked->h;

	/* Check if the slider part has been moved off one end or the
	 * other of the gadget.
	 */
	if (clicked->jxgprslider.pos < 0)
		clicked->jxgprslider.pos=0;
	else if (clicked->jxgprslider.pos > xy-clicked->jxgprslider.lsize-4)
		clicked->jxgprslider.pos=xy-clicked->jxgprslider.lsize-4;

	/* re-calculate the proportional position of the slider inside
	 * the gadget.
	 */
	if (clicked->jxgprslider.frac < 1.0)
		clicked->jxgprslider.npos = clicked->jxgprslider.pos/
		 (float)(xy-clicked->jxgprslider.lsize-4);
	else
		clicked->jxgprslider.npos = 1.0;

	JXRedrawGadget(display,clicked);
	if (event->type == ButtonRelease || event->type == MotionNotify) {
		/* send move event */
		if (event->type == ButtonRelease) {
			/* only when the button is released on the slider
			 * is the NewSliderPos message sent.
			 */
			JXNotify(display,clicked->parent,clicked->number,
			 NewSliderPos);
			}
		else if (clicked->jxgprslider.report) {
			/* the ClickEvent message is sent every time
			 * the slider is moved, even while the mouse
			 * button is still down.
			 */
			JXNotify(display,clicked->parent,clicked->number,
			 ClickEvent);
			if (clicked->jxgprslider.smooth)
				JXRedrawGadget(display,clicked);
			}
		}
	break;

case prscroller :
	switch(event->type) {
	case ClientMessage :
		slid=clicked->jxgprscroller.slg->jxgprslider.npos;
		if (JXGMatch(clicked->jxgprscroller.dir1,event)) {
			/* up arrow clicked */
			JXMovePrSlider(display,clicked->jxgprscroller.
			 slg,slid-0.1);
			JXNotify(display,clicked->parent,clicked->number,
			 NewSliderPos);
			}
		if (JXGMatch(clicked->jxgprscroller.dir2,event)) {
			/* down arrow clicked */
			JXMovePrSlider(display,clicked->jxgprscroller.
			 slg,slid+0.1);
			JXNotify(display,clicked->parent,clicked->number,
			 NewSliderPos);
			}
		if (JXGMatch(clicked->jxgprscroller.slg,event)) {
			/* slider moved. Send the slider movement mesg
			 * on to this gadget's parent.
			 */
			JXNotify(display,clicked->parent,clicked->number,
			 DataFromEvent(event));
			}
		break;
		}
	break;

case file:
	switch(event->type) {
	case Expose:
		JXRedrawGadget(display,clicked);
		break;
	case ClientMessage:
		/* A message from one of the sub-parts of the file
		 * gadget.
		 */
		if (JXGMatch(ButtonFromFile(clicked),event)) {
			/* Load/Save pressed.
			 */
			JXNotify(display,clicked->parent,clicked->number,
			 ClickEvent);
			}
		else if (JXGMatch(FilelistFromFile(clicked),event)) {
			/* A file chosen from list.
			 * If this is a directory, enter it otherwise
			 * put it in the name string gadget.
			 */
			int i;
			JXtext *tx;
			char tpath[200];

			/* Find name clicked. */
			tx = FilelistFromFile(clicked)->jxgslist.buf;
			if (!tx)
				break;
			for(i=0; i<DataFromEvent(event) && tx->next; i++)
				tx = tx->next;

			sprintf(tpath,"%s/%s",FilepathStringFromFile(clicked),
				tx->str);
			if (JXIsDirectory(tpath) == 0) {
				/* Click is on an ordinary file.
				 */
				strcpy(FilenameStringFromFile(clicked),
				 tx->str);
				FilenameFromFile(clicked)->jxgstring.len =
				 strlen(tx->str);
				JXRedrawGadget(display,
				 FilenameFromFile(clicked));
				}
			else if (JXIsDirectory(tpath) == 1) {
				/* Click is on a directory.
				 */
				char *pth;

				pth = FilepathStringFromFile(clicked);
				if (pth[strlen(pth)-1] != '/') 
					strcat(pth,"/");
				strcat(pth,tx->str);
				FilepathFromFile(clicked)->jxgstring.len =
				 strlen(pth);
				JXRedrawGadget(display,
				 FilepathFromFile(clicked));
				JXFileList(display,clicked,pth);
				JXRedrawGadget(display,
				 FilelistFromFile(clicked));
				}
			else if (JXIsDirectory(tpath) == -1) {
				/* Click is on an un-openable file.
				 */
				char *err = "File cannot be opened";

				strcpy(FilenameStringFromFile(clicked),err);
				FilenameFromFile(clicked)->jxgstring.len =
				 strlen(err);
				JXRedrawGadget(display,
				 FilenameFromFile(clicked));
				}
			}
		else if (JXGMatch(ParentFromFile(clicked),event)) {
			/* Parent gadget clicked.
			 */
			char *pth;
			int i;

			pth = FilepathStringFromFile(clicked);
			if (!strcmp(pth,"/")) {
				/* At the root directory. */
				break;
				}
			for(i=strlen(pth)-1; i>=0 && pth[i] != '/'; i--)
				;
			if (i<0) {
				/* No parent of this directory */
				break;
				}
			pth[i] = '\0';
			if (!strlen(pth))
				strcpy(pth,"/");
			FilepathFromFile(clicked)->jxgstring.len = strlen(pth);
			JXRedrawGadget(display,FilepathFromFile(clicked));
			JXFileList(display,clicked,pth);
			JXRedrawGadget(display,FilelistFromFile(clicked));
			}
		else if (JXGMatch(FilepathFromFile(clicked),event)) {
			/* Return hit in filepath string box.
			 */
			int l;
			char *path;

			path = FilepathStringFromFile(clicked);
			l = strlen(path);
			if (path[l-1] == '/') {
				path[l-1] = '\0';
				FilepathFromFile(clicked)->jxgstring.len--;
				}
			JXFileList(display,clicked,path);
			JXRedrawGadget(display,FilelistFromFile(clicked));
			}
		else if (JXGMatch(CancelFromFile(clicked),event)) {
			/* Cancel gadget clicked.
			 */
			JXNotify(display,clicked->parent,clicked->number,
			 CancelEvent);
			}
		break;
		}
	break;

case choice:
	switch(event->type) {
	case ClientMessage: {
		/* A message has arrived from one of the buttons.
		 */
		int i;

		for(i=0; i<clicked->jxgchoice.bcount; i++)
			if (JXGMatch(clicked->jxgchoice.buts[i],event))
				JXNotify(display,clicked->parent,clicked->
					 number,i);
		break;
		}
	case Expose:
		JXRedrawGadget(display,clicked);
		break;
		}
	break;

case combo:
	switch(event->type) {
	case ClientMessage:
		if (JXGMatch(clicked->jxgcombo.but,event)) {
			/* A click on the menu button. Raise or lower
			 * the menu.
			 */
			if (!clicked->jxgcombo.open &&
			    clicked->jxgcombo.lines != 0) {
				/* Open the menu */
				int w,h,x,y,i,fh;
				JXtext *tx;
				XWindowAttributes winattr;

				clicked->jxgcombo.open = True;

				/* find menu width */
				w = clicked->w;
				for(tx=clicked->jxgcombo.buf; tx; tx=tx->next)
					if (tx->width+4 > w)
						w=tx->width+4;

				/* find menu height */
				fh = font_info->ascent + font_info->descent + 5;
				h = fh * clicked->jxgcombo.lines;
				clicked->jxgcombo.w = w;
				clicked->jxgcombo.h = h;

				/* find menu position */
				JXGetWindowPos(display,clicked->window,&x,&y);
				y += clicked->h;

				/* create memu */
				clicked->jxgcombo.mwin =
				 XCreateSimpleWindow(display,root,x,y,w,h,0,
						     clicked->botcolour,
						     clicked->backcolour);
				XSetTransientForHint(display,
						     clicked->jxgcombo.mwin,
						     clicked->window);
				XGetWindowAttributes(display,
						     clicked->window,
						     &winattr);
				XSetWindowColormap(display,
						   clicked->jxgcombo.mwin,
						   winattr.colormap);
				XSelectInput(display,clicked->jxgcombo.mwin,
				 ButtonPressMask | ExposureMask);
				XMapWindow(display,clicked->jxgcombo.mwin);
				XFlush(display);

				JXRedrawMenu(display,clicked);
				}
			else {
				if (!clicked->jxgcombo.open)
					break;
				/* close the menu */
				clicked->jxgcombo.open = False;
				XUnmapWindow(display,clicked->jxgcombo.mwin);
				XDestroyWindow(display,clicked->jxgcombo.mwin);
				}
			}
		break;

	case ButtonPress:
		if (clicked->jxgcombo.open && ewin == clicked->jxgcombo.mwin) {
			/* A click somewhere in the menu.
			 */
			int fontht, item;
			JXtext *tx;

			/* close the menu */
			clicked->jxgcombo.open = False;
			XUnmapWindow(display,clicked->jxgcombo.mwin);
			XDestroyWindow(display,clicked->jxgcombo.mwin);

			/* find which item was chosen, and copy it into
			 * the string gadget. */
			fontht = font_info->ascent + font_info->descent + 5;
			item   = event->xbutton.y / fontht;
			clicked->jxgcombo.cch = item;
			for(tx=clicked->jxgcombo.buf; item; item--,tx=tx->next)
				;
			strcpy(clicked->jxgcombo.str->label,tx->str);
			clicked->jxgcombo.str->jxgstring.len = strlen(tx->str);
			JXRedrawGadget(display,clicked->jxgcombo.str);
			JXNotify(display,clicked->parent,clicked->number,
			 ClickEvent);
			}
		break;

	case Expose:
		if (clicked->jxgcombo.open && ewin == clicked->jxgcombo.mwin)
			JXRedrawMenu(display,clicked);
		break;
		}
	break;

case twoword:
	switch(event->type) {
	case ClientMessage:
		if (JXGMatch(clicked->jxgtwoword.ok,event))
			JXNotify(display,clicked->parent,clicked->number,
				 ClickEvent);
		else if (JXGMatch(clicked->jxgtwoword.can,event))
			JXNotify(display,clicked->parent,clicked->number,
				 CancelEvent);
		break;
		}
	break;

case window:
	switch(event->type) {
	case Expose:
		JXRedrawGadget(display,clicked);
		break;
	case ClientMessage:
		if (!clicked->jxgwindow.toplevel) {
			/* A message from one of the sub-gadgets, for the parent
			 */
			JXNotify(display,clicked->parent,SenderFromEvent(event),
				 DataFromEvent(event));
			}
		else {
			/* This gadget isn't forwarding sub-gadget messages,
			 * because it's the top-level one. Let the caller deal
			 * with it.
			 */
			return 0;
			}
		break;
	case ConfigureNotify:
		if (event->xconfigure.width != clicked->w ||
		    event->xconfigure.height != clicked->h)
			JXMoveGadget(display,clicked,-1,-1,
						     event->xconfigure.width,
						     event->xconfigure.height);
		break;
		}
	break;

case bevelbox:
	JXRedrawGadget(display,clicked);
	break;

case pane:
	switch(event->type) {
	case Expose:
		JXRedrawGadget(display,clicked);
		break;
	case ButtonPress:
		clicked->jxgpane.mx = event->xbutton.x + clicked->jxgpane.x;
		clicked->jxgpane.my = event->xbutton.y + clicked->jxgpane.y;
		break;
	case MotionNotify: {
		clicked->jxgpane.x = clicked->jxgpane.mx - event->xmotion.x;
		clicked->jxgpane.y = clicked->jxgpane.my - event->xmotion.y;

		if (clicked->jxgpane.x < 0) clicked->jxgpane.x = 0;
		if (clicked->jxgpane.y < 0) clicked->jxgpane.y = 0;
		if (clicked->jxgpane.x > clicked->jxgpane.w - clicked->w)
			clicked->jxgpane.x = clicked->jxgpane.w - clicked->w;
		if (clicked->jxgpane.y > clicked->jxgpane.h - clicked->h)
			clicked->jxgpane.y = clicked->jxgpane.h - clicked->h;
		JXRedrawGadget(display,clicked);

		if (clicked->jxgpane.report)
			JXNotify(display,clicked->parent,clicked->number,
				 ClickEvent);
		} break;
		} /* end switch */
	break;

case scrollpane:
	switch(event->type) {
	case Expose:
		JXRedrawGadget(display,clicked);
		break;
	case ClientMessage:
		if (JXGMatch(clicked->jxgscrollpane.hz,event)) {
			/* horizontal scroller moved
			 */
			float np = clicked->jxgscrollpane.hz->jxgprscroller.slg
				   ->jxgprslider.npos;
			JXgadget *pn = clicked->jxgscrollpane.pn;

			pn->jxgpane.x = np*(pn->jxgpane.w - pn->w);
			JXRedrawGadget(display,pn);
			}
		else if (JXGMatch(clicked->jxgscrollpane.vt,event)) {
			/* vertical scroller moved
			 */
			float np = clicked->jxgscrollpane.vt->jxgprscroller.slg
				   ->jxgprslider.npos;
			JXgadget *pn = clicked->jxgscrollpane.pn;

			pn->jxgpane.y = np*(pn->jxgpane.h - pn->h);
			JXRedrawGadget(display,pn);
			}
		else if (JXGMatch(clicked->jxgscrollpane.pn,event)) {
			/* pane moved
			 */
			JXgadget *pn = clicked->jxgscrollpane.pn;
			double nph = pn->jxgpane.x /
				     (double)(pn->jxgpane.w - pn->w);
			double npv = pn->jxgpane.y /
				     (double)(pn->jxgpane.h - pn->h);

			JXSetOption(display,clicked->jxgscrollpane.hz,
				    SliderNPos,nph,0);
			JXRedrawGadget(display,clicked->jxgscrollpane.hz);
			JXSetOption(display,clicked->jxgscrollpane.vt,
				    SliderNPos,npv,0);
			JXRedrawGadget(display,clicked->jxgscrollpane.vt);
			}
		if (clicked->jxgscrollpane.report)
			JXNotify(display,clicked->parent,clicked->number,
				 ClickEvent);
		break;
		}
	break;

case message:
	switch(event->type) {
	case Expose:
		JXRedrawGadget(display,clicked);
		break;
	case ClientMessage:
		JXFreeGadget(display,clicked);	/* suicide */
		break;
		}
	break;

case statusgauge:
	clicked->jxgstatusgauge.totalredraw = 1;
	JXRedrawGadget(display,clicked);
	break;

default :
	break;
        }
return 1;
}

