#include "jx.h"

/* JXMoveGadget - moves or re-sizes an existing gadget, and re-draws it
 * in it's new size. This entails removing the gadget's window, re-creating
 * a new window at the correct size, changing the gadget size entries
 * and re-calculating the pixel position (for a slider gadget). Of course,
 * this may require the moving of sub-gadgets as well, through recursive
 * calls :(
 *
 * display		- connection to the X server
 * g			- the gadget to change
 * x			- new x position
 * y			- new y position
 * w			- new width
 * h			- new height
 *
 * note - a value of -1 for x,y,w or h indicates no change
 */
void JXMoveGadget(Display *display, JXgadget *g, int x, int y, int w,
 int h)
{
int xy;
XEvent event;
int oldw = g->w, oldh = g->h;

/* find new x,y width and height */
g->x = (x < 0 ? g->x : x);
g->y = (y < 0 ? g->y : y);
g->w = (w < 0 ? g->w : w);
g->h = (h < 0 ? g->h : h);
if (x < 0 || y < 0)
	XResizeWindow(display,g->window,g->w,g->h);
else
	XMoveResizeWindow(display,g->window,g->x,g->y,g->w,g->h);

switch(g->type) {
case string:
	XMoveResizeWindow(display,g->jxgstring.twin,2,2,g->w-4, g->h-4);
	break;
case slider:
	/* need to re-calc slider pos and size.
	 */
	if (g->jxgslider.horiz) {
		g->jxgslider.slsize = g->h - 4;
		xy = g->w;
		}
	else {
		g->jxgslider.slsize = g->w - 4;
		xy = g->h;
		}
	g->jxgslider.pos = (int)(g->jxgslider.npos * 
				(xy - g->jxgslider.slsize - 4));
	/* re-create the background pixmap.
	 */
	XFreePixmap(display,g->dbuf);
	g->dbuf = XCreatePixmap(display,g->window,g->w,g->h,def_depth);
	break;
case slist:
	g->jxgslist.cline = -1;		/* for redraw */
	/* call JXMoveGadget again on slider part, to re-position it
	 * correctly.
	 */
	JXMoveGadget(display,g->jxgslist.slg,g->w-16,0,16,g->h);
	break;
case scroller:
	/* call JXMoveGadget on the arrows.
	 */
	JXMoveGadget(display,g->jxgscroller.dir1,0,0,min(g->w,g->h),
	 min(g->w,g->h));
	JXMoveGadget(display,g->jxgscroller.dir2,
	 (g->w > g->h ? g->w-min(g->w,g->h) : 0),
	 (g->w > g->h ? 0 : g->h-min(g->w,g->h)),min(g->w,g->h),
	 min(g->w,g->h));
	/* and on the slider.
	 */
	JXMoveGadget(display,g->jxgscroller.slg,((g->w > g->h) ? 
	 min(g->w,g->h): 0),
	 ((g->w > g->h) ? 0 : min(g->w,g->h)),((g->w > g->h) ? 
	 g->w-min(g->w,g->h)*2 : g->w),
	 ((g->w > g->h) ? g->h : g->h-min(g->w,g->h)*2));
	/* if anyone actually reads this, please ignore the above
	 * coding style bogosity.
	 */
	break;
case prslider:
	/* need to re-calc slider pos and size */
	if (g->jxgprslider.horiz) {
		g->jxgprslider.wsize = g->h - 4;
		g->jxgprslider.lsize = (int)(g->jxgprslider.frac < 1 ?
		 (g->w-4)*g->jxgprslider.frac : g->w-4);
		xy = g->w - 4;
		}
	else {
		g->jxgprslider.wsize = g->w - 4;
		g->jxgprslider.lsize = (int)(g->jxgprslider.frac < 1 ?
		 (g->h-4)*g->jxgprslider.frac : g->h-4);
		xy = g->h - 4;
		}
	g->jxgprslider.pos = (int)(g->jxgprslider.npos * 
				  (xy - g->jxgprslider.lsize));
	/* re-create the background pixmap.
	 */
	XFreePixmap(display,g->dbuf);
	g->dbuf = XCreatePixmap(display,g->window,g->w,g->h,def_depth);
	break;
case prscroller:
	/* call JXMoveGadget on the arrows.
	 */
	JXMoveGadget(display,g->jxgprscroller.dir1,0,0,min(g->w,g->h),
	 min(g->w,g->h));
	JXMoveGadget(display,g->jxgprscroller.dir2,
	 (g->w > g->h ? g->w-min(g->w,g->h) : 0),
	 (g->w > g->h ? 0 : g->h-min(g->w,g->h)),min(g->w,g->h),
	 min(g->w,g->h));
	/* and on the slider.
	 */
	JXMoveGadget(display,g->jxgprscroller.slg,((g->w > g->h) ? 
	 min(g->w,g->h): 0),
	 ((g->w > g->h) ? 0 : min(g->w,g->h)),((g->w > g->h) ? 
	 g->w-min(g->w,g->h)*2 : g->w),
	 ((g->w > g->h) ? g->h : g->h-min(g->w,g->h)*2));
	/* The above code is incomprensible, even to me :)
	 */
	break;
case file:
	JXMoveGadget(display,g->jxgfile.ls,4,5,g->w-8,g->h-95);
	JXMoveGadget(display,g->jxgfile.fname,4,g->h-85,g->w-8,22);
	JXMoveGadget(display,g->jxgfile.path,4,g->h-55,g->w-8,22);
	JXMoveGadget(display,g->jxgfile.par,4,g->h-25,g->w/3-8,22);
	JXMoveGadget(display,g->jxgfile.but,g->w/3+4,g->h-25,g->w/3-8,20);
	JXMoveGadget(display,g->jxgfile.can,g->w*2/3+4,g->h-25,g->w/3-8,20);
	break;
case combo:
	JXMoveGadget(display,g->jxgcombo.str,0,0,g->w-g->h,g->h);
	JXMoveGadget(display,g->jxgcombo.but,g->w-g->h,0,g->h,g->h);
	break;
case window: {
	/* move and resize all sub-gadgets
	 */
	int i;
	double xsc = (double)g->w / (double)oldw;
	double ysc = (double)g->h / (double)oldh;

	for(i=0; i<g->jxgwindow.gcount; i++) {
		JXgadget *sg = g->jxgwindow.gl[i];

		JXMoveGadget(display,sg,sg->x*xsc,sg->y*ysc,sg->w*xsc,
			     sg->h*ysc);
		}
	} break;
case pane:
	if (g->jxgpane.x > g->jxgpane.w - g->w)
		g->jxgpane.x = g->jxgpane.w - g->w;
	if (g->jxgpane.y > g->jxgpane.h - g->h)
		g->jxgpane.y = g->jxgpane.h - g->h;
	break;
case scrollpane:
	JXMoveGadget(display,g->jxgscrollpane.pn,2,2,g->w-24,g->h-24);
	JXMoveGadget(display,g->jxgscrollpane.hz,2,g->h-22,g->w-24,20);
	JXMoveGadget(display,g->jxgscrollpane.vt,g->w-22,2,20,g->h-24);
	break;
	}

/* re-draw the gadget, and it's children */
JXRedrawGadget(display,g);
}

