#include "jx.h"

/* JXNewGadget - new, improved, varargs way of creating a gadget.
 * This is only a varargs stub which calls JXRealAddGadget to do the
 * actual work.
 *
 * display	- connection to server
 * win		- window gadget is in
 * col1		- highlight colour
 * col2		- dark-side colour
 * col3		- background colour
 * type		- gadget type (boolean, toggle, etc...)
 *
 * rest of args depend on type...
 *
 * boolean   : int x, int y, int w, int h, char *label
 * toggle    : int x, int y, int w, int h, int state, char *label
 * string    : int x, int y, int w, int h, char *string, int maxstr
 * slider    : int x, int y, int w, int h, double start
 * slist     : int x, int y, int w, int h, {char *line, ... }, NULL
 * pboolean  : int x, int y, int w, int h, Pixmap pic, int pic_w, int pic_h
 * ptoggle   : int x, int y, int w, int h, int state, Pixmap pic, int pic_w,
 *             int pic_h
 * leftarr   : int x, int y, int w, int h
 * rightarr  : int x, int y, int w, int h
 * uparr     : int x, int y, int w, int h
 * downarr   : int x, int y, int w, int h
 * scroller  : int x, int y, int w, int h, double start
 * menu      : int x, int y, int w, int h, char *name, {char *entry, ... }, NULL
 * stext     : int x, int y, int w, int h, JXalign pos, char *text
 * prslider  : int x, int y, int w, int h, double start, double frac
 * prscroller: int x, int y, int w, int h, double start, double frac
 * file      : int x, int y, int w, int h, char *name, char *path, char *but
 * filewin   : int x, int y, int w, int h, char *name, char *path, char *but
 * choice    : int x, int y, int bw, int bh, char *head, int bc, {char *but,...}
 * choicewin : int x, int y, int bw, int bh, char *head, int bc, {char *but,...}
 * combo     : int x, int y, int w, int h, int default, {char * item, ...}, NULL
 * twoword   : int x, int y, int w, int h, char *prompt1, char *prompt2
 * twowordwin: int x, int y, int w, int h, char *prompt1, char *prompt2
 * window    : int x, int y, int w, int h, char *title, Bool popup, Bool bevel
 * bevelbox  : int x, int y, int w, int h, Bool indent
 * pane      : int x, int y, int w, int h, Pixmap pm
 * scrollpane: int x, int y, int w, int h, Pixmap pm
 * message   : int x, int y, char **lines, int linec
 */
JXgadget *JXNewGadget(Display *display,Window win,long col1,long col2,
 long col3,JXgtype type,...)
{
va_list ap;
JXgadget *ret;

va_start(ap,type);				/* start varargs */
ret = JXRealNewGadget(display,win,col1,col2,col3,type,ap);
va_end(ap);
return ret;
}

/* JXRealNewGadget - does the work of creating a new gadget
 */
JXgadget *JXRealNewGadget(Display *display,Window win,long col1,long col2,
 long col3,JXgtype type,va_list ap)
{
JXgadget *gadg;
XEvent event;
Bool map=False;

gadg=(JXgadget *)calloc(sizeof(JXgadget),1);
gadg->next=glist;
glist=gadg;
gadg->parent=win;
gadg->topcolour=col1;
gadg->botcolour=col2;
gadg->hicolour = gadg->backcolour=col3;
gadg->type=type;
gadg->number=gadget_count++;
gadg->owner=NULL;
gadg->dbuf = (Pixmap)0;
gadg->active = 1;

switch(type) {
case boolean : {
	int x,y,w,h;
	char *label;

	/* parse arguments */
   	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	label=va_arg(ap,char *);

	/* store into new gadget */
	gadg->x=x;
	gadg->y=y;
	gadg->w=w;
	gadg->h=h;
	gadg->label=(char *)malloc(strlen(label)+1);
	strcpy(gadg->label,label);
	gadg->jxgboolean.state=0;

	map=True;
	gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSelectInput(display,gadg->window,Mboolean);
	}
	break;

case toggle : {
	int x,y,w,h;
	int state;
	char *label;

	/* parse arguments */
   	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	state=va_arg(ap,int);
	label=va_arg(ap,char *);

	/* store into new gadget */
	gadg->x=x;
	gadg->y=y;
	gadg->w=w;
	gadg->h=h;
	gadg->label=(char *)malloc(strlen(label)+1);
	strcpy(gadg->label,label);
	gadg->jxgtoggle.state=state;

	map=True;
	gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSelectInput(display,gadg->window,Mtoggle);
	}
	break;

case string : {
	int x,y,w,h;
	char *str;
	int maxstr;

	/* get varargs */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	str=va_arg(ap,char *);
	maxstr=va_arg(ap,int);
	if (maxstr < strlen(str)+1)
		maxstr=strlen(str)+1;

	/* set up gadget */
	gadg->x=x;
	gadg->y=y;
	gadg->w=w;
	gadg->h=h;
	gadg->label=(char *)malloc(maxstr);
	strcpy(gadg->label,str);
	gadg->jxgstring.len      = strlen(str);
	gadg->jxgstring.maxlen   = maxstr;
	gadg->jxgstring.grab     = False;
	gadg->jxgstring.aln      = a_left;
	gadg->jxgstring.cur      = strlen(str);
	gadg->jxgstring.editable = True;

	map=True;
	gadg->window = XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	gadg->jxgstring.twin = XCreateSimpleWindow(display,gadg->window,2,2,
			       w-4,h-4,0,0,col3);
	XMapWindow(display,gadg->jxgstring.twin);
	XSelectInput(display,gadg->jxgstring.twin,Mstring);
	XSelectInput(display,gadg->window,Mstring);
	}
	break;

case slider : {
	int x,y,w,h;
	double start;
	int xy;

	/* parse arguments */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	start=va_arg(ap,double);

	/* store into gadget */
	gadg->x=x;
	gadg->y=y;
	gadg->w=w;
	gadg->h=h;
	gadg->label=NULL;
	gadg->jxgslider.npos=start;
	gadg->jxgslider.report = True;
	gadg->jxgslider.smooth = False;
	gadg->jxgslider.wrap   = False;
	if (h > w) {
		/* vertical */
		gadg->jxgslider.horiz=False;
		gadg->jxgslider.slsize=w-4;
		xy=h;
		}
	else {
		/* horizontal */
		gadg->jxgslider.horiz=True;
		gadg->jxgslider.slsize=h-4;
		xy=w;
		}
	gadg->jxgslider.pos= (int)(start*(xy-gadg->jxgslider.slsize-4));

	/* Create the window */
	map=True;
	gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSelectInput(display,gadg->window,Mslider);

	/* Create a pixmap for double buffering */
	gadg->dbuf = XCreatePixmap(display,gadg->window,w,h,def_depth);
	}
	break;

case slist : {
	int x,y,w,h;
	char *tx;

	/* varargs bit */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);

	/* store position and size */
	gadg->x=x;
	gadg->y=y;
	gadg->w=w;
	gadg->h=h;
	gadg->jxgslist.buf   = NULL;
	gadg->jxgslist.lines = 0;
	gadg->jxgslist.cline = -1;
	gadg->jxgslist.wrap  = True;

	/* map itself */
	gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSelectInput(display,gadg->window,Mslist);
        XMapWindow(display,gadg->window);
        XFlush(display);
        /* XIfEvent(display,&event,exposepred,""); */
	XWindowEvent(display,gadg->window,StructureNotifyMask,&event);

	/* create child */
	gadg->jxgslist.slg=JXNewGadget(display,gadg->window,col1,col2,
	 col3,prscroller,w-16,0,16,h,0.0,1.0);
	gadg->jxgslist.slg->owner=gadg;

	/* add new lines to list */
	JXRedrawGadget(display,gadg);
	while(tx=va_arg(ap,char *))
		JXAddListText(display,gadg,tx);
	JXRedrawGadget(display,gadg);
	map=False;
	}
	break;

case pboolean : {
	int x,y,w,h;
	Pixmap pic;
	int pic_w,pic_h;

	/* parse var-args */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	pic=va_arg(ap,Pixmap);
	pic_w=va_arg(ap,int);
	pic_h=va_arg(ap,int);

	/* store into gadget */
	gadg->x=x;
	gadg->y=y;
	gadg->w=w;
	gadg->h=h;
	gadg->jxgpboolean.state=0;
	gadg->jxgpboolean.pic=pic;
	gadg->jxgpboolean.pic_w=pic_w;
	gadg->jxgpboolean.pic_h=pic_h;
	gadg->jxgpboolean.state=0;
	gadg->label=NULL;

	map=True;
	gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSelectInput(display,gadg->window,Mboolean);
	}
	break;

case ptoggle : {
	int x,y,w,h,state;
	Pixmap pic;
	int pic_w,pic_h;

	/* parse var-args */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	state=va_arg(ap,int);
	pic=va_arg(ap,Pixmap);
	pic_w=va_arg(ap,int);
	pic_h=va_arg(ap,int);

	/* store into gadget */
	gadg->x=x;
	gadg->y=y;
	gadg->w=w;
	gadg->h=h;
	gadg->jxgptoggle.state=state;
	gadg->jxgptoggle.pic=pic;
	gadg->jxgptoggle.pic_w=pic_w;
	gadg->jxgptoggle.pic_h=pic_h;
	gadg->jxgptoggle.state=0;
	gadg->label=NULL;

	map=True;
	gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSelectInput(display,gadg->window,Mtoggle);
	}
	break;

case leftarr :
case rightarr :
case uparr :
case downarr : {
	int x,y,w,h;
	Pixmap temp;
	JXgadget *ngad;

	/* get args from command line */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);

	/* choose the right pixmap */
	switch(type) {
	case leftarr :
		temp=jx_left;
		break;
	case rightarr :
		temp=jx_right;
		break;
	case uparr :
		temp=jx_up;
		break;
	case downarr :
		temp=jx_down;
		break;
		}
	/* create a pboolean gadget */
	gadg->type=pboolean;
	gadg->x=x;
        gadg->y=y;
        gadg->w=w;
        gadg->h=h;
        gadg->jxgpboolean.pic=temp;
        gadg->jxgpboolean.pic_w=arrow_w;
        gadg->jxgpboolean.pic_h=arrow_h;
        gadg->jxgpboolean.state=0;
        gadg->label=NULL;

        map=True;
        gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
        XSelectInput(display,gadg->window,Mboolean);
        }
        break;

case scroller : {
	int x,y,w,h;
	double start;
	int cx,cy;

	/* get info out of var-args */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	start=va_arg(ap,double);

	/* store into gadget */
	gadg->x=x;
	gadg->y=y;
	gadg->h=h;
	gadg->w=w;
	gadg->label=NULL;
	gadg->jxgscroller.horiz=(x > y);
	gadg->jxgscroller.step = 0.1;

	/* map main window */
	gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSelectInput(display,gadg->window,Mscroller);
	XMapWindow(display,gadg->window);
	XFlush(display);
	/* XIfEvent(display,&event,exposepred,""); */
	XWindowEvent(display,gadg->window,StructureNotifyMask,&event);

	/* create children */
	gadg->jxgscroller.dir1=JXNewGadget(display,gadg->window,col1,
	 col2,col3,(w > h) ? leftarr : uparr,0,0,min(w,h),min(w,h));
	gadg->jxgscroller.dir2=JXNewGadget(display,gadg->window,col1,
	 col2,col3,(w > h) ? rightarr : downarr,(w > h) ? w-min(w,h) : 0,
	 (w > h) ? 0 : h-min(w,h),min(w,h),min(w,h));
	gadg->jxgscroller.slg=JXNewGadget(display,gadg->window,col1,
	 col2,col3,slider,(w > h) ? min(w,h): 0,(w > h) ? 0 : min(w,h),
	 (w > h) ? w-min(w,h)*2 : w,(w > h) ? h : h-min(w,h)*2,start);

	map=False;
	}
	break;

case menu: {
	int x,y,w,h;
	char *name, *tx;

	/* get info out of var-args */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	name=va_arg(ap,char *);

	/* store into gadget */
	gadg->x = x;
	gadg->y = y;
	gadg->w = w;
	gadg->h = h;
	gadg->label = (char *)malloc(strlen(name)+1);
	strcpy(gadg->label,name);
	gadg->jxgmenu.open = False;
	gadg->jxgmenu.buf = NULL;
	gadg->jxgmenu.lines = 0;

	/* map main window */
	gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSelectInput(display,gadg->window,Mmenu);

	/* add menu items */
	while(tx=va_arg(ap,char *))
		JXAddMenuItem(display,gadg,tx);
	map = True;
	}
	break;

case stext: {
	int x,y,w,h;
	JXalign pos;
	char *text;

	/* get info out of var-args */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	pos=va_arg(ap,JXalign);
	text=va_arg(ap,char *);

	/* store into gadget */
	gadg->x = x;
	gadg->y = y;
	gadg->w = w;
	gadg->h = h;
	gadg->jxgstext.align = pos;
	gadg->label = (char *)malloc(strlen(text)+1);
	strcpy(gadg->label,text);

	/* create gadget window */
	map=True;
	gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSetWindowBackground(display,gadg->window,col3);
	XSelectInput(display,gadg->window,Mstext);
	}
	break;
	
case prslider : {
	int x,y,w,h;
	double start,frac;
	int xy;

	/* parse arguments */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	start=va_arg(ap,double);
	frac=va_arg(ap,double);

	/* store into gadget */
	gadg->x=x;
	gadg->y=y;
	gadg->w=w;
	gadg->h=h;
	gadg->label=NULL;
	gadg->jxgprslider.npos=start;
	gadg->jxgprslider.frac=frac;
	gadg->jxgprslider.report = True;
	gadg->jxgprslider.smooth = False;
	gadg->jxgprslider.wrap   = False;
	if (h > w) {
		/* slider moves vertically */
		gadg->jxgprslider.horiz=False;
		gadg->jxgprslider.wsize=w-4;
		gadg->jxgprslider.lsize = (int)(frac < 1 ? (h-4)*frac : h-4);
		xy = h-4;
		}
	else {
		/* slider moves horizontally */
		gadg->jxgprslider.horiz=True;
		gadg->jxgprslider.wsize=h-4;
		gadg->jxgprslider.lsize = (int)(frac < 1 ? (w-4)*frac : w-4);
		xy = w-4;
		}
	gadg->jxgprslider.pos=(int)(start*(xy-gadg->jxgprslider.lsize));

	/* create the window */
	map=True;
	gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSelectInput(display,gadg->window,Mprslider);

	/* Create a pixmap for double buffering */
	gadg->dbuf = XCreatePixmap(display,gadg->window,w,h,def_depth);
	}
	break;

case prscroller : {
	int x,y,w,h;
	double start,frac;
	int cx,cy;

	/* get info out of var-args */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	start=va_arg(ap,double);
	frac=va_arg(ap,double);

	/* store into gadget */
	gadg->x=x;
	gadg->y=y;
	gadg->h=h;
	gadg->w=w;
	gadg->label=NULL;
	gadg->jxgprscroller.horiz=(x > y);
	gadg->jxgprscroller.step = 0.1;

	/* map main window */
	gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSelectInput(display,gadg->window,Mprscroller);
	XMapWindow(display,gadg->window);
	XFlush(display);
	/* XIfEvent(display,&event,exposepred,""); */
	XWindowEvent(display,gadg->window,StructureNotifyMask,&event);

	/* create children */
	gadg->jxgprscroller.dir1=JXNewGadget(display,gadg->window,col1,
	 col2,col3,(w > h) ? leftarr : uparr,0,0,min(w,h),min(w,h));
	gadg->jxgprscroller.dir2=JXNewGadget(display,gadg->window,col1,
	 col2,col3,(w > h) ? rightarr : downarr,(w > h) ? w-min(w,h) : 0,
	 (w > h) ? 0 : h-min(w,h),min(w,h),min(w,h));
	gadg->jxgprscroller.slg=JXNewGadget(display,gadg->window,col1,
	 col2,col3,prslider,(w > h) ? min(w,h): 0,(w > h) ? 0 : min(w,h),
	 (w > h) ? w-min(w,h)*2 : w,(w > h) ? h : h-min(w,h)*2,start,frac);

	map=False;
	}
	break;

case filewin:
case file: {
	int x,y,w,h;
	char *name, *path, *but;

	/* get info out of var-args */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	name=va_arg(ap,char *);
	path=va_arg(ap,char *);
	but=va_arg(ap,char *);

	/* store into gadget */
	gadg->x = x;
	gadg->y = y;
	gadg->w = w;
	gadg->h = h;

	/* map main window */
	if (type == filewin) {
		XWindowAttributes winattr;

		/* Window is separate from creator window.
		 */
		gadg->window = XCreateSimpleWindow(display,root,x,y,
			       w,h,0,0,col3);
		XSetTransientForHint(display,gadg->window,win);
		gadg->type = file;

		/* Use the colourmap of the parent.
		 */
		XGetWindowAttributes(display,win,&winattr);
		XSetWindowColormap(display,gadg->window,winattr.colormap);	
		}
	else {
		/* Window is a child of the creator window.
		 */
		gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
		}
	XSelectInput(display,gadg->window,Mfile);
	XMapWindow(display,gadg->window);
	XFlush(display);
	/* XIfEvent(display,&event,exposepred,""); */
	XWindowEvent(display,gadg->window,StructureNotifyMask,&event);

	/* Create the file gadget parts */
	gadg->jxgfile.ls = JXNewGadget(display,gadg->window,col1,col2,
			    col3,slist,4,5,w-8,h-95,NULL);
	gadg->jxgfile.fname = JXNewGadget(display,gadg->window,col1,col2,
			      col3,string,4,h-85,w-8,22,name,80);
	gadg->jxgfile.path   = JXNewGadget(display,gadg->window,col1,col2,
			      col3,string,4,h-55,w-8,22,path,80);
	gadg->jxgfile.par   = JXNewGadget(display,gadg->window,col1,col2,
			      col3,boolean,4,h-25,w/3-8,20,"Parent");
	gadg->jxgfile.but   = JXNewGadget(display,gadg->window,col1,col2,
			      col3,boolean,w/3+4,h-25,w/3-8,20,but);
	gadg->jxgfile.can   = JXNewGadget(display,gadg->window,col1,col2,
			      col3,boolean,w*2/3+4,h-25,w/3-8,20,"Cancel");
	JXSetOption(display,gadg->jxgfile.ls,WrapList,False,0);

	/* Use right alignment for the file/path names */
	JXSetOption(display,gadg->jxgfile.fname,StringAlign,a_right,0);
	JXSetOption(display,gadg->jxgfile.path,StringAlign,a_right,0);

	/* fill in the list box */
	JXFileList(display,gadg,path);

	map = False;
	}
	break;

case choice:
case choicewin: {
	int x,y,bw,bh;
	int w,h;
	char *title, *but;
	int bc;
	int i;

	/* get info out of var-args */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	bw=va_arg(ap,int);
	bh=va_arg(ap,int);
	title=va_arg(ap,char *);
	bc=va_arg(ap,int);
	w = (bw+10)*bc;
	h = bh + font_info->ascent + font_info->descent + 20;

	/* store into gadget */
	gadg->x = x;
	gadg->y = y;
	gadg->w = w;
	gadg->h = h;

	/* map main window */
	if (type == choicewin) {
		XWindowAttributes winattr;

		/* Window is separate from creator window.
		 */
		gadg->window = XCreateSimpleWindow(display,root,x,y,
			       w,h,0,0,col3);
		XSetTransientForHint(display,gadg->window,win);
		gadg->type = choice;

		/* Use the colourmap of the parent.
		 */
		XGetWindowAttributes(display,win,&winattr);
		XSetWindowColormap(display,gadg->window,winattr.colormap);	
		}
	else {
		/* Window is a child of the creator window.
		 */
		gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
		}
	XSelectInput(display,gadg->window,Mchoice);
	XMapWindow(display,gadg->window);
	XFlush(display);
	/* XIfEvent(display,&event,exposepred,""); */
	XWindowEvent(display,gadg->window,StructureNotifyMask,&event);

	/* Create heading.
	 */
	gadg->jxgchoice.head = JXNewGadget(display,gadg->window,col1,col2,
			       col3,stext,4,10,w-8,h-bh-8,a_centre,title);
	/* Create child buttons.
	 */
	gadg->jxgchoice.bcount = bc;
	gadg->jxgchoice.buts = (JXgadget **)calloc(bc,sizeof(JXgadget *));
	for(i=0; i<bc; i++) {
		but = va_arg(ap,char *);
		gadg->jxgchoice.buts[i] = JXNewGadget(display,gadg->window,col1,
					  col2,col3,boolean,i*(bw+10)+5,h-bh-3,
					  bw,bh,but);
		}
	map = False;
	}
	break;

case combo: {
	int x,y,w,h;
	int def;
	char *tx;
	int tnum = 0;

	/* parse var-args */
	x = va_arg(ap,int);
	y = va_arg(ap,int);
	w = va_arg(ap,int);
	h = va_arg(ap,int);
	def = va_arg(ap,int);

	/* store into gadget */
	gadg->x = x;
	gadg->y = y;
	gadg->w = w;
	gadg->h = h;
	gadg->jxgcombo.open = False;
	gadg->jxgcombo.buf = NULL;
	gadg->jxgcombo.lines = 0;
	gadg->jxgcombo.cch = def;

	/* Create main window */
	gadg->window = XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSelectInput(display,gadg->window,Mcombo);
	XMapWindow(display,gadg->window);
	XFlush(display);
	/* XIfEvent(display,&event,exposepred,""); */
	XWindowEvent(display,gadg->window,StructureNotifyMask,&event);

	/* Create the sub-gadgets */
	gadg->jxgcombo.str = JXNewGadget(display,gadg->window,col1,col2,col3,
			     string,0,0,w-h,h,"",100);
	gadg->jxgcombo.but = JXNewGadget(display,gadg->window,col1,col2,col3,
			     downarr,w-h,0,h,h);

	/* Add menu items */
	while(tx=va_arg(ap,char *)) {
		JXAddMenuItem(display,gadg,tx);
		if (tnum++ == def)
			JXSetOption(display,gadg,CurrentString,tx,0);
		}
	map = False;
	}
	break;

case twoword:
case twowordwin: {
	int x,y,w,h;
	char *pr1, *pr2;

	/* extract va_args */
	x = va_arg(ap,int);
	y = va_arg(ap,int);
	w = va_arg(ap,int);
	h = va_arg(ap,int);
	pr1 = va_arg(ap,char *);
	pr2 = va_arg(ap,char *);

	/* store into gadget */
	gadg->x = x;
	gadg->y = y;
	gadg->w = w;
	gadg->h = h;

	/* map main window */
	if (type == twowordwin) {
		XWindowAttributes winattr;

		gadg->window = XCreateSimpleWindow(display,root,x,y,
						   w,h,0,0,col3);
		XSetTransientForHint(display,gadg->window,win);
		gadg->type = twoword;

		XGetWindowAttributes(display,win,&winattr);
		XSetWindowColormap(display,gadg->window,winattr.colormap);
		}
	else {
		gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
		}
	XSelectInput(display,gadg->window,Mtwoword);
	XMapWindow(display,gadg->window);
	XFlush(display);
	/* XIfEvent(display,&event,exposepred,""); */
	XWindowEvent(display,gadg->window,StructureNotifyMask,&event);

	/* create sub-gadgets */
	gadg->jxgtwoword.pr1  = JXNewGadget(display,gadg->window,col1,col2,
				col3,stext,5,5,w-10,15,a_left,pr1);
	gadg->jxgtwoword.str1 = JXNewGadget(display,gadg->window,col1,col2,
				col3,string,5,25,w-10,23,"",256);
	gadg->jxgtwoword.pr2  = JXNewGadget(display,gadg->window,col1,col2,
				col3,stext,5,50,w-10,15,a_left,pr2);
	gadg->jxgtwoword.str2 = JXNewGadget(display,gadg->window,col1,col2,
				col3,string,5,70,w-10,23,"",256);
	gadg->jxgtwoword.ok   = JXNewGadget(display,gadg->window,col1,col2,
				col3,boolean,5,h-25,w/2-10,20,"Ok");
	gadg->jxgtwoword.can  = JXNewGadget(display,gadg->window,col1,col2,
				col3,boolean,w/2+5,h-25,w/2-10,20,"Cancel");

	map = False;
	}
	break;

case window: {
	int x,y,w,h;
	char *title;
	Bool popup;
	Bool bevel;
	XWindowAttributes winattr;
	XSizeHints shints;

	/* read va_args */
	x = va_arg(ap,int);
	y = va_arg(ap,int);
	w = va_arg(ap,int);
	h = va_arg(ap,int);
	title = va_arg(ap,char *);
	popup = va_arg(ap,Bool);
	bevel = va_arg(ap,Bool);

	/* make sure window isn't off-screen */
	XGetWindowAttributes(display,root,&winattr);
	if (x+w > winattr.width-2)  x = (winattr.width-2)-w;
	if (y+h > winattr.height-2) y = (winattr.height-2)-h;

	/* store into gadget */
	gadg->x = x;
	gadg->y = y;
	gadg->w = w;
	gadg->h = h;
	gadg->label = (char *)malloc(strlen(title)+1);
	strcpy(gadg->label,title);
	gadg->jxgwindow.popup = popup;
	gadg->jxgwindow.bevel = bevel;
	gadg->jxgwindow.toplevel = 0;
	gadg->jxgwindow.gcount = 0;
	gadg->jxgwindow.gl = (JXgadget **)calloc(MAX_WINDOW_GADGETS,
						 sizeof(JXgadget *));

	/* map window */
	gadg->window = XCreateSimpleWindow(display,root,x,y,w,h,0,0,col3);
	if (popup) {
		XSetTransientForHint(display,gadg->window,win);
		}
	shints.min_width  = shints.max_width  = gadg->w;
	shints.min_height = shints.max_height = gadg->h;
	shints.flags = PMinSize | PMaxSize;
	XSetStandardProperties(display,gadg->window,title,title,None,
			       NULL,0,&shints);
	XGetWindowAttributes(display,win,&winattr);
	XSetWindowColormap(display,gadg->window,winattr.colormap);
	XSelectInput(display,gadg->window,Mtwoword);

	XMapWindow(display,gadg->window);
	XFlush(display);
	/* XIfEvent(display,&event,exposepred,""); */
	XWindowEvent(display,gadg->window,StructureNotifyMask,&event);

	map = False;
	}
	break;

case bevelbox: {
	int x,y,w,h;
	Bool indent;

	/* Parse varargs
	 */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	indent=va_arg(ap,Bool);

	/* store into gadget */
	gadg->x = x;
	gadg->y = y;
	gadg->w = w;
	gadg->h = h;
	gadg->jxgbevelbox.indent = indent;

	/* create gadget window */
	map=True;
	gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSetWindowBackground(display,gadg->window,col3);
	XSelectInput(display,gadg->window,Mbevelbox);
	}
	break;

case pane: {
	int x,y,w,h;
	Pixmap pm;
	Window rt;		/* useless stuff returned by XGetGeometry */
	int wx,wy;
	unsigned int bw,dp;
	Font cfont;		/* for defining a cursor */
	XColor cfc,cbc;

	/* Parse varargs
	 */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	pm=va_arg(ap,Pixmap);

	/* store into gadget */
	gadg->x = x;
	gadg->y = y;
	gadg->w = w;
	gadg->h = h;
	gadg->jxgpane.report = True;
	gadg->jxgpane.pm = pm;
	gadg->jxgpane.x = gadg->jxgpane.y = 0;
	gadg->jxgpane.mx = gadg->jxgpane.my = -1;
	XGetGeometry(display,pm,&rt,&wx,&wy,(unsigned int *)&gadg->jxgpane.w,
		     (unsigned int *)&gadg->jxgpane.h,&bw,&dp);

	/* create gadget window */
	map=True;
	gadg->window=XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSetWindowBackground(display,gadg->window,col3);
	XSelectInput(display,gadg->window,Mpane);

	/* set an appropriate cursor for the pane */
	cfont = XLoadFont(display,"cursor");
	cfc.red = cfc.green = cfc.blue = 65535;
	cbc.red = cbc.green = cbc.blue = 0;
	gadg->jxgpane.cr = XCreateGlyphCursor(display,cfont,cfont,XC_hand2,
					      XC_hand2+1,&cfc,&cbc);
	XDefineCursor(display,gadg->window,gadg->jxgpane.cr);
	XUnloadFont(display,cfont);
	}
	break;

case scrollpane: {
	int x,y,w,h;
	Pixmap pm;
	double hfrac, vfrac;

	/* Parse varargs
	 */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	pm=va_arg(ap,Pixmap);

	/* store into gadget */
	gadg->x = x;
	gadg->y = y;
	gadg->w = w;
	gadg->h = h;
	gadg->jxgscrollpane.report = True;

	/* map main window */
	gadg->window = XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSelectInput(display,gadg->window,Mscrollpane);
	XMapWindow(display,gadg->window);
	XFlush(display);
	XWindowEvent(display,gadg->window,StructureNotifyMask,&event);

	/* create sub-gadgets */
	gadg->jxgscrollpane.pn = JXNewGadget(display,gadg->window,col1,col2,
				  col3,pane,2,2,w-24,h-24,pm);
	hfrac =	(double)gadg->jxgscrollpane.pn->w /
		(double)gadg->jxgscrollpane.pn->jxgpane.w;
	vfrac =	(double)gadg->jxgscrollpane.pn->h /
		(double)gadg->jxgscrollpane.pn->jxgpane.h;
	gadg->jxgscrollpane.hz = JXNewGadget(display,gadg->window,col1,col2,
				  col3,prscroller,2,h-22,w-24,20,0.0,hfrac);
	gadg->jxgscrollpane.vt = JXNewGadget(display,gadg->window,col1,col2,
				  col3,prscroller,w-22,2,20,h-24,0.0,vfrac);

	map = False;
	}
	break;

case message: {
	int x,y,linec, maxw,i;
	char **line;
	XWindowAttributes winattr;
	int lineht = (font_info->ascent + font_info->descent + 3);

	/* Parse varargs
	 */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	line = va_arg(ap,char **);
	linec = va_arg(ap,int);
	gadg->x = x;
	gadg->y = y;
	gadg->jxgmessage.lnc = linec;

	/* work out gadget size
	 */
	maxw=0;
	for(i=0; i<linec; i++) {
		int lw = XTextWidth(font_info,line[i],strlen(line[i]));
		if (lw > maxw) maxw = lw;
		}
	gadg->h = linec*lineht + 35;
	gadg->w = maxw + 10;

	/* make sure window isn't off-screen */
	XGetWindowAttributes(display,root,&winattr);
	if (gadg->x+gadg->w > winattr.width-2)
		gadg->x = (winattr.width-2) - gadg->w;
	if (gadg->y+gadg->h > winattr.height-2)
		gadg->y = (winattr.height-2) - gadg->h;

	/* map main window */
	gadg->window = XCreateSimpleWindow(display,root,x,y,gadg->w,gadg->h,
					   0,0,col3);
	XSetTransientForHint(display,gadg->window,win);
	XGetWindowAttributes(display,win,&winattr);
	XSetWindowColormap(display,gadg->window,winattr.colormap);
	XSelectInput(display,gadg->window,Mmessage);
	XMapWindow(display,gadg->window);
	XFlush(display);
	XWindowEvent(display,gadg->window,StructureNotifyMask,&event);

	/* create sub-gadgets */
	gadg->jxgmessage.ok = JXNewGadget(display,gadg->window,col1,col2,
			       col3,boolean,5,gadg->h-25,gadg->w/2,20,"Ok");
	gadg->jxgmessage.ln = calloc(linec,sizeof(JXgadget *));
	for(i=0; i<linec; i++) {
		gadg->jxgmessage.ln[i] = JXNewGadget(display,gadg->window,col1,
					  col2,col3,stext,5,i*lineht+5,maxw,
					  lineht,a_left,line[i]);
		}
	map = False;
	}
	break;

case statusgauge: {
	int x,y,w,h;
	double prop;

	/* Parse varargs
	 */
	x=va_arg(ap,int);
	y=va_arg(ap,int);
	w=va_arg(ap,int);
	h=va_arg(ap,int);
	prop=va_arg(ap,double);

	/* store into gadget */
	gadg->x = x;
	gadg->y = y;
	gadg->w = w;
	gadg->h = h;
	gadg->jxgstatusgauge.p = prop;
	gadg->jxgstatusgauge.totalredraw = 1;

	/* create gadget window */
	map = True;
	gadg->window = XCreateSimpleWindow(display,win,x,y,w,h,0,0,col3);
	XSetWindowBackground(display,gadg->window,col3);
	XSelectInput(display,gadg->window,Mstatusgauge);
	}
	break;
	
default:
	break;
	}

/* map and re-draw the new gadget, if necessary */
if (map) {
	XMapWindow(display,gadg->window);
        XFlush(display);
        /* XIfEvent(display,&event,exposepred,""); */
	XWindowEvent(display,gadg->window,StructureNotifyMask,&event);
        }
JXRedrawGadget(display,gadg);

return(gadg);
}






