#include "jx.h"

/* JXBorderWindow - draw a border around the window of a gadget, after
 * the pointer has been moved over the gadget.
 *
 * display		- connection to the X server
 * gadg			- gadget to draw border around
 *
 * note - this function is for internal use only
 */
static void JXBorderWindow(Display *display, JXgadget *gadg)
{
XSetForeground(display,gc,gadg->hicolour);
if (gadg->dbuf) {
	/* If this gadget has a double-buffer pixmap, draw into
	 * it instead.
	 */
	XDrawRectangle(display,gadg->dbuf,gc,0,0,gadg->w-1,gadg->h-1);
	}
else 
	XDrawRectangle(display,gadg->window,gc,0,0,gadg->w-1,gadg->h-1);
}


/* JXRedrawGadget - repaints the specified gadget. Usually used after
 * some attribute of the gadget has been changed, or the gadget has
 * been Exposed.
 *
 * display		- connection to the X server
 * gadg			- gadget to redraw
 */
void JXRedrawGadget(Display *display, JXgadget *gadg)
{
short w=gadg->w, h=gadg->h;
int tx,ty,txw;
int x,y,s;
int stln;
int inx,iny;
int sx,sy;
double npos;
double maxlns;
char perbuf[5];

if (!gadg->active) {
	/* Draw gadget in 'inactive' style
	 */
	XSetFillStyle(display,gc,FillStippled);
	}

switch(gadg->type) {
case boolean :
	/* draw the bevelled gadget, in a style depending on
	 * it's state.
	 */
        if (gadg->jxgboolean.state) {
		JXBevelBox(display,gadg->window,0,0,w,h,2,gadg->
		 botcolour,gadg->backcolour,gadg->topcolour);
                XSetForeground(display,gc,gadg->botcolour);
		}
        else {
		JXBevelBox(display,gadg->window,0,0,w,h,2,gadg->
		 topcolour,gadg->backcolour,gadg->botcolour);
                XSetForeground(display,gc,gadg->topcolour);
		}
	/* draw the label.
	 */
        tx=(w/2)-(XTextWidth(font_info,gadg->label,strlen(gadg->label))/2);
        ty=(h/2)+5;
        XDrawString(display,gadg->window,gc,tx,ty,gadg->label,
         strlen(gadg->label));
	/* outline the gadget if the pointer is in it.
	 */
	if (gadg->in) {
		JXBorderWindow(display,gadg);
		}
        break;
case toggle :
        if (gadg->jxgtoggle.state) {
		JXBevelBox(display,gadg->window,0,0,w,h,2,gadg->
		 botcolour,gadg->backcolour,gadg->topcolour);
                XSetForeground(display,gc,gadg->botcolour);
		}
        else {
		JXBevelBox(display,gadg->window,0,0,w,h,2,gadg->
		 topcolour,gadg->backcolour,gadg->botcolour);
                XSetForeground(display,gc,gadg->topcolour);
		}
        tx=(w/2)-(XTextWidth(font_info,gadg->label,
         strlen(gadg->label))/2);
        ty=(h/2)+5;
        XDrawString(display,gadg->window,gc,tx,ty,gadg->label,
         strlen(gadg->label));
	/* outline the gadget if the pointer is in it.
	 */
	if (gadg->in) {
		JXBorderWindow(display,gadg);
		}
        break;
case string :
	JXBevelBox(display,gadg->window,0,0,w,h,2,gadg->botcolour,
		   gadg->backcolour,gadg->topcolour);
	/* If the gadget has grabbed the keyboard, fill the string sub-
	 * window with the highlight colour.
	 */
	XSetWindowBackground(display,gadg->jxgstring.twin,gadg->jxgstring.grab
			     ? gadg->hicolour : gadg->backcolour);
	XClearWindow(display,gadg->jxgstring.twin);

	/* Draw the gadget string with the proper justification.
	 */
	txw = XTextWidth(font_info,gadg->label,gadg->jxgstring.len);
	if (gadg->jxgstring.aln == a_left)
		tx = 1;
	else if (gadg->jxgstring.aln == a_right)
		tx = gadg->w - 4 - txw;
	else
		tx = (gadg->w - 4 - txw)/2;
	sy = gadg->h - 4 - font_info->descent;
        XSetForeground(display,gc,gadg->topcolour);
	XDrawString(display,gadg->jxgstring.twin,gc,tx,sy,gadg->label,
		    gadg->jxgstring.len);

	if (gadg->jxgstring.editable) {
		/* Draw text cursor
		 */
		txw = XTextWidth(font_info,gadg->label,gadg->jxgstring.cur);
		XSetForeground(display,gc,gadg->botcolour);
		XDrawLine(display,gadg->jxgstring.twin,gc,tx+txw,0,tx+txw,
			  gadg->h-4);
		}

	/* outline the gadget if the pointer is over it.
	 */
	if (gadg->in) {
		JXBorderWindow(display,gadg);
		}
        break;
case slider :
	/* draw into background pixmap.
	 */
	JXBevelBox(display,gadg->dbuf,0,0,w,h,2,gadg->botcolour,
	 gadg->backcolour,gadg->topcolour);
	if (gadg->jxgslider.horiz) {
		x=gadg->jxgslider.pos+2;
		y=2;
		}
	else
		{
		x=2;
		y=gadg->jxgslider.pos+2;
		}
	s=gadg->jxgslider.slsize;
	JXBevelBox(display,gadg->dbuf,x,y,x+s-1,y+s-1,2,gadg->
	 topcolour,gadg->backcolour,gadg->botcolour);
	/* outline the gadget if the pointer is in it.
	 */
	if (gadg->in) {
		JXBorderWindow(display,gadg);
		}
	/* Blit background pixmap to front.
	 */
	XCopyArea(display,gadg->dbuf,gadg->window,gc,0,0,gadg->w,gadg->h,0,0);
	break;
case slist :
	/* Calculate the first line to be shown in the list box. If all
	 * the lines can be shown, start from the first.
	 */
	if (gadg->jxgslist.slg->jxgprscroller.slg->jxgprslider.frac >= 1) {
		npos = 1.0;
		stln = 0;
		}
	else {
		npos = gadg->jxgslist.slg->jxgprscroller.slg->jxgprslider.npos;

		/* npos == 1  => draw lines from the bottom of the list
		 * npos == 0  => draw lines from the top of the list
		 */
		maxlns = (gadg->h-6)/(font_info->ascent+font_info->descent);
		stln = (int)((1-npos) * (gadg->jxgslist.lines - maxlns));
		}

	/* only re-draw if actual list position has changed */
	if (stln != gadg->jxgslist.cline) {
		JXBevelBox(display,gadg->window,0,0,w-17,h,2,gadg->botcolour,
		 gadg->backcolour,gadg->topcolour);

		/* only draw text if any to draw */
		if (gadg->jxgslist.lines) {
			int i;
			JXtext *pos;
			int mxlns;			/* maximum lines */
			int y;

			gadg->jxgslist.cline=stln;
			/* stln=0 => at bottom of list */
			/* stln=lines => only last line displayed */
			pos=gadg->jxgslist.buf;

			/* step along list to first line to draw */
			for(i=0;i < stln;i++)
				pos=pos->next;

			/* draw lines from the bottom of the list,
			 * stepping upwards with each line.
			 */
			mxlns=(gadg->h-6)/(font_info->ascent+font_info
			 ->descent);
			y=gadg->h-3-font_info->descent;
			for(i=0;i < mxlns && pos;i++) {
				char *expn = JXExpandTabs(pos->str);

				XDrawString(display,gadg->window,gc,3,y,
				 expn,strlen(expn));
				pos=pos->next;
				y=y-(font_info->ascent+font_info->descent);
				}
			}
		}
	/* redraw sub-parts
	 */
	JXRedrawGadget(display,gadg->jxgslist.slg);
	break;

case pboolean : 
	XSetBackground(display,gc,gadg->backcolour);
	if (gadg->jxgpboolean.state) {
                JXBevelBox(display,gadg->window,0,0,w,h,2,gadg->
                 botcolour,gadg->backcolour,gadg->topcolour);
                XSetForeground(display,gc,gadg->botcolour);
                }
        else {
                JXBevelBox(display,gadg->window,0,0,w,h,2,gadg->
                 topcolour,gadg->backcolour,gadg->botcolour);
                XSetForeground(display,gc,gadg->topcolour);
                }
	inx=(gadg->w-gadg->jxgpboolean.pic_w)/2;
	iny=(gadg->h-gadg->jxgpboolean.pic_h)/2;
	XCopyPlane(display,gadg->jxgpboolean.pic,gadg->window,gc,0,0,
	 gadg->jxgpboolean.pic_w,gadg->jxgpboolean.pic_h,inx,iny,1);
	/* outline the gadget if the pointer is in it.
	 */
	if (gadg->in) {
		JXBorderWindow(display,gadg);
		}
        break;

case ptoggle :
        XSetBackground(display,gc,gadg->backcolour);
        if (gadg->jxgptoggle.state) {
                JXBevelBox(display,gadg->window,0,0,w,h,2,gadg->
                 botcolour,gadg->backcolour,gadg->topcolour);
                XSetForeground(display,gc,gadg->botcolour);
                }
        else {
                JXBevelBox(display,gadg->window,0,0,w,h,2,gadg->
                 topcolour,gadg->backcolour,gadg->botcolour);
                XSetForeground(display,gc,gadg->topcolour);
                }
        inx=(gadg->w-gadg->jxgptoggle.pic_w)/2;
        iny=(gadg->h-gadg->jxgptoggle.pic_h)/2;
        XCopyPlane(display,gadg->jxgptoggle.pic,gadg->window,gc,0,0,
         gadg->jxgptoggle.pic_w,gadg->jxgptoggle.pic_h,inx,iny,1);
	/* outline the gadget if the pointer is in it.
	 */
	if (gadg->in) {
		JXBorderWindow(display,gadg);
		}
        break;

case scroller :
	JXRedrawGadget(display,gadg->jxgscroller.dir1);
	JXRedrawGadget(display,gadg->jxgscroller.dir2);
	JXRedrawGadget(display,gadg->jxgscroller.slg);
	break;

case menu:
	/* re-draw the button at the top of the menu.
	 */
        if (gadg->jxgmenu.open) {
		JXBevelBox(display,gadg->window,0,0,w,h,2,gadg->
		 botcolour,gadg->backcolour,gadg->topcolour);
                XSetForeground(display,gc,gadg->botcolour);
		}
        else {
		JXBevelBox(display,gadg->window,0,0,w,h,2,gadg->
		 topcolour,gadg->backcolour,gadg->botcolour);
                XSetForeground(display,gc,gadg->topcolour);
		}
	/* draw the label.
	 */
        tx=(w/2)-(XTextWidth(font_info,gadg->label,strlen(gadg->label))/2);
        ty=(h/2)+5;
        XDrawString(display,gadg->window,gc,tx,ty,gadg->label,
         strlen(gadg->label));
	/* outline the gadget if the pointer is in it.
	 */
	if (gadg->in) {
		JXBorderWindow(display,gadg);
		}
        break;

case stext:
	XClearWindow(display,gadg->window);
	XSetForeground(display,gc,gadg->topcolour);
	if (gadg->jxgstext.align == a_left) {
		XDrawString(display,gadg->window,gc,0,gadg->h - 
		 font_info->descent,gadg->label,strlen(gadg->label));
		}
	else if (gadg->jxgstext.align == a_right) {
		int tw;

		tw = XTextWidth(font_info,gadg->label,strlen(gadg->label));
		XDrawString(display,gadg->window,gc,gadg->w - tw,
		 gadg->h - font_info->descent,gadg->label,strlen(gadg->label));
		}
	else if (gadg->jxgstext.align == a_centre) {
		int tw;

		tw = XTextWidth(font_info,gadg->label,strlen(gadg->label));
		XDrawString(display,gadg->window,gc,(gadg->w - tw)/2,
		 gadg->h - font_info->descent,gadg->label,strlen(gadg->label));
		}
	break;

case prslider :
	JXBevelBox(display,gadg->dbuf,0,0,w,h,2,gadg->botcolour,
	 gadg->backcolour,gadg->topcolour);
	if (gadg->jxgprslider.horiz) {
		x = gadg->jxgprslider.pos+2;
		y = 2;
		sx = gadg->jxgprslider.lsize;
		sy = gadg->jxgprslider.wsize;
		}
	else
		{
		x = 2;
		y = gadg->jxgprslider.pos+2;
		sx = gadg->jxgprslider.wsize;
		sy = gadg->jxgprslider.lsize;
		}
	JXBevelBox(display,gadg->dbuf,x,y,x+sx-1,y+sy-1,2,gadg->
	 topcolour,gadg->backcolour,gadg->botcolour);
	/* outline the gadget if the pointer is in it.
	 */
	if (gadg->in) {
		JXBorderWindow(display,gadg);
		}
	/* Blit background pixmap to front.
	 */
	XCopyArea(display,gadg->dbuf,gadg->window,gc,0,0,gadg->w,gadg->h,0,0);
	break;

case prscroller:
	JXRedrawGadget(display,gadg->jxgprscroller.dir1);
	JXRedrawGadget(display,gadg->jxgprscroller.dir2);
	JXRedrawGadget(display,gadg->jxgprscroller.slg);
	break;

case file:
	JXBevelBox(display,gadg->window,0,0,gadg->w,gadg->h,2,gadg->
	 topcolour,gadg->backcolour,gadg->botcolour);
	break;

case choice:
	JXBevelBox(display,gadg->window,0,0,gadg->w,gadg->h,2,gadg->
	 topcolour,gadg->backcolour,gadg->botcolour);
	break;

case combo:
	break;

case twoword:
	JXBevelBox(display,gadg->window,0,0,gadg->w,gadg->h,2,gadg->topcolour,
		   gadg->backcolour,gadg->botcolour);
	break;

case window:
	if (gadg->jxgwindow.bevel) {
		JXBevelBox(display,gadg->window,0,0,gadg->w,gadg->h,2,gadg->
		 topcolour,gadg->backcolour,gadg->botcolour);
		}
	break;

case bevelbox:
	if (gadg->jxgbevelbox.indent) {
		JXBevelBox(display,gadg->window,0,0,gadg->w,gadg->h,1,gadg->
		 botcolour,gadg->backcolour,gadg->topcolour);
		}
	else {
		JXBevelBox(display,gadg->window,0,0,gadg->w,gadg->h,1,gadg->
		 topcolour,gadg->backcolour,gadg->botcolour);
		}
	break;

case pane:
	XCopyArea(display,gadg->jxgpane.pm,gadg->window,gc,
		  gadg->jxgpane.x,gadg->jxgpane.y,
		  gadg->jxgpane.w,gadg->jxgpane.h,
		  0,0);
	break;

case scrollpane:
	JXBevelBox(display,gadg->window,0,0,gadg->w,gadg->h,1,
		   gadg->topcolour,gadg->backcolour,gadg->botcolour);
	JXBevelBox(display,gadg->window,gadg->w-22,gadg->h-22,gadg->w-2,
		   gadg->h-2,2,gadg->topcolour,gadg->backcolour,
		   gadg->botcolour);
	break;

case message:
	JXBevelBox(display,gadg->window,0,0,gadg->w,gadg->h,1,
		   gadg->topcolour,gadg->backcolour,gadg->botcolour);
	break;

case statusgauge:
	if (gadg->jxgstatusgauge.totalredraw) {
		JXBevelBox(display,gadg->window,0,0,gadg->w,gadg->h,2,
			   gadg->botcolour,gadg->backcolour,gadg->topcolour);
		gadg->jxgstatusgauge.totalredraw = 0;
		}
	else {
		XSetForeground(display,gc,gadg->backcolour);
		XFillRectangle(display,gadg->window,gc,2,2,gadg->w-4,gadg->h-4);
		}
	/* Write status bar
	 */
	XSetForeground(display,gc,gadg->hicolour);
	npos = gadg->jxgstatusgauge.p;
	npos = (npos<0) ? 0 : (npos>1) ? 1 : npos;
	XFillRectangle(display,gadg->window,gc,2,2,(gadg->w-4)*npos,gadg->h-4);
	XSetForeground(display,gc,gadg->topcolour);
	/* Write percentage
	 */
	sprintf(perbuf,"%d%%",(int)(npos*100.0));
	tx = (gadg->w - XTextWidth(font_info,perbuf,strlen(perbuf)))/2;
	XDrawString(display,gadg->window,gc,tx,gadg->h-font_info->descent-4,
		    perbuf,strlen(perbuf));
	break;
	}

XSetFillStyle(display,gc,FillSolid);
XFlush(display);
}

