#include "jx.h"

static void JXChildSetOption(Display *display, JXgadget *gadg, ...);
static void JXRealSetOption(Display *display, JXgadget *gadg, va_list ap);

/* Does the real work of setting options, using a va_list instead of
 * variable arguments.
 */
static void JXRealSetOption(Display *display, JXgadget *gadg, va_list ap)
{
int attrib;

while(attrib = va_arg(ap,int)) {
	switch(attrib) {
	case HighlightCol: {
		int c;

		c = va_arg(ap,int);
		gadg->hicolour = c;
		JXChildSetOption(display,gadg,HighlightCol,c,0);
		JXRedrawGadget(display,gadg);
		break;
		}
	case ReportMove: {
		Bool rm;

		rm = va_arg(ap,Bool);
		if (gadg->type == slider)
			gadg->jxgslider.report = rm;
		else if (gadg->type == prslider)
			gadg->jxgprslider.report = rm;
		else if (gadg->type == scroller)
			gadg->jxgscroller.slg->jxgslider.report = rm;
		else if (gadg->type == prscroller)
			gadg->jxgprscroller.slg->jxgprslider.report = rm;
		else if (gadg->type == pane)
			gadg->jxgpane.report = rm;
		else if (gadg->type == scrollpane)
			gadg->jxgscrollpane.report = rm;
		else
			printf("JX : attempted to set ReportMove on an "
			 "innapropriate gadget\n");
		break;
		}
	case TopColour: {
		int c;

		c = va_arg(ap,int);
		gadg->topcolour = c;
		JXChildSetOption(display,gadg,TopColour,c,0);
		JXRedrawGadget(display,gadg);
		break;
		}
	case BottomColour: {
		int c;

		c = va_arg(ap,int);
		gadg->botcolour = c;
		JXChildSetOption(display,gadg,BottomColour,c,0);
		JXRedrawGadget(display,gadg);
		break;
		}
	case BackColour: {
		int c;

		c = va_arg(ap,int);
		gadg->backcolour = c;
		JXChildSetOption(display,gadg,BackColour,c,0);
		JXRedrawGadget(display,gadg);
		break;
		}
	case ScrollStep: {
		double s;

		s = va_arg(ap,double);
		if (gadg->type == scroller)
			gadg->jxgscroller.step = s;
		else if (gadg->type == prscroller)
			gadg->jxgprscroller.step = s;
		else
			printf("JX : attempted to set ScrollStep on an "
			 "inappropriate gadget\n");
		JXRedrawGadget(display,gadg);
		break;
		}
	case PropFrac: {
		double f;

		f = va_arg(ap,double);
		if (gadg->type == prslider) {
			int xy;

			if (gadg->jxgprslider.horiz)
				gadg->jxgprslider.lsize =
				 (int)(f < 1 ? (gadg->w-4)*f : gadg->w-4);
			else
				gadg->jxgprslider.lsize =
				 (int)(f < 1 ? (gadg->h-4)*f : gadg->h-4);
			gadg->jxgprslider.frac = f;
			/* Check if the change in the slider means that
			 * it has to be moved a bit.
			 */
			if (gadg->jxgprslider.horiz)
				xy = gadg->w;
			else
				xy = gadg->h;
			if (gadg->jxgprslider.pos < 0)
				gadg->jxgprslider.pos=0;
			else if (gadg->jxgprslider.pos > xy-
			 gadg->jxgprslider.lsize-4)
				gadg->jxgprslider.pos=xy-
				 gadg->jxgprslider.lsize-4;
			JXRedrawGadget(display,gadg);
			}
		else if (gadg->type == prscroller)
			JXSetOption(display,gadg->jxgprscroller.slg,
			 PropFrac,f,0);
		else if (gadg->type == statusgauge) {
			gadg->jxgstatusgauge.p = f;
			JXRedrawGadget(display,gadg);
			}
		else
			printf("JX : attempted to set PropFrac on an "
			 "inappropriate gadget\n");
		break;
		}
	case SliderNPos: {
		double f;

		f = va_arg(ap,double);
		if (gadg->type == slider) {
			gadg->jxgslider.npos = f;
			if (gadg->jxgslider.horiz) {
				gadg->jxgslider.pos = (int)(f * (gadg->w -
				 gadg->jxgslider.slsize - 4));
				}
			else {
				gadg->jxgslider.pos = (int)(f * (gadg->h -
				 gadg->jxgslider.slsize - 4));
				}
			}
		else if (gadg->type == prslider) {
			gadg->jxgprslider.npos = f;
			if (gadg->jxgprslider.horiz) {
				gadg->jxgprslider.pos = (int)(f * (gadg->w -
				 gadg->jxgprslider.lsize - 4));
				}
			else {
				gadg->jxgprslider.pos = (int)(f * (gadg->h -
				 gadg->jxgprslider.lsize - 4));
				}
			}
		else if (gadg->type == scroller)
			JXSetOption(display,gadg->jxgscroller.slg,
			 SliderNPos,f,0);
		else if (gadg->type == prscroller)
			JXSetOption(display,gadg->jxgprscroller.slg,
			 SliderNPos,f,0);
		else
			printf("JX : attempted to set SliderNPos on a "
			 "non-slider gadget\n");
		break;
		}
	case StringAlign: {
		JXalign al;

		al = va_arg(ap,JXalign);
		if (gadg->type == string)
			gadg->jxgstring.aln = al;
		else
			printf("JX : attempted to set StringAlign on a "
			 "non-string gadget\n");
		JXRedrawGadget(display,gadg);
		break;
		}
	case CurrentString: {
		char *s;

		s = va_arg(ap,char *);
		if (gadg->type == string) {
			strcpy(gadg->label,s);
			gadg->jxgstring.len = strlen(s);
			gadg->jxgstring.cur = strlen(s);
			}
		else if (gadg->type == combo)
			JXSetOption(display,gadg->jxgcombo.str,
				    CurrentString,s,0);
		else if (gadg->type == twoword)
			JXSetOption(display,gadg->jxgtwoword.str1,
				    CurrentString,s,0);
		else if (gadg->type == stext) {
			strcpy(gadg->label,s);
			}
		else
			printf("JX : attempted to set CurrentString on a "
			       "gadget with no string\n");
		JXRedrawGadget(display,gadg);
		break;
		}
	case CurrentString2: {
		char *s;

		s = va_arg(ap,char *);
		if (gadg->type == twoword)
			JXSetOption(display,gadg->jxgtwoword.str2,
				    CurrentString,s,0);
		else
			printf("JX : attempted to set CurrentString2 on a "
			       "gadget without 2 strings\n");
		JXRedrawGadget(display,gadg);
		break;
		}
	case CheckItem: {
		int item;
		int m;

		item = va_arg(ap,int);
		m = va_arg(ap,int);
		if (gadg->type == menu) {
			JXtext *tx;
			int i;

			for(i=0,tx=gadg->jxgmenu.buf; tx; tx=tx->next,i++)
				if (i == item)
					tx->mark = m;
			}
		else
			printf("JX : attempted to check a menu item of a "
			       "non-menu gadget\n");
		break;
		}
	case ButtonLabel: {
		char *str = va_arg(ap,char *);
		if (gadg->type == boolean) {
			strcpy(gadg->label,str);
			}
		else if (gadg->type == toggle) {
			strcpy(gadg->label,str);
			}
		else {
			printf("JX : attempted to set ButtonLabel of a "
			       "non-button gadget\n");
			}
		JXRedrawGadget(display,gadg);
		break;
		}
	case SmoothMove: {
		Bool sm;

		sm = va_arg(ap,Bool);
		if (gadg->type == slider)
			gadg->jxgslider.smooth = sm;
		else if (gadg->type == prslider)
			gadg->jxgprslider.smooth = sm;
		else if (gadg->type == scroller)
			gadg->jxgscroller.slg->jxgslider.smooth = sm;
		else if (gadg->type == prscroller)
			gadg->jxgprscroller.slg->jxgprslider.smooth = sm;
		else
			printf("JX : attempted to set SmoothMove on an "
			 "innapropriate gadget\n");
		break;
		}
	case Active: {
		Bool ac;

		ac = va_arg(ap,Bool);
		gadg->active = ac;
		gadg->in     = False;
		JXClearGadget(display,gadg);
		if (gadg->type == slist)
			gadg->jxgslist.cline = -1;	/* for redraw */
		JXRedrawGadget(display,gadg);
		JXChildSetOption(display,gadg,Active,ac,0);
		break;
		}
	case WrapSlider: {
		Bool wr;

		wr = va_arg(ap,Bool);
		if (gadg->type == slider)
			gadg->jxgslider.wrap = wr;
		else if (gadg->type == prslider)
			gadg->jxgprslider.wrap = wr;
		else if (gadg->type == scroller)
			JXSetOption(display,gadg->jxgscroller.slg,
				    WrapSlider,wr,0);
		else if (gadg->type == prscroller)
			JXSetOption(display,gadg->jxgprscroller.slg,
				    WrapSlider,wr,0);
		else
			printf("JX : attempted to set WrapSlider on a non-"
			 "slider gadget\n");
		break;
		}
	case ListPos: {
		double lp;

		lp = va_arg(ap,double);
		if (gadg->type == slist) {
			JXSetOption(display,gadg->jxgslist.slg,SliderNPos,lp,0);
			JXRedrawGadget(display,gadg);
			}
		else
			printf("JX : attempted to set ListPos on a non-list\n");
		break;
		}
	case CursorColour: {
		XColor cfc;

		cfc = va_arg(ap,XColor);
		if (gadg->type == pane) {
			XColor cbc;

			cbc.red = cbc.green = cbc.blue = 0;
			XRecolorCursor(display,gadg->jxgpane.cr,&cfc,&cbc);
			}
		else if (gadg->type == scrollpane) {
			JXSetOption(display,gadg->jxgscrollpane.pn,
				    CursorColour,cfc,0);
			}
		else
			printf("JX : attempted to set CursorColour on a gadget "
			       "that has no cursor\n");
		break;
		}
	case WrapList: {
		Bool wr;

		wr = va_arg(ap,Bool);
		if (gadg->type == slist) 
			gadg->jxgslist.wrap = wr;
		else if (gadg->type == file)
			JXSetOption(display,gadg->jxgfile.ls,WrapList,wr,0);
		else
			printf("JX : attempted to set WrapList on a non-list "
			       "gadget\n");
		break;
		}
	case StringEditable: {
		Bool edt;

		edt = va_arg(ap,Bool);
		if (gadg->type == string)
			gadg->jxgstring.editable = edt;
		else
			printf("JX : attempted to set StringEditable on a "
			       "non-string gadget\n");
		break;
		}
	case LimitToList: {
		Bool lim;

		lim = va_arg(ap,Bool);
		if (gadg->type == combo)
			JXSetOption(display,gadg->jxgcombo.str,StringEditable,
				    !lim,0);
		else
			printf("JX : attempted to set LimitToList on a non-"
			       "combo box gadget\n");
		break;
		}
	case TopLevel: {
		Bool tl;

		tl = va_arg(ap,Bool);
		if (gadg->type == window)
			gadg->jxgwindow.toplevel = tl;
		else
			printf("JX : attempted to set TopLevel on a non-window "
			       "gadget\n");
		break;
		}

		} /* end switch */
	}
}

/* Applys a variable argument list of options to all the children
 * of a gadget.
 */
static void JXChildSetOption(Display *display, JXgadget *gadg, ...)
{
va_list ap;
int i;

va_start(ap,gadg);
switch(gadg->type) {
case boolean:
case toggle:
case string:
case slider:
case pboolean:
case ptoggle:
case prslider:
case menu:
	/* All these gadget types have no children, so do nothing.
	 */
	break;
case slist:
	JXRealSetOption(display,gadg->jxgslist.slg,ap);
	break;
case scroller:
	JXRealSetOption(display,gadg->jxgscroller.dir1,ap);
	JXRealSetOption(display,gadg->jxgscroller.dir2,ap);
	JXRealSetOption(display,gadg->jxgscroller.slg,ap);
	break;
case prscroller:
	JXRealSetOption(display,gadg->jxgprscroller.dir1,ap);
	JXRealSetOption(display,gadg->jxgprscroller.dir2,ap);
	JXRealSetOption(display,gadg->jxgprscroller.slg,ap);
	break;
case file:
	JXRealSetOption(display,gadg->jxgfile.ls,ap);
	JXRealSetOption(display,gadg->jxgfile.fname,ap);
	JXRealSetOption(display,gadg->jxgfile.but,ap);
	JXRealSetOption(display,gadg->jxgfile.path,ap);
	JXRealSetOption(display,gadg->jxgfile.par,ap);
	JXRealSetOption(display,gadg->jxgfile.can,ap);
	break;
case choice:
	JXRealSetOption(display,gadg->jxgchoice.head,ap);
	for(i=0; i<gadg->jxgchoice.bcount; i++)
		JXRealSetOption(display,gadg->jxgchoice.buts[i],ap);
	break;
case combo:
	JXRealSetOption(display,gadg->jxgcombo.str,ap);
	JXRealSetOption(display,gadg->jxgcombo.but,ap);
	break;
case twoword:
	JXRealSetOption(display,gadg->jxgtwoword.str1,ap);
	JXRealSetOption(display,gadg->jxgtwoword.str2,ap);
	JXRealSetOption(display,gadg->jxgtwoword.pr1,ap);
	JXRealSetOption(display,gadg->jxgtwoword.pr2,ap);
	JXRealSetOption(display,gadg->jxgtwoword.ok,ap);
	JXRealSetOption(display,gadg->jxgtwoword.can,ap);
	break;
case window:
	for(i=0; i<gadg->jxgwindow.gcount; i++)
		JXRealSetOption(display,gadg->jxgwindow.gl[i],ap);
	break;
case scrollpane:
	JXRealSetOption(display,gadg->jxgscrollpane.pn,ap);
	JXRealSetOption(display,gadg->jxgscrollpane.hz,ap);
	JXRealSetOption(display,gadg->jxgscrollpane.vt,ap);
	break;
case message:
	for(i=0; i<gadg->jxgmessage.lnc; i++)
		JXRealSetOption(display,gadg->jxgmessage.ln[i],ap);
	JXRealSetOption(display,gadg->jxgmessage.ok,ap);
	break;
	}
va_end(ap);
}

/* JXSetOption - Allows the changing of various attributes of an existing
 * gadget, and setting of parameters too messy to pass in when creating
 * a the gadget.
 *
 * display		- connection to the X server
 * gadg			- gadget options apply to
 *
 * The rest of the parameters are a 0 terminated list of attrib / value
 * pairs -
 *
 * HighlightCol, int c	- borders gadget in colour c when pointer is over it.
 * ReportMove, Bool rm	- for slider/scroller gadgets, determines if this
 *			  gadget should report every movement of the slider.
 * TopColour, int c	- sets the "bright side" colour for this gadget.
 * BottomColour, int c	- sets the "dark side" colour for this gadget.
 * BackColour, int c	- sets the "background" colour for this gadget.
 * ScrollStep, double s	- sets the amount a scroller moves when one of
 *			  it's arrow gadgets is pressed.
 * PropFrac, double f	- sets the fraction of total shown by a proportional
 *			  slider / scroller.
 * SliderNPos, double n - sets the normalized position of a slider/scroller
 *			  to n.
 * StringAlign, JXalign a  - sets the alignment of a string gadget
 * CurrentString, char *s  - sets the string in a combo or string gadget
 * CurrentString2, char *s - sets the second string in a twoword gadget
 * SortedList, Bool s	- sets sorting on a list box
 */
void JXSetOption(Display *display, JXgadget *gadg, ...)
{
va_list ap;

va_start(ap,gadg);
JXRealSetOption(display,gadg,ap);
va_end(ap);
}


/* JXSetOptionAll
 * Set options on all existing gadgets
 */
void JXSetOptionAll(Display *display, ...)
{
va_list ap;
JXgadget *g;

for(g=glist; g; g=g->next) {
	va_start(ap,display);
	JXRealSetOption(display,g,ap);
	va_end(ap);
	}
}

