#define XK_MISCELLANY
#include "jx.h"

GC gc=0;
XFontStruct *font_info;
JXgadget *glist;
int gadget_count;

Bool exposepred(Display *display, XEvent *event, char *arg)
{
if (event->type == Expose)
	return(True);
else
	return(False);
}

/* JXCreateGadget - creates a new gadget, and draws it
 *
 * win		- parent window of gadget
 * x		- x position
 * y		- y position
 * width	- width of gadget
 * height	- height of gadget
 * type		- gadget type (boolean, toggle, etc...)
 * str		- label or initial string of gadget
 * display	- connection to the server
 * col1		- the highlight colour of the gadget
 * col2		- the body colour of the gadget
 * col3		- the background colour of the gadget
 */
JXgadget *JXCreateGadget(Window win, int x, int y, int width, int height,
 JXgtype type, char *str, Display *display, long col1, long col2,
 long col3)
{
JXgadget *gadg;
XEvent event;
XSetWindowAttributes winattr;

gadg=malloc(sizeof(JXgadget));
gadg->next=glist;
glist=gadg;
gadg->window=XCreateSimpleWindow(display,win,x,y,width-1,height-1,0,
 0,col3);
winattr.do_not_propagate_mask=-1;
XChangeWindowAttributes(display,gadg->window,CWDontPropagate,&winattr);
gadg->parent=win;
gadg->x=x;
gadg->y=y;
gadg->w=width-1;
gadg->h=height-1;
gadg->topcolour=col1;
gadg->botcolour=col2;
gadg->backcolour=col3;
gadg->type=type;
gadg->number=gadget_count++;
if (type==boolean) {
	XSelectInput(display,gadg->window,ExposureMask |
	 ButtonReleaseMask | LeaveWindowMask | ButtonPressMask);
	strcpy(gadg->label,str);
	gadg->jxgboolean.state=0;
	}
else if (type==toggle) {
	XSelectInput(display,gadg->window, ButtonPressMask |
	 ExposureMask);
	strcpy(gadg->label,str);
	gadg->jxgtoggle.state=0;
	}
else if (type==string) {
	XSelectInput(display,gadg->window,ExposureMask |
	 KeyPressMask);
	strcpy(gadg->label,str);
	gadg->jxgstring.len=strlen(str);
	}
XMapWindow(display,gadg->window);
XFlush(display);
XIfEvent(display,&event,exposepred,"");
JXRedrawGadget(display,gadg);
return(gadg);
}

void JXRedrawGadget(Display *display, JXgadget *gadg)
{
XPoint toplef[7];
XPoint botrig[7];
short w=gadg->w, h=gadg->h;
int tx,ty;

toplef[0].x=0; toplef[0].y=0;		/* define edges of gadget */
toplef[1].x=w; toplef[1].y=0;
toplef[2].x=w-3; toplef[2].y=3;
toplef[3].x=3; toplef[3].y=3;
toplef[4].x=3; toplef[4].y=h-3;
toplef[5].x=0; toplef[5].y=h;
toplef[6].x=0; toplef[6].y=0;

botrig[0].x=w; botrig[0].y=h;
botrig[1].x=w; botrig[1].y=0;
botrig[2].x=w-3; botrig[2].y=3;
botrig[3].x=w-3; botrig[3].y=h-3;
botrig[4].x=3; botrig[4].y=h-3;
botrig[5].x=0; botrig[5].y=h;
botrig[6].x=w; botrig[6].y=h;

XClearWindow(display,gadg->window);
XSetBackground(display,gc,gadg->backcolour);
switch(gadg->type) {
case boolean :
	if (gadg->jxgboolean.state)
		XSetForeground(display,gc,gadg->botcolour);
	else
		XSetForeground(display,gc,gadg->topcolour);
	tx=(w/2)-(XTextWidth(font_info,gadg->label,
	 strlen(gadg->label))/2);
	ty=(h/2)+5;
	XDrawString(display,gadg->window,gc,tx,ty,gadg->label,
	 strlen(gadg->label));
	XFillPolygon(display,gadg->window,gc,toplef,7,Nonconvex,
	 CoordModeOrigin);
	if (gadg->jxgboolean.state)
		XSetForeground(display,gc,gadg->topcolour);
	else
		XSetForeground(display,gc,gadg->botcolour);
	XFillPolygon(display,gadg->window,gc,botrig,7,Nonconvex,
	 CoordModeOrigin);
	break;
case toggle :
	if (gadg->jxgtoggle.state)
		XSetForeground(display,gc,gadg->botcolour);
	else
		XSetForeground(display,gc,gadg->topcolour);
	tx=(w/2)-(XTextWidth(font_info,gadg->label,
	 strlen(gadg->label))/2);
	ty=(h/2)+5;
	XDrawString(display,gadg->window,gc,tx,ty,gadg->label,
	 strlen(gadg->label));
	XFillPolygon(display,gadg->window,gc,toplef,7,Nonconvex,
	 CoordModeOrigin);
	if (gadg->jxgtoggle.state)
		XSetForeground(display,gc,gadg->topcolour);
	else
		XSetForeground(display,gc,gadg->botcolour);
	XFillPolygon(display,gadg->window,gc,botrig,7,Nonconvex,
	 CoordModeOrigin);
	break;
case string :
	XSetForeground(display,gc,gadg->topcolour);
	XDrawString(display,gadg->window,gc,2,(h/2)+5,gadg->label,
	 gadg->jxgstring.len);
	XSetForeground(display,gc,gadg->botcolour);
	XDrawRectangle(display,gadg->window,gc,0,0,w-1,h-1);
	break;
	}
}

void JXGadgetEvent(Display *display, XEvent *event)
{
JXgadget *pos;
JXgadget *clicked=NULL;
Window ewin;
char c;
KeySym keysym;
int key;
XComposeStatus compose;

switch(event->type) {
case KeyPress :
	ewin=event->xkey.window;
	break;
case ButtonPress :
case ButtonRelease :
	ewin=event->xbutton.window;
	break;
case LeaveNotify :
	ewin=event->xcrossing.window;
	break;
case Expose :
	ewin=event->xexpose.window;
	break;
default :
	return;
	}

for(pos=glist;pos != NULL;pos=pos->next)
	if (pos->window==ewin)
		clicked=pos;
if (!clicked)
	return;
switch (clicked->type) {
case boolean :
	switch (event->type) {
	case ButtonPress :
		clicked->jxgboolean.state=1;
		JXRedrawGadget(display,clicked);
		break;
	case ButtonRelease :
		if (event->xbutton.x > 0 && event->xbutton.y >0 &&
		 event->xbutton.x < clicked->w && event->xbutton.y <
		 clicked->h && clicked->jxgboolean.state) {
			XEvent mesg={0};

			clicked->jxgboolean.state=0;
			mesg.xbutton.type=ClickEvent;
			mesg.xbutton.send_event=True;
			mesg.xbutton.display=display;
			mesg.xbutton.window=clicked->window;
			mesg.xbutton.root=clicked->parent;
			mesg.xbutton.x=event->xbutton.x;
			mesg.xbutton.y=event->xbutton.y;
			mesg.xbutton.button=clicked->number;
			XPutBackEvent(display,&mesg);
			JXRedrawGadget(display,clicked);
			}
		break;
	case LeaveNotify :
		clicked->jxgboolean.state=0;
		JXRedrawGadget(display,clicked);
		break;
	case Expose :
		JXRedrawGadget(display,clicked);
		break;
		}
	break;
case toggle :
	switch (event->type) {
	case ButtonPress :
		clicked->jxgtoggle.state = !clicked->jxgtoggle.state;
		JXRedrawGadget(display,clicked);
		break;
	case Expose :
		JXRedrawGadget(display,clicked);
		break;
		}
	break;
case string :
	switch (event->type) {
	case KeyPress :
		XLookupString(&(event->xkey),&c,1,&keysym,&compose);
		key=(int)keysym;
		if (key==XK_Delete || key==XK_BackSpace) {
			if (clicked->jxgstring.len) {
				clicked->jxgstring.len -= 1;
				clicked->label[clicked->jxgstring.len]
				 =='\0';
				}
			}
		else {
			if (key < 256) {
				clicked->label[clicked->jxgstring.len]=c;
				clicked->jxgstring.len++;
				clicked->label[clicked->jxgstring.len]=='\0';
				}
			}
		JXRedrawGadget(display,clicked);
		break;
	case Expose :
		JXRedrawGadget(display,clicked);
		break;
		}
	break;
	}
event->type=GadgetEvent;
}

void JXStartup(Display *display, int screen_num)
{
gc=XCreateGC(display,RootWindow(display,screen_num),0,NULL);
font_info=XLoadQueryFont(display,"9x15");
XSetFont(display,gc,font_info->fid);
glist=NULL;
gadget_count=0;
}

Bool JXGM(JXgadget *gadg, int gnum)
{
if (gadg->number==gnum)
	return(True);
else
	return(False);
}

