#include "x.h"
#include <stdarg.h>

#define GadgetEvent 999
#define ClickEvent 998
#define NewSliderPos 997
#define CancelEvent 996

#define arrow_w 10
#define arrow_h 10
#ifndef min
#define min(x,y) ((x) < (y) ? (x) : (y))
#endif
#ifndef max
#define max(x,y) ((x) > (y) ? (x) : (y))
#endif
#define MAX_WINDOW_GADGETS 20

/* a line of text, used in menus and the scrolling list */
typedef struct JXtext {
	char *str;			/* the text */
	int width;			/* width in pixels */
	int mark;			/* for menus, draw with a tick */
	struct JXtext *next;		/* next line of text */
	} JXtext;

typedef enum {boolean, toggle, string, menu, slider, slist,
	pboolean, ptoggle, leftarr, rightarr, uparr, downarr,
	scroller, stext, prslider, prscroller, file, filewin,
	choice, choicewin, combo, twoword, twowordwin, window,
	bevelbox, pane, scrollpane, message, statusgauge } JXgtype;

typedef enum {a_left,a_right,a_centre} JXalign;

typedef struct JXgadget {		/* gadget structure */
	Window window;			/* subwindow for this gadget */
	Window parent;			/* window gadget is in */
	int x;				/* position */
	int y;
	int w;				/* size (-1 for both from */
	int h;				/* width and height) */
	long topcolour;			/* colours of gadget */
	long botcolour;
	long backcolour;
	long hicolour;
	JXgtype type;			/* type of gadget */
	int number;			/* gadget number */
	struct JXgadget *owner;		/* used if this gadget is */
					/* part of another */
	char *label;			/* writing on gadget, or in box */
	Bool in;			/* is the pointer in the window */
	Bool active;			/* is this gadget clickable? */
	Pixmap dbuf;			/* pixmap for double buffering */
	union {
		struct {		/* simple "click" gadget */
			int state;
			} jxgboolean_;
		struct {		/* two state gadget */
			int state;	/* 0=off, 1=on */
			} jxgtoggle_;
		struct {		/* a string gadget */
			int len;	/* characters in string */
			int maxlen;	/* max size of string */
			Bool grab;	/* is the keyboard being grabbed? */
			Window twin;	/* sub-window text appears in */
			JXalign aln;	/* alignment of text */
			int cur;	/* char cursor is before */
			Bool editable;	/* is this string editable? */
			} jxgstring_;
		struct {		/* pop up/down menu */
			Bool open;	/* is this menu open? */
			Window mwin;	/* window menu list appears in */
			JXtext *buf;	/* list of menu items */
			int lines;	/* number of menu items */
			int w,h;	/* size of the menu window */
			} jxgmenu_;
		struct {		/* non-proportional slider */
			int pos;	/* pixel position of slider */
			float npos;	/* normalized position (0-1) */
			int slsize;	/* size of slider */
			Bool horiz;	/* true if horizontal, false */
					/* if vertical */
			int xyrel;	/* position inside slider */
			Bool report;	/* report all movements of slider? */
			Bool smooth;	/* redraw after every move? */
			Bool wrap;	/* wrap around when clicked off edges?*/
			} jxgslider_;
		struct {		/* a scrollable list of text */
			struct JXgadget *slg;
					/* slider used for scrolling */
			JXtext *buf;	/* linked list of lines */
			int lines;	/* number of lines */
			int cline;	/* current first line */
			Bool wrap;	/* wrap too-long lines? */
			} jxgslist_;
		struct {		/* "click" gadget with a picture */
			int state;	/* 0=out, 1=in */
			Pixmap pic;	/* picture on this button */
			int pic_w;	/* pixmap width */
			int pic_h;	/* pixmap height */
			} jxgpboolean_;
		struct {		/* on/off gadget with a picture */
			int state;	/* 0=out, 1=in */
			Pixmap pic;	/* picture on button */
			int pic_w;	/* pixmap width */
			int pic_h;	/* pixmap height */
			} jxgptoggle_;
		struct {		/* slider with arrows at end */
			struct JXgadget *dir1;	/* left/up arrow */
			struct JXgadget *dir2;	/* down/right arrow */
			struct JXgadget *slg;	/* slider */
			Bool horiz;		/* true=horizontal */
			double step;	/* amount to move on arrow press */
			} jxgscroller_;
		struct {		/* unchanging text */
			JXalign align;	/* alignment of text */
			} jxgstext_;
		struct {		/* proportional slider */
			int pos;	/* pixel position of slider */
			float npos;	/* normalized position (0-1) */
			int lsize;	/* length of slider */
			int wsize;	/* width of slider */
			Bool horiz;	/* true if horizontal, false */
					/* if vertical */
			int xyrel;	/* position inside slider */
			Bool report;	/* report all movements of slider? */
			Bool smooth;	/* redraw after every move? */
			float frac;	/* fraction of total shown */
			Bool wrap;	/* wrap around? */
			} jxgprslider_;
		struct {		/* prop. slider with arrows at ends */
			struct JXgadget *dir1;	/* left/up arrow */
			struct JXgadget *dir2;	/* down/right arrow */
			struct JXgadget *slg;	/* prslider */
			Bool horiz;		/* true=horizontal */
			double step;	/* amount to move on arrow press */
			} jxgprscroller_;
		struct {		/* file load/save selector */
			struct JXgadget *ls;	/* the list of files */
			struct JXgadget *fname;	/* file to load/save */
			struct JXgadget *path;	/* path to the file */
			struct JXgadget *par;	/* parent button */
			struct JXgadget *but;	/* load/save button */
			struct JXgadget *can;	/* cancel button */
			} jxgfile_;
		struct {		/* Choose one of many buttons */
			struct JXgadget *head;	/* heading text */
			struct JXgadget **buts;	/* the choice buttons */
			int bcount;		/* number of choices */
			} jxgchoice_;
		struct {		/* Windows like combo box */
			struct JXgadget *str;	/* thing chosen */
			struct JXgadget *but;	/* menu button */
			Bool open;	/* is the menu open? */
			Window mwin;	/* window menu list appears in */
			JXtext *buf;	/* list of menu items */
			int lines;	/* number of menu items */
			int w,h;	/* size of the menu window */
			int cch;	/* last choice from list */
			} jxgcombo_;
		struct {		/* Enter two strings */
			struct JXgadget *str1;	/* gadgets for strings */
			struct JXgadget *str2;
			struct JXgadget *pr1;	/* gadgets for prompts */
			struct JXgadget *pr2;
			struct JXgadget *ok;	/* ok button */
			struct JXgadget *can;	/* cancel button */
			} jxgtwoword_;
		struct {		/* A top-level window */
			Bool popup;	/* transient window? */
			Bool bevel;	/* window is bevelled? */
			Bool toplevel;	/* is this a top-level window? */
			int gcount;
			struct JXgadget **gl;	/* gadgets in this window */
			} jxgwindow_;
		struct {		/* a self-renewing bevelled box */
			Bool indent;	/* reversed box? */
			} jxgbevelbox_;
		struct {
			Pixmap pm;	/* pixmap to scroll around */
			int w,h;	/* size of pixmap */
			int x,y;	/* position of window in pixmap */
			int mx,my;	/* click position of mouse in pixmap */
			Cursor cr;	/* special cursor */
			Bool report;	/* report movement */
			} jxgpane_;
		struct {
			struct JXgadget *pn;	/* pane gadget */
			struct JXgadget *hz;	/* horizontal scroller */
			struct JXgadget *vt;	/* vertical scroller */
			Bool report;		/* report movement */
			} jxgscrollpane_;
		struct {
			struct JXgadget **ln;	/* lines */
			int lnc;		/* number of lines */
			struct JXgadget *ok;	/* Ok gadget */
			} jxgmessage_;
		struct {
			double p;		/* amount full */
			int totalredraw;	/* need total redraw? */
			} jxgstatusgauge_;

		} JXclass;
	struct JXgadget *next;
	} JXgadget;

#define jxgboolean	JXclass.jxgboolean_
#define jxgtoggle	JXclass.jxgtoggle_ 
#define jxgstring	JXclass.jxgstring_
#define jxgslider	JXclass.jxgslider_
#define jxgslist	JXclass.jxgslist_
#define jxgpboolean	JXclass.jxgpboolean_
#define jxgptoggle	JXclass.jxgptoggle_
#define jxgscroller	JXclass.jxgscroller_
#define jxgmenu		JXclass.jxgmenu_
#define jxgstext	JXclass.jxgstext_
#define jxgprslider	JXclass.jxgprslider_
#define jxgprscroller	JXclass.jxgprscroller_
#define jxgfile		JXclass.jxgfile_
#define jxgchoice	JXclass.jxgchoice_
#define jxgcombo	JXclass.jxgcombo_
#define jxgtwoword	JXclass.jxgtwoword_
#define jxgwindow	JXclass.jxgwindow_
#define jxgbevelbox	JXclass.jxgbevelbox_
#define jxgpane		JXclass.jxgpane_
#define jxgscrollpane	JXclass.jxgscrollpane_
#define jxgmessage	JXclass.jxgmessage_
#define jxgstatusgauge	JXclass.jxgstatusgauge_


/* Definitions for JXSetOption
 */
#define HighlightCol	1
#define ReportMove	2
#define TopColour	3
#define BottomColour	4
#define BackColour	5
#define ScrollStep	6
#define PropFrac	7
#define SliderNPos	8
#define StringAlign	9
#define CurrentString	10
#define SortedList	11
#define CurrentString2	12
#define CheckItem	13
#define ButtonLabel	14
#define SmoothMove	15
#define Active		16
#define WrapSlider	17
#define ListPos		18
#define CursorColour	19
#define WrapList	20
#define StringEditable	21
#define LimitToList	22
#define TopLevel	23

/* Masks for various gadget types.
 */
#define Mboolean	ExposureMask | ButtonReleaseMask | \
         		LeaveWindowMask | ButtonPressMask | \
			StructureNotifyMask | EnterWindowMask
#define Mtoggle		ExposureMask | ButtonPressMask | \
			StructureNotifyMask | EnterWindowMask | \
			LeaveWindowMask
#define Mstring		ExposureMask | KeyPressMask | \
			StructureNotifyMask | EnterWindowMask | \
			LeaveWindowMask | ButtonPressMask
#define Mslider		ButtonPressMask | Button1MotionMask | \
			ExposureMask | ButtonReleaseMask | \
			StructureNotifyMask | EnterWindowMask | \
			LeaveWindowMask | ButtonReleaseMask
#define Mslist		ExposureMask | StructureNotifyMask | \
			ButtonPressMask | EnterWindowMask | \
			LeaveWindowMask
#define Mscroller	ExposureMask | EnterWindowMask | \
			LeaveWindowMask | StructureNotifyMask
#define Mmenu		ExposureMask | EnterWindowMask | \
			LeaveWindowMask | ButtonPressMask | \
			ButtonReleaseMask | StructureNotifyMask
#define Mstext		ExposureMask | StructureNotifyMask
#define Mprslider	ButtonPressMask | Button1MotionMask | \
			Button1MotionMask | ExposureMask | \
			ButtonReleaseMask | StructureNotifyMask | \
			EnterWindowMask | LeaveWindowMask
#define Mprscroller	ExposureMask | EnterWindowMask | \
			LeaveWindowMask | StructureNotifyMask
#define Mfile		ExposureMask | EnterWindowMask | \
			LeaveWindowMask | StructureNotifyMask
#define Mchoice		ExposureMask | EnterWindowMask | \
			LeaveWindowMask | StructureNotifyMask
#define Mcombo		ExposureMask | EnterWindowMask | \
			LeaveWindowMask | StructureNotifyMask
#define Mtwoword	ExposureMask | EnterWindowMask | \
			LeaveWindowMask | StructureNotifyMask
#define Mwindow		ExposureMask | EnterWindowMask | \
			LeaveWindowMask | StructureNotifyMask 
#define Mbevelbox	ExposureMask | StructureNotifyMask
#define Mpane		ExposureMask | StructureNotifyMask | \
			ButtonPressMask | Button1MotionMask
#define Mscrollpane	ExposureMask | StructureNotifyMask 
#define Mmessage	ExposureMask | StructureNotifyMask
#define Mstatusgauge	ExposureMask | StructureNotifyMask

/* Function prototypes for JX calls
 */
Bool exposepred(Display *,XEvent *,char *);
JXgadget *JXCreateGadget(Window,int,int,int,int,JXgtype,char *,
 Display *,long,long,long);
void JXRedrawGadget(Display *,JXgadget *);
void JXStartup(Display *,int);
int JXGadgetEvent(Display *,XEvent *);
Bool JXGM(JXgadget *,int);
Bool JXGMatch(JXgadget *, XEvent *);
void JXBevelBox(Display *,Window,int,int,int,int,int,long,long,long);
void JXMoveSlider(Display *,JXgadget *,float);
void JXAddListText(Display *,JXgadget *,char *);
void JXAddListTextSorted(Display *display, JXgadget *gadg, char *text);
JXgadget *JXNewGadget(Display *,Window,long,long,long,JXgtype,...);
JXgadget *JXRealNewGadget(Display *,Window,long,long,long,JXgtype,va_list);
void JXNotify(Display *, Window, int, long);
void JXMoveGadget(Display *, JXgadget *, int, int, int, int);
void JXFreeGadget(Display *, JXgadget *);
void JXAddMenuItem(Display *,JXgadget *,char *);
void JXRedrawMenu(Display *, JXgadget *);
void JXMovePrSlider(Display *,JXgadget *,float);
void JXSetOption(Display *, JXgadget *, ...);
void JXSetOptionAll(Display *, ...);
void JXFileList(Display *, JXgadget *, char *);
JXgadget *JXWindowAdd(Display *, JXgadget *, long, long, long, JXgtype , ...);
void JXClearListText(Display *, JXgadget *);
char *JXListEntry(JXgadget *g, int en);
void JXClearGadget(Display *, JXgadget *);
void JXWindowDelete(Display *display, JXgadget *wn, JXgadget *v);
void JXWindowClear(Display *display, JXgadget *wn);

/* from JXUtil.c */
void JXDeleteText(JXtext *);
int JXIsDirectory(char *);
void JXGetWindowPos(Display *, Window, int *, int *);
char *JXExpandTabs(char *);

/* Macros to make access to gadget parts opaque.
 */
#define NumberFromGadget(g)		(g->number)

#define StateFromToggle(g)		(g->jxgtoggle.state)

#define StringFromString(g)		(g->label)

#define NPosFromSlider(g)		(g->jxgslider.npos)

#define NPosFromScroller(g)		(g->jxgscroller.slg->jxgslider.npos)

#define NPosFromPrSlider(g)		(g->jxgprslider.npos)

#define NPosFromPrScroller(g)		(g->jxgprscroller.slg->jxgprslider.npos)

#define FilenameFromFile(g)		(g->jxgfile.fname)
#define FilenameStringFromFile(g)	(FilenameFromFile(g)->label)
#define FilepathFromFile(g)		(g->jxgfile.path)
#define FilepathStringFromFile(g)	(FilepathFromFile(g)->label)
#define FilelistFromFile(g)		(g->jxgfile.ls)
#define ButtonFromFile(g)		(g->jxgfile.but)
#define ParentFromFile(g)		(g->jxgfile.par)
#define CancelFromFile(g)		(g->jxgfile.can)

#define ButtonFromChoice(g,n)		(g->jxgchoice.buts[n])
#define CountFromChoice(g)		(g->jxgchoice.bcount)

#define StringFromCombo(g)		(g->jxgcombo.str->label)
#define TextFromCombo(g)		StringFromCombo(g)
#define StrFromCombo(g)			(g->jxgcombo.str)
#define ChoiceFromCombo(g)		(g->jxgcombo.cch)

#define String1FromTwoword(g)		(g->jxgtwoword.str1->label)
#define String2FromTwoword(g)		(g->jxgtwoword.str2->label)
#define Str1FromTwoword(g)		(g->jxgtwoword.str1)
#define Str2FromTwoword(g)		(g->jxgtwoword.str2)

#define GadFromWin(g,n)			(g ? g->jxgwindow.gl[n] : NULL)
#define CountFromWin(g)			(g->jxgwindow.gcount)

#define XFromPane(g)			(g->jxgpane.x)
#define YFromPane(g)			(g->jxgpane.y)

#define PaneFromScrollpane(g)		(g->jxgscrollpane.pn)
#define HorizFromScrollpane(g)		(g->jxgscrollpane.hz)
#define VertFromScrollpane(g)		(g->jxgscrollpane.vt)

#define DataFromEvent(e)		((e)->xclient.data.l[1])
#define SenderFromEvent(e)		((e)->xclient.data.l[0])


/* Global variables used by the JX functions.
 * Mostly commonly used X11 parameters needed for calling Xlib
 * functions.
 */
extern GC gc;
extern XFontStruct *font_info;
extern JXgadget *glist;
extern int gadget_count;
extern Pixmap jx_left;
extern Pixmap jx_right;
extern Pixmap jx_up;
extern Pixmap jx_down;
extern Window root;
extern int def_depth;

