#include "sim.h"
#include "client.h"

/* display_mime_data
 * Use content_type to call correct application to view data in given
 * file.
 */
void display_mime_data(char *fn, char *content_type, int content_length)
{
int fd = -1;		/* fd for .mailcap file */
char conttype[256];	/* copy of content_type */
char *cmaj;		/* major type of content-type */
char *cmin;		/* minor type of content-type */

strcpy(conttype,content_type);
cmaj = conttype;
cmin = strchr(conttype,'/'); *(cmin++) = '\0';

/* Try to open mailcap file
 */
if (getenv("HOME")) {
	char mailcap[256];

	sprintf(mailcap,"%s/%s",getenv("HOME"),MAILCAP);
	fd = open(mailcap,O_RDONLY);
	}
if (fd == -1) {
	fd = open(MAILCAP,O_RDONLY);
	}
if (fd != -1) {
	/* Lookup viewer in file
	 */
	char line[256];

	while(fdgets(fd,line,256) != NULL) {
		char *type;		/* mime type */
		char *cmd;		/* command for that type */
		char *maj, *min;	/* major and minor parts of type */

		if ((type = strtok(line," ;\t\n")) == NULL ||
		    (cmd = strtok(NULL,"\n")) == NULL)
			continue;
		maj = type;
		if ((min = strchr(type,'/')))
			*(min++) = '\0';
		else
			continue;
		if ((strequ(min,"*") || strequ(min,cmin)) &&
		    (strequ(maj,"*") || strequ(maj,cmaj))) {
			/* Found matching viewer! Use it
			 */
			close(fd);
			run_mime_viewer(cmd,fn);
			return;
			}
		}
	close(fd);
	}

/* Use internal viewer names
 */
if (strequ(cmaj,"image"))
	run_mime_viewer("xv %s",fn);
else if (strequ(cmaj,"text") && strequ(cmin,"plain"))
	run_mime_viewer("xterm -e less %s",fn);
else if (strequ(cmaj,"text") && strequ(cmin,"html"))
	run_mime_viewer("xmosaic %s",fn);
else if (strequ(cmaj,"text") && strequ(cmin,"x-url"))
	run_mime_viewer("xmosaic `cat %s`",fn);
else if (strequ(cmaj,"text") && strequ(cmin,"x-tel"))
	run_mime_viewer("xterm -e telnet `cat %s`",fn);
else if (strequ(cmaj,"application") && strequ(cmin,"postscript"))
	run_mime_viewer("ghostview %s",fn);
else if (strequ(cmaj,"video") && strequ(cmin,"mpeg"))
	run_mime_viewer("mpeg_play %s",fn);
else if (strequ(cmaj,"video") && strequ(cmin,"quicktime"))
	run_mime_viewer("xanim %s",fn);
else if (strequ(cmaj,"video") && strequ(cmin,"x-sgi-movie"))
	run_mime_viewer("movieplayer %s",fn);
else
	logerror("Couldn't find viewer for mime type %s/%s",cmaj,cmin);
}


/* run_mime_viewer
 * Runs the given viewer command for a file. Returns 0 if unsuccessful,
 * non-zero otherwise. 
 */
void run_mime_viewer(char *cmd, char *file)
{
char combuf[256];
pid_t pid;

if (strstr(cmd,"%s"))
	sprintf(combuf,cmd,file);
else
	sprintf(combuf,"%s < %s",cmd,file);
if (!(pid = sfork())) {
	/* Call sh to do the dirty work
	 */
	execlp("/bin/sh","sh","-c",combuf,NULL);
	logerror("Couldn't run /bin/sh!");
	exit(127);
	}
/* Hack to wait for process to finish
 */
while(kill(pid,0) != -1)
	sleep(1);
}

