/* netclient.c
 *
 * Networking functions used by sim clients only
 */
#include "sim.h"
#include "client.h"

/* connect_server
 * Make initial connection to the given server and determine which
 * sim server this player is really on.
 */
int connect_server(char *name, player **p, int ilvl, int (*shapeparser)(int fd))
{
int fd;			/* connection to server */
char nhost[256];	/* address of correct host */
char nportstr[20];	/* port string on correct host */
int  nport;		/* port on correct host */
userservprefs csp;	/* current prefs, for getting and changing port */

get_sprefs(&csp);
if ((fd = open_tcp_send(csp.server,csp.port)) == -1) {
	logerror("Couldn't connect to %s port %d",csp.server,csp.port);
	return -1;
	}
if (!send_version(fd))
	return -1;
/*logmessage("Made initial connection to %s port %d",csp.server,csp.port);*/

fdprintf(fd,"%d\n",con_newplayer);
fdprintf(fd,"%s\n",name);

fdgetw(fd,nhost,256);
fdgetw(fd,nportstr,20);
if (strequ(nhost,"fail")) {
	logerror("Server %s is down. Request to join rejected",nportstr);
	return -1;
	}
csp.port = atoi(nportstr);
strcpy(csp.server,nhost);
put_sprefs(&csp);
close(fd);

return reconnect_server(name,p,ilvl,shapeparser);
}


/* reconnect_server
 * Connect to a server this player is known to be on, and download
 * player and map info
 */
int reconnect_server(char *name, player **p, int ilvl,
		     int (*shapeparser)(int fd))
{
int fd;			/* connection to server */
char resp[20];		/* response to oldplayer request */
userservprefs *csp;	/* current prefs (for server and port) */
userprefs *cp;		/* current prefs (for password) */

csp = get_sprefs(NULL);
cp  = get_prefs(NULL);
if ((fd = open_tcp_send(csp->server,csp->port)) == -1) {
	logerror("Couldn't reconnect to %s port %d",csp->server,csp->port);
	return -1;
	}
if (!send_version(fd))
	return -1;
/*logmessage("Re-connected to %s port %d",csp->server,csp->port);*/

fdprintf(fd,"%d\n",con_oldplayer);
fdprintf(fd,"%s\n",name);
fdprintf(fd,"%s\n",cp->password);
fdgetw(fd,resp,20);
if (strequ(resp,"fail")) {
	logerror("Player %s not found on server %s port %d!!",name,csp->server,
							      csp->port);
	return -1;
	}
if (strequ(resp,"already")) {
	logerror("A player, name or object called %s is already active "
		 "on server %s port %d",name,csp->server,csp->port);
	return -1;
	}
if (strequ(resp,"wrong")) {
	logerror("Wrong password for player %s",name);
	return -1;
	}

/* Parse map and player info
 */
if (parse_nodes(fd))
	return -1;
if (parse_paths(fd))
	return -1;
if (parse_persons(fd))
	return -1;
if (parse_objects(fd))
	return -1;
if (parse_player(fd,p))
	return -1;

/* Request and parse 3d info
 */
fdprintf(fd,"%d\n",ilvl);
if (ilvl != lvl_none) {
	if (shapeparser(fd))
		return -1;
	}
return fd;
}


/* client_request
 * Send a request message to the connected server.
 * Depending on the req argument, the following args are:
 *  req_move : int link
 *  req_say : char **words, int wordc
 *  req_yell : char **words, int wordc
 *  req_get : char *item
 *  req_do : char *action, char *thing, char **params, int paramc
 *  req_drop : char *item
 *  req_turn : double angle
 *  req_give : char *item, char *person
 *  req_upload : char *file, int size, int port
 *  req_send : char *data, char *person
 *  req_actlist : char *object
 *  req_ls : 
 *  req_rm : char *data
 *  req_setpass : char *pass
 *  req_suicide :
 *  req_avatar : 
 *  req_email : char *address
 *  req_fullname : char *name
 */
void client_request(int fd, int req, ...)
{
va_list ap;
static mesgargs rma[] = {
	{req_move,{arg_int,0}},
	{req_say,{arg_strs,arg_int,0}},
	{req_yell,{arg_strs,arg_int,0}},
	{req_get,{arg_str,0}},
	{req_do,{arg_str,arg_str,arg_strs,arg_int,0}},
	{req_drop,{arg_str,0}},
	{req_turn,{arg_dbl,0}},
	{req_give,{arg_str,arg_str,0}},
	{req_upload,{arg_str,arg_int,arg_int,0}},
	{req_send,{arg_str,arg_str,0}},
	{req_actlist,{arg_str,0}},
	{req_ls,{0}},
	{req_rm,{arg_str,0}},
	{req_setpass,{arg_str,0}},
	{req_suicide,{0}},
	{req_avatar,{0}},
	{req_email,{arg_str,0}},
	{req_fullname,{arg_str,0}}
	};

va_start(ap,req);
send_message(fd,req,ap,rma,sizeof(rma)/sizeof(mesgargs));
va_end(ap);
}


/* recv_display_data
 * Concurrently read data from the server, then depending on act
 *  0 - save to filename given by fn
 *  2 - display with external viewer
 *  3 - throw away
 */
void recv_display_data(char *servhost, char *type, int len, int port, int act,
		       char *fn)
{
char datafile[256];		/* name of file to store data to */

if (act == 3)
	return;		/* sender will time out */
else if (act == 0) {
	strcpy(datafile,fn);
	}
else {
	/* Add a sane extension to the file, for viewers that can't live without
	 * one. ie - if the type is image/gif add .gif to the temp file name
	 */
	if (strchr(type,'/'))
		sprintf(datafile,"%s.%s",tmpnam(NULL),strchr(type,'/')+1);
	else
		strcpy(datafile,tmpnam(NULL));
	}

if (!sfork()) {
	int tfd;
	struct hostent *hent;

	set_signals(SIG_DFL);
	hent = gethostbyname(servhost);
	reset_fds();
	tfd = creat(datafile,0700);
	if (!tfd) {
		logerror("Couldn't create %s file %s",act ? "temp" : "data",
			 datafile);
		exit(0);
		}
	if (!hent) {
		logerror("Couldn't locate server %s!",servhost);
		exit(0);
		}
	if (recv_data(*(struct in_addr *)hent->h_addr,port,tfd,len) == 0) {
		logerror("Error reading data from server");
		exit(0);
		}
	close(tfd);
	if (act == 2) {
		display_mime_data(datafile,type,len);
		unlink(datafile);
		}
	exit(0);
	}
}

