/* node.c
 *
 * Functions for manipulating nodes and paths.
 */
#include "sim.h"

static hashtable nt = {0};
static path *pathhead = NULL;
static path **pathtailptr = &pathhead;
static path *pathpos;

/* create_node
 * Allocate and initalize a node
 */
node *create_node(void)
{
node *nn = scalloc(1,sizeof(node));

nn->x = nn->y = nn->z = 0.0;
nn->desc = NULL;
nn->serv.port = SIM_PORT;
return nn;
}

/* hash table functions
 */
void add_node(node *n)
{ add_entry(&nt,n); }

node *lookup_node(char *name)
{ return lookup_entry(&nt,name); }

void reset_node(void)
{ reset_position(&nt); }

node *next_node(void)
{ return next_position(&nt); }

void remove_node(node *n)
{ remove_entry(&nt,n); }

void clear_nodes(void)
{ clear_entries(&nt,(freefunc)free_node); }


/* free_node
 * Deep-free a node structure
 */
void free_node(node *n)
{
if (n->desc)
	free(n->desc);
free(n);
}


/* close_node
 * Returns non-zero if nodes n1 and n2 have a path between them, or
 * are the same node.
 */
int close_node(node *n1, node *n2)
{
int i;

if (!n1 || !n2)
	return 0;
if (n1 == n2)
	return 1;
for(i=0; i<n1->linkc; i++)
	if (other_end(n1->link[i],n1) == n2)
		return 1;
return 0;
}


/* create_path
 * Allocate a new path struct
 */
path *create_path(void)
{
path *np = scalloc(1,sizeof(path));

return np;
}


/* add_path
 * Connect a path to it's nodes and add it to the path list
 */
void add_path(path *p)
{
p->na->link[p->na->linkc++] = p;
p->nb->link[p->nb->linkc++] = p;

*pathtailptr = p;
pathtailptr = &p->next;
}


/* other_end
 * Returns the end-node of a path which is not the given node
 */
node *other_end(path *pt, node *n)
{
if (pt->na == n)
	return pt->nb;
else
	return pt->na;
}


/* reset_path
 * Resets the step-through pointer for paths to the head of the list
 */
void reset_path(void)
{
pathpos = pathhead;
}


/* next_path
 * Returns and increments step-through pointer
 */
path *next_path(void)
{
path *next = pathpos;

if (pathpos)
	pathpos = pathpos->next;
return next;
}


/* free_path
 * Deep-free a path structure
 */
void free_path(path *p)
{
free(p);
}


/* clear_paths
 * Completely clear out and free up the paths list
 */
void clear_paths(void)
{
path *pp, *nxt;

for(pp=pathhead; pp; pp=nxt) {
	nxt = pp->next;
	free_path(pp);
	}
pathhead = NULL;
pathtailptr = &pathhead;
pathpos = NULL;
}


