%{
/* nodeparser.y
 *
 * Yacc script for parsing a node.
 */
#include "sim.h"
%}

%union {
	double num;
	char string[256];
	}
%token <num> NUM
%token <string> STRING
%token END,NODE,POS,DESC,REMOTE,ADDR,PORT
%type <string> strings
%%
input		:	END
			{ return -1;
			}
		|	node
			{ return 0;
			}
		;

node		:	NODE STRING '{'
			{ nn = create_node();
			  strcpy(nn->name,$2);
			}
			nodeattribs '}'
		;

nodeattribs	:	nodeattribs nodeattrib
		|
		;

nodeattrib	:	POS NUM NUM NUM
			{ nn->x = $2;
			  nn->y = $3;
			  nn->z = $4;
			}
		|	DESC strings
			{ nn->desc = strdupe($2);
			}
		|	REMOTE '{'
			{ nn->special = 1;
			}
			remoteattribs '}'
		;

remoteattribs	:	remoteattribs remoteattrib
		|
		;

remoteattrib	:	ADDR STRING
			{ nn->serv.addr = strdupe($2);
			}
		|	PORT NUM
			{ nn->serv.port = (int)$2;
			}
		;

strings		:	strings STRING
			{ sprintf($$,"%s%s",$1,$2);
			}
		|	STRING
			{ sprintf($$,"%s",$1);
			}
		;
%%
static node *nn;		/* node being read */

static reserved rwtab[] = {	"end",END,"node",NODE,"pos",POS,"desc",DESC,
				"remote",REMOTE,"addr",ADDR,"port",PORT };
#define error yyerror
#define lex yylex
#define lval yylval

#include "lexer.h"

/* parsenode
 * Parse one node. Returns 0 if ok, 1 on error and -1 if 'end' was read.
 */
int parse_node(int filed, node **nptr)
{
int rv;

init_lexer(filed);
rv = yyparse();
*nptr = nn;
return rv;
}


/* parsenodes
 * Parses nodes until an 'end' is encountered, adding each one to the
 * node table. Returns 0 if successful, or 1 on error.
 */
int parse_nodes(int filed)
{
int rv;

init_lexer(filed);
while((rv = yyparse()) == 0)
	add_node(nn);
if (rv == 1)
	return 1;
else
	return 0;
}


/* write_node
 * Output one node to a file descriptor
 */
void write_node(int fd, int tl, node *n)
{
fdprintf(fd,"%snode \"%s\" {\n",tabs(tl),n->name);
fdprintf(fd,"%spos %f %f %f\n",tabs(tl+1),n->x,n->y,n->z);
fdprintf(fd,"%sdesc \"%s\"\n",tabs(tl+1),n->desc);
if (n->special) {
	fdprintf(fd,"%sremote {\n",tabs(tl+1));
	fdprintf(fd,"%saddr \"%s\"\n",tabs(tl+2),n->serv.addr);
	fdprintf(fd,"%sport %d\n",tabs(tl+2),n->serv.port);
	fdprintf(fd,"%s}\n",tabs(tl+2));
	}
fdprintf(fd,"%s}\n",tabs(tl+1));
}


/* write_nodes
 * Output all nodes in hash table
 */
void write_nodes(int fd)
{
node *n;

reset_node();
while((n = next_node())) {
	write_node(fd,0,n);
	fdputc(fd,'\n');
	}
fdprintf(fd,"end\n");
}

