%{
/* objectparser.y
 *
 * Yacc script for parsing an object
 */
#include "sim.h"
%}

%union {
	double num;
	char string[256];
	}
%token <num> NUM
%token <string> STRING
%token END,OBJECT,DESC,POS,GETTABLE,UNGETTABLE,WEIGHT,INV
%type <string> strings
%%
input		:	END
			{ return -1;
			}
		|	object
			{ return 0;
			}
		;

object		:	OBJECT STRING '{'
			{ no = create_object();
			  strcpy(no->name,$2);
			}
			objectattribs '}'
		;

objectattribs	:	objectattribs objectattrib
		|
		;

objectattrib	:	DESC strings
			{ no->desc = strdupe($2);
			}
		|	POS STRING
			{ no->pos = lookup_node($2);
			  if (!no->pos) {
			    yyerror("Node does not exist");
			    return 1;
			    }
			}
		|	GETTABLE
			{ no->gettable = 1;
			}
		|	UNGETTABLE
			{ no->gettable = 0;
			}
		|	WEIGHT NUM
			{ no->weight = $2;
			}
		|	INV STRING
			{ no->inv = lookup_person($2);
			}
		;


strings		:	strings STRING
			{ sprintf($$,"%s%s",$1,$2);
			}
		|	STRING
			{ sprintf($$,"%s",$1);
			}
		;

%%
static object *no;		/* object being read */

static reserved rwtab[] = {	"end",END,"desc",DESC,"pos",POS,
				"gettable",GETTABLE,"ungettable",UNGETTABLE,
				"weight",WEIGHT,"object",OBJECT,"inv",INV };

#define error yyerror
#define lex yylex
#define lval yylval

#include "lexer.h"

/* parseobject
 * Parse one object. Returns 0 if ok, 1 on error and -1 if 'end' was read.
 */
int parse_object(int filed, object **optr)
{
int rv;

init_lexer(filed);
rv = yyparse();
*optr = no;
return rv;
}


/* parse_objects
 * Read objects till an 'end' is hit, adding them to the object table
 */
int parse_objects(int filed)
{
int rv;

init_lexer(filed);
while((rv = yyparse()) == 0)
	add_object(no);
if (rv == 1)
	return 1;
else
	return 0;
}


/* write_object
 * Output one object to a file descriptor
 */
void write_object(int fd, int tl, object *o)
{
fdprintf(fd,"%sobject \"%s\" {\n",tabs(tl),o->name);
fdprintf(fd,"%sdesc \"%s\"\n",tabs(tl+1),o->desc);
if (o->pos)
	fdprintf(fd,"%spos \"%s\"\n",tabs(tl+1),o->pos->name);
if (o->gettable)
	fdprintf(fd,"%sgettable\n",tabs(tl+1));
else
	fdprintf(fd,"%sungettable\n",tabs(tl+1));
fdprintf(fd,"%sweight %f\n",tabs(tl+1),o->weight);
if (o->inv)
	fdprintf(fd,"%sinv \"%s\"\n",tabs(tl+1),o->inv->name);
fdprintf(fd,"%s}\n",tabs(tl+1));
}


/* write_objects
 * Output all objects in hash table
 */
void write_objects(int fd)
{
object *o;

reset_object();
while((o = next_object())) {
	write_object(fd,0,o);
	fdputc(fd,'\n');
	}
fdprintf(fd,"end\n");
}

