%{
/* pathparser.y
 *
 * Yacc script for parsing paths
 */
#include "sim.h"
%}

%union {
	double num;
	char string[256];
	}
%token <num> NUM
%token <string> STRING
%token END,PATH,CLOSED
%%
input		:	END
			{ return -1;
			}
		|	path
			{ return 0;
			}
		;

path		:	PATH
			{ np = create_path();
			}
			optional_closed STRING STRING
			{ if ((np->na = lookup_node($4)) == NULL) {
			    yyerror("Node does not exist");
			    return 1;
			    }
			  if ((np->nb = lookup_node($5)) == NULL) {
			    yyerror("Node does not exist");
			    return 1;
			    }
			}
		;

optional_closed	:	CLOSED
			{ np->closed = 1;
			}
		|
		;

%%
static path *np;		/* path being read */

static reserved rwtab[] = {	"end",END,"path",PATH,"closed",CLOSED };

#define error yyerror
#define lex yylex
#define lval yylval

#include "lexer.h"

/* parsepath
 * Parse one path. Returns 0 if successful, 1 on error or -1 if 'end' hit
 */
int parse_path(int filed, path **pptr)
{
int rv;

init_lexer(filed);
rv = yyparse();
*pptr = np;
return rv;
}


/* parsepaths
 * Parses paths and adds them to the map. Returns 0 if ok, or 1 one error
 */
int parse_paths(int filed)
{
int rv;

init_lexer(filed);
while((rv = yyparse()) == 0)
	add_path(np);
if (rv == 1)
	return 1;
else;
	return 0;
}


/* write_path
 * Output one path to a fd
 */
void write_path(int fd, int tl, path *p)
{
fdprintf(fd,"%spath %s\"%s\" \"%s\"\n",tabs(tl),p->closed ? "closed " : "",
				       p->na->name,p->nb->name);
}


/* write_paths
 * Output all paths in path list
 */
void write_paths(int fd)
{
path *p;

reset_path();
while((p = next_path())) {
	write_path(fd,0,p);
	fdputc(fd,'\n');
	}
fdprintf(fd,"end\n");
}

