/* person.c
 *
 * Functions for manipulating persons
 */
#include "sim.h"

static hashtable pt = {0};

/* create_person
 * Alloc and init a person structure
 */
person *create_person(void)
{
person *np = scalloc(1,sizeof(person));

np->invc = 0;
np->ang  = 0.0;
return np;
}


/* Hash table functions
 */
void add_person(person *p)
{ add_entry(&pt,p); }

person *lookup_person(char *name)
{ return lookup_entry(&pt,name); }

void reset_person(void)
{ reset_position(&pt); }

person *next_person(void)
{ return next_position(&pt); }

void remove_person(person *p)
{ remove_entry(&pt,p); }

void clear_persons(void)
{ clear_entries(&pt,(freefunc)free_person); }


/* free_person
 * Deep-free a person structure
 */
void free_person(person *p)
{
int i;

for(i=0; i<p->invc; i++)
	free(p->inv[i]);
if (p->email)    free(p->email);
if (p->fullname) free(p->fullname);
free(p);
}


/* pickup_item
 * Add an object to a person's inventory and remove it from the node it
 * was at.
 */
void pickup_item(person *p, object *o)
{
o->pos = NULL;
o->inv = p;
p->inv[p->invc++] = strdupe(o->name);
}


/* putdown_item
 * Remove an object from a person's inventory and place it at their location
 */
void putdown_item(person *p, object *o)
{
int i;

for(i=0; !strequ(p->inv[i],o->name); i++)
	;
free(p->inv[i]);
for(; i<p->invc-1; i++)
	p->inv[i] = p->inv[i+1];
p->invc--;
o->pos = p->pos;
o->inv = NULL;
}

