%{
/* personparser.y
 *
 * Yacc script for parsing a person (players or creature)
 */
#include "sim.h"
%}

%union {
	double num;
	char string[256];
	}
%token <num> NUM
%token <string> STRING
%token END,PERSON,POS,INV,ANG,EMAIL,FULLNAME
%%
input		:	END
			{ return -1;
			}
		|	person
			{ return 0;
			}
		;

person		:	PERSON STRING '{'
			{ np = create_person();
			  strcpy(np->name,$2);
			}
			personattribs '}'
		;

personattribs	:	personattribs personattrib
		|
		;

personattrib	:	POS STRING
			{ np->pos = lookup_node($2);
			  if (!np->pos) {
			    yyerror("Node does not exist");
			    return 1;
			    }
			}
		|	ANG NUM
			{ np->ang = $2;
			}
		|	INV items
		|	EMAIL STRING
			{ np->email = strdupe($2);
			}
		|	FULLNAME STRING
			{ np->fullname = strdupe($2);
			}
		;

items		:	items item
		|
		;

item		:	STRING
			{ np->inv[np->invc++] = strdupe($1);
			}
		;
%%
static person *np;		/* person being read */

static reserved rwtab[] = {	"end",END,"person",PERSON,"pos",POS,
				"inv",INV,"ang",ANG,"email",EMAIL,
				"fullname",FULLNAME };

#define error yyerror
#define lex yylex
#define lval yylval

#include "lexer.h"

/* parseperson
 * Parse one person.
 */
int parse_person(int filed, person **pptr)
{
int rv;

init_lexer(filed);
rv = yyparse();
*pptr = np;
return rv;
}


/* parsepersons
 * Parse persons till an 'end', adding them to the persons table
 */
int parse_persons(int filed)
{
int rv;

init_lexer(filed);
while((rv = yyparse()) == 0)
	add_person(np);
if (rv == 1)
	return 1;
else
	return 0;
}


/* write_person
 * Output one person to a file descriptor
 */
void write_person(int fd, int tl, person *p)
{
int i;

fdprintf(fd,"%sperson \"%s\" {\n",tabs(tl),p->name);
fdprintf(fd,"%spos \"%s\"\n",tabs(tl+1),p->pos->name);
fdprintf(fd,"%sang %f\n",tabs(tl+1),p->ang);
fdprintf(fd,"%sinv",tabs(tl+1));
for(i=0; i<p->invc; i++)
	fdprintf(fd," \"%s\"",p->inv[i]);
fdprintf(fd,"\n");
if (p->email)
	fdprintf(fd,"%semail \"%s\"\n",tabs(tl+1),p->email);
if (p->fullname)
	fdprintf(fd,"%sfullname \"%s\"\n",tabs(tl+1),p->fullname);
fdprintf(fd,"%s}\n",tabs(tl+1));
}


/* write_persons
 * Output all persons in the hash table
 */
void write_persons(int fd)
{
person *p;

reset_person();
while((p = next_person())) {
	write_person(fd,0,p);
	fdputc(fd,'\n');
	}
fdprintf(fd,"end\n");
}

