/* player.c
 *
 * Hash table of players.
 * Functions for manipulating players
 */
#include "sim.h"

static hashtable pt = {0};

/* create_player
 * Alloc and init a new player struct
 */
player *create_player(void)
{
player *np = scalloc(1,sizeof(player));

np->ilevel = 0;
np->fd     = -1;
np->hp     = 10;
np->money  = 10;
np->wizard = 1;
np->inout  = -1;
return np;
}


/* Hash table functions
 */
void add_player(player *p)
{ add_entry(&pt,p); }

player *lookup_player(char *name)
{ return lookup_entry(&pt,name); }

void reset_player(void)
{ reset_position(&pt); }

player *next_player(void)
{ return next_position(&pt); }

void remove_player(player *p)
{ remove_entry(&pt,p); }

void clear_players(void)
{ clear_entries(&pt,(freefunc)free_player); }


/* free_player
 * Deep-free a player structure
 */
void free_player(player *p)
{
if (p->password) free(p->password);
free(p);
}


/* next_online_player
 * Like next_player, but skips over those with fd = -1, indicating not
 * currently logged in
 */
player *next_online_player(void)
{
player *pp;

do {
	pp = next_player();
	} while(pp && pp->fd == -1);
return pp;
}

