%{
/* playerparser.y
 *
 * Yacc script for parsing a player
 */
#include "sim.h"
%}

%union {
	double num;
	char string[256];
	}
%token <num> NUM
%token <string> STRING
%token END,PLAYER,HP,MONEY,ANG,PASSWORD,WIZARD,INOUT
%%
input		:	END
			{ return -1;
			}
		|	player
			{ return 0;
			}
		;

player		:	PLAYER STRING '{'
			{ np = create_player();
			  strcpy(np->name,$2);
			}
			playerattribs '}'
		;

playerattribs	:	playerattribs playerattrib
		|
		;

playerattrib	:	HP NUM
			{ np->hp = $2;
			}
		|	MONEY NUM
			{ np->money = $2;
			}
		|	PASSWORD STRING
			{ np->password = strdupe($2);
			}
		|	WIZARD NUM
			{ np->wizard = $2;
			}
		|	INOUT NUM
			{ np->inout = $2;
			}
		;
%%
static player *np;		/* player being read */

static reserved rwtab[] = {	"end",END,"player",PLAYER,"hp",HP,
				"money",MONEY,"password",PASSWORD,
				"wizard",WIZARD,"inout",INOUT };

#define error yyerror
#define lex yylex
#define lval yylval

#include "lexer.h"

/* parseplayer
 * Parse and return one player
 */
int parse_player(int filed, player **pptr)
{
int rv;

init_lexer(filed);
rv = yyparse();
*pptr = np;
return rv;
}


/* parseplayers
 * Parse players and add them to the players table
 */
int parse_players(int filed)
{
int rv;

init_lexer(filed);
while((rv = yyparse()) == 0)
	add_player(np);
if (rv == 1)
	return 1;
else
	return 0;
}


/* write_player
 * Output one player to a file descriptor
 */
void write_player(int fd, int tl, player *p)
{
fdprintf(fd,"%splayer \"%s\" {\n",tabs(tl),p->name);
fdprintf(fd,"%shp %d\n",tabs(tl+1),p->hp);
fdprintf(fd,"%smoney %d\n",tabs(tl+1),p->money);
fdprintf(fd,"%sinout %d\n",tabs(tl+1),p->inout);
if (p->password)
	fdprintf(fd,"%spassword \"%s\"\n",tabs(tl+1),p->password);
if (p->wizard)
	fdprintf(fd,"%swizard %d\n",tabs(tl+1),p->wizard);
fdprintf(fd,"%s}\n",tabs(tl+1));
}


/* write_players
 * Output all players in the player hash table
 */
void write_players(int fd)
{
player *pp;

reset_player();
while((pp = next_player())) {
	write_player(fd,0,pp);
	fdputc(fd,'\n');
	}
fdprintf(fd,"end\n");
}

