#include "s3dt.h"
#include "s3dtint.h"

/* S3AddPoint
 * Add one point to an object
 */
void S3AddPoint(S3object *obj, double x, double y, double z)
{
S3point *pt;
real radius;

if (obj->ptc >= MAX_POINTS_PER_OBJECT) {
	fprintf(stderr,"S3AddPoint : exceeded %d points per object\n",
		MAX_POINTS_PER_OBJECT);
	exit(10);
	}
obj->vpt[obj->ptc] = S3Malloc(sizeof(S3point));
pt = obj->pt[obj->ptc++] = S3Malloc(sizeof(S3point));
pt->x = DblToReal(x);
pt->y = DblToReal(y);
pt->z = DblToReal(z);

/* Increase bounding sphere
 */
radius = SqrtReal( AddReal3( SqrReal(pt->x), SqrReal(pt->y), SqrReal(pt->z)) );
if (RealGt(radius,obj->radius))
	obj->radius = radius;
}


/* S3AddPoints
 * Add a list of points to an object
 */
void S3AddPoints(S3object *obj, int ptc, S3point pt[])
{
int i;

for(i=0; i<ptc; i++) {
	if (obj->ptc >= MAX_POINTS_PER_OBJECT) {
		fprintf(stderr,"S3AddPoints : exceeded %d points per object\n",
			MAX_POINTS_PER_OBJECT);
		exit(10);
		}
	obj->pt[obj->ptc++] = S3MemDup(&pt[i],sizeof(S3point));
	}
S3BoundingSphere(obj);
}

