#include "s3dt.h"
#include "s3dtint.h"

/* S3AddPolygon
 * Add one polygon to an object
 */
void S3AddPolygon(S3object *obj, int *vert, int vertc, int colour, int type)
{
S3polygon *npoly;

if (vertc < 2) {
	fprintf(stderr,"S3AddPolygon : attempted to add polygon with "
		"%d points\n",vertc);
	exit(10);
	}
if (obj->facec >= MAX_FACES_PER_OBJECT) {
	fprintf(stderr,"S3AddPolygon : exceeded %d polygons per object\n",
		MAX_FACES_PER_OBJECT);
	exit(10);
	}

npoly = S3Calloc(1,sizeof(S3polygon));
npoly->vert = S3MemDup(vert,vertc*sizeof(int));
npoly->vertc = vertc;
npoly->colour = colour;
npoly->type = type;
npoly->obj = obj;

obj->face[obj->facec++] = npoly;
S3ComputeFace(obj,npoly);
}


/* S3AddFace
 * Adds a single polygon to an object, using a variable argument list to
 * specify the vertices
 */
void S3AddFace(S3object *obj, int colour, int type, int vertc, ...)
{
va_list ap;
int vert[MAX_VERTS_PER_FACE];	/* vertex list */
int i;

va_start(ap,vertc);
for(i=0; i<vertc; i++)
	vert[i] = va_arg(ap,int);
S3AddPolygon(obj,vert,vertc,colour,type);
}

