#include "s3dt.h"
#include "s3dtint.h"

/* S3BoundingBox
 * Find the minimum and maximum x,y and z values for an object and all
 * it's children (in world coords)
 */
void S3BoundingBox(S3object *obj, double *x1, double *y1, double *z1,
				  double *x2, double *y2, double *z2)
{
int i;

/* Find min and max coords for points in this object
 */
*x1 = *y1 = *z1 = CoordMax;
*x2 = *y2 = *z2 = -CoordMax;
for(i=0; i<obj->ptc; i++) {
	double x = RealToDbl( AddReal(obj->pos.x,obj->pt[i]->x) );
	double y = RealToDbl( AddReal(obj->pos.y,obj->pt[i]->y) );
	double z = RealToDbl( AddReal(obj->pos.z,obj->pt[i]->z) );

	*x1 = min(*x1,x); *y1 = min(*y1,y); *z1 = min(*z1,z);
	*x2 = max(*x2,x); *y2 = max(*y2,y); *z2 = max(*z2,z);
	}

/* Check sub-objects as well
 */
for(i=0; i<obj->soc; i++) {
	double cx1,cy1,cz1;
	double cx2,cy2,cz2;

	S3BoundingBox(obj->so[i],&cx1,&cy1,&cz1,&cx2,&cy2,&cz2);
	*x1 = min(*x1,cx1); *y1 = min(*y1,cy1); *z1 = min(*z1,cz1);
	*x2 = max(*x2,cx2); *y2 = max(*y2,cy2); *z2 = max(*z2,cz2);
	}
}


/* S3BoundingBoxTable
 * Find the min and max x,y and z values for every object in an object table
 */
void S3BoundingBoxTable(S3objtab *ot, double *x1, double *y1, double *z1,
				      double *x2, double *y2, double *z2)
{
S3object *obj;

*x1 = *y1 = *z1 = CoordMax;
*x2 = *y2 = *z2 = -CoordMax;
for(obj=ot->head; obj; obj=obj->next) {
	double cx1,cy1,cz1;
	double cx2,cy2,cz2;

	S3BoundingBox(obj,&cx1,&cy1,&cz1,&cx2,&cy2,&cz2);
	*x1 = min(*x1,cx1); *y1 = min(*y1,cy1); *z1 = min(*z1,cz1);
	*x2 = max(*x2,cx2); *y2 = max(*y2,cy2); *z2 = max(*z2,cz2);
	}
}

