#include "s3dt.h"
#include "s3dtint.h"

/* S3BoundingSphere
 * Compute the bounding sphere radius for a single object
 */
void S3BoundingSphere(S3object *obj)
{
int i;

obj->radius = Real0;
for(i=0; i<obj->ptc; i++) {
	S3point *p = obj->pt[i];
	real dist;

	dist = AddReal3( SqrReal(p->x), SqrReal(p->y), SqrReal(p->z) );
	if (RealGt(dist,obj->radius))
		obj->radius = dist;
	}
obj->radius = SqrtReal(obj->radius);

for(i=0; i<obj->facec; i++)
	S3ComputeFace(obj,obj->face[i]);
}


/* S3ComputeFace
 * Compute the centroid and normal of a face in world-coordinate space.
 */
void S3ComputeFace(S3object *o, S3polygon *p)
{
int i;
S3point a;

p->cen.x = p->cen.y = p->cen.z = Real0;
for(i=0; i<p->vertc; i++) {
	S3point *pt = o->pt[p->vert[i]];

	p->cen.x = AddReal(p->cen.x,pt->x);
	p->cen.y = AddReal(p->cen.y,pt->y);
	p->cen.z = AddReal(p->cen.z,pt->z);
	}
p->cen.x = DivReal(p->cen.x,IntToReal(p->vertc));
p->cen.y = DivReal(p->cen.y,IntToReal(p->vertc));
p->cen.z = DivReal(p->cen.z,IntToReal(p->vertc));

a.x = SubReal( o->pt[p->vert[0]]->x, o->pt[p->vert[1]]->x );
a.y = SubReal( o->pt[p->vert[0]]->y, o->pt[p->vert[1]]->y );
a.z = SubReal( o->pt[p->vert[0]]->z, o->pt[p->vert[1]]->z );
if (a.x == Real0 && a.y == Real0 && a.z == Real0)
	return;
if (p->vertc >= 3) {
	/* proper face.. find correct normal
	 */
	S3point b;

	b.x = SubReal( o->pt[p->vert[2]]->x, o->pt[p->vert[1]]->x );
	b.y = SubReal( o->pt[p->vert[2]]->y, o->pt[p->vert[1]]->y );
	b.z = SubReal( o->pt[p->vert[2]]->z, o->pt[p->vert[1]]->z );
	if (b.x == Real0 && b.y == Real0 && b.z == Real0)
		return;
	S3CrossProduct(&a,&b,&p->norm);
	}
else {
	/* line.. pick the normal closest to vertical
	 */
	real orth;

	if (a.x == Real0 && a.y == Real0) {
		p->norm.x = Real1; p->norm.y = Real0; p->norm.z = Real0;
		}
	else {
		p->norm.x = Real0; p->norm.y = Real0; p->norm.z = Real1;
		}
	orth = DivReal( S3DotProduct(&p->norm,&a), S3DotProduct(&a,&a));
	p->norm.x = SubReal( p->norm.x, MulReal(orth,a.x) );
	p->norm.y = SubReal( p->norm.y, MulReal(orth,a.y) );
	p->norm.z = SubReal( p->norm.z, MulReal(orth,a.z) );
	}
}

