#include "s3dt.h"
#include "s3dtint.h"

/* S3CreateObject
 * Create a new, empty object and initialize it with values from a 0
 * terminated varargs list. If no id is given in the list, one is
 * assigned automatically.
 */
S3object *S3CreateObject(char *name, ...)
{
S3object *nobj;
va_list ap;

va_start(ap,name);
nobj = S3VCreateObject(name,ap);
if (nobj->id == -1)
	nobj->id = S3AllocId();
va_end(ap);
return nobj;
}


/* S3VCreateObject (internal use only)
 * Creates a new empty object, and assigns selected attributes to it from
 * a variable arguments list
 */
S3object *S3VCreateObject(char *name, va_list ap)
{
S3object *nobj = S3Calloc(1,sizeof(S3object));
S3ecreate crf;		/* function to create extra data */
int attrib;		/* attribute to set */

/* Initialize attributes to sensible defaults
 */
nobj->id     = -1;
strcpy(nobj->name,name);
nobj->pt     = S3Calloc(MAX_POINTS_PER_OBJECT,sizeof(S3point *));
nobj->vpt    = S3Calloc(MAX_POINTS_PER_OBJECT,sizeof(S3point *));
nobj->ptc    = 0;
nobj->face   = S3Calloc(MAX_FACES_PER_OBJECT,sizeof(S3polygon *));
nobj->facec  = 0;
nobj->radius = 0.0;
nobj->pos.x  = 0.0;
nobj->pos.y  = 0.0;
nobj->pos.z  = 0.0;
nobj->next   = NULL;
nobj->hnext  = NULL;
nobj->so     = NULL;
nobj->soc    = 0;
nobj->somax  = 0;
nobj->par    = NULL;
nobj->highlight = -1;
nobj->rotang = 0.0;
nobj->rax = nobj->ray = nobj->raz = 0.0;
nobj->rx = nobj->ry = nobj->rz = 0.0;

/* Initialize extra data
 */
if ((crf = S3GetECreateFunc()))
	nobj->extra_data = crf();

S3VSetObjectAttribs(nobj,ap);

return nobj;
}

