#include "s3dt.h"
#include "s3dtint.h"
#include "s3view.h"

/* S3AddViewLight
 * Add a new light to a view, and return the light's id for later reference.
 */
int S3AddViewLight(S3view *vw, double x, double y, double z, double intensity)
{
static int maxlid = 0;
S3light *nlt = S3Malloc(sizeof(S3light));

nlt->id        = maxlid++;
nlt->pos.x     = DblToReal(x);
nlt->pos.y     = DblToReal(y);
nlt->pos.z     = DblToReal(z);
nlt->intensity = DblToReal(intensity);

vw->lt[vw->ltc++] = nlt;
return nlt->id;
}


/* S3MoveViewLight
 * Change the position of a light source
 */
void S3MoveViewLight(S3view *vw, int num, double x, double y, double z)
{
vw->lt[num]->pos.x = DblToReal(x);
vw->lt[num]->pos.y = DblToReal(y);
vw->lt[num]->pos.z = DblToReal(z);
}


/* S3FreeViewLight
 * Free up a numbered light in a view, and shift the others down
 */
void S3FreeViewLight(S3view *vw, int num)
{
int i;

S3Free(vw->lt[num]);
for(i=num; i<vw->ltc-1; i++)
	vw->lt[i] = vw->lt[i+1];
vw->ltc--;
}

