#include "s3dt.h"
#include "s3dtint.h"

/* S3DotProd
 * Dot product of two vectors
 */
real S3DotProduct(S3point *p1, S3point *p2)
{
return AddReal3( MulReal(p1->x,p2->x), MulReal(p1->y,p2->y),
		 MulReal(p1->z,p2->z) );
}


/* S3Normalise
 * Convert a vector to unit length
 */
void S3Normalise(S3point *p)
{
real mag = SqrtReal( AddReal3( SqrReal(p->x), SqrReal(p->y), SqrReal(p->z) ) );
p->x = DivReal(p->x,mag);
p->y = DivReal(p->y,mag);
p->z = DivReal(p->z,mag);
}


/* S3MMult
 * Multiply a point by a matrix
 */
void S3MMult(S3point *p, real m[3][3], S3point *r)
{
r->x = AddReal3( MulReal(p->x,m[0][0]), MulReal(p->y,m[0][1]),
		 MulReal(p->z,m[0][2]) );
r->y = AddReal3( MulReal(p->x,m[1][0]), MulReal(p->y,m[1][1]),
		 MulReal(p->z,m[1][2]) );
r->z = AddReal3( MulReal(p->x,m[2][0]), MulReal(p->y,m[2][1]),
		 MulReal(p->z,m[2][2]) );
}


/* S3Magnitude
 * Returns the magnitude of a vector
 */
real S3Magnitude(S3point *p)
{
return SqrtReal( AddReal3( SqrReal(p->x), SqrReal(p->y), SqrReal(p->z) ) );
}


/* S3CrossProduct
 * Compute the cross product of two vectors
 */
void S3CrossProduct(S3point *p1, S3point *p2, S3point *r)
{
r->x = SubReal( MulReal(p1->y,p2->z), MulReal(p1->z,p2->y) );
r->y = SubReal( MulReal(p1->z,p2->x), MulReal(p1->x,p2->z) );
r->z = SubReal( MulReal(p1->x,p2->y), MulReal(p1->y,p2->x) );
}


/* S3Scale
 * Scale a vector by a scalar
 */
void S3Scale(S3point *p, real s)
{
p->x = MulReal(p->x,s);
p->y = MulReal(p->y,s);
p->z = MulReal(p->z,s);
}


/* S3Subtract
 * Subtract one vector from another
 */
void S3Subtract(S3point *a, S3point *b)
{
a->x = SubReal(a->x,b->x);
a->y = SubReal(a->y,b->y);
a->z = SubReal(a->z,b->z);
}


/* S3Add
 * Add one vector to another
 */
void S3Add(S3point *a, S3point *b)
{
a->x = AddReal(a->x,b->x);
a->y = AddReal(a->y,b->y);
a->z = AddReal(a->z,b->z);
}

