#include "s3dt.h"
#include "s3dtint.h"

/* S3MovePoints
 * Move all the points in an object by the given amounts
 */
void S3MovePoints(S3object *ob, double x, double y, double z)
{
int i;

for(i=0; i<ob->ptc; i++)
	S3MovePoint(ob,i,x,y,z);
}


/* S3MovePoint
 * Move one point in an object by the given amount
 */
void S3MovePoint(S3object *ob, int p, double x, double y, double z)
{
S3point *pt = ob->pt[p];
real radius;
int i;

pt->x = AddReal(pt->x,DblToReal(x));
pt->y = AddReal(pt->y,DblToReal(y));
pt->z = AddReal(pt->z,DblToReal(z));
radius = SqrtReal( AddReal3( SqrReal(pt->x), SqrReal(pt->y), SqrReal(pt->z)) );
if (RealGt(radius,ob->radius))
	ob->radius = radius;

for(i=0; i<ob->facec; i++) {
	S3polygon *poly = ob->face[i];
	int j;

	for(j=0; j<poly->vertc; j++)
		if (poly->vert[j] == p) {
			S3ComputeFace(ob,poly);
			break;
			}
	}
}


/* S3PositionPoint
 * Move a point in an object to the given absolute position
 */
void S3PositionPoint(S3object *ob, int p, double x, double y, double z)
{
double dx = x - RealToDbl(ob->pt[p]->x);
double dy = y - RealToDbl(ob->pt[p]->y);
double dz = z - RealToDbl(ob->pt[p]->z);

S3MovePoint(ob,p,dx,dy,dz);
}

