#include "s3dt.h"
#include "s3dtint.h"

static int S3HashString(char *str);

/* S3AddObjectByName 
 * Insert an object into an object table, hashed by name.
 * Note that using the same object table for id and name lookups is not
 * going to work.
 */
void S3AddObjectByName(S3objtab *tab, S3object *obj)
{
int hash;
S3object **op;
int i;

/* Link into view list
 */
obj->next = tab->head;
tab->head = obj;

if (obj->name[0]) {
	/* Hash to correct position in table
	 */
	hash = S3HashString(obj->name);
	for(op=&tab->tab[hash]; *op; op=&((*op)->hnext))
		if (strequ(obj->name,(*op)->name)) {
			fprintf(stderr,"S3AddObjectByName : attempted to add "
				"object with duplicate name : %s\n",obj->name);
			exit(10);
			}
	(*op) = obj;
	}

/* add polygons to view list
 */
for(i=0; i<obj->facec; i++)
	S3AddViewFace(tab,obj->face[i]);
}


/* S3DeleteObjectByName
 * Lookup and remove a named object from a hash table
 */
void S3DeleteObjectByName(S3objtab *tab, char *name)
{
S3object *obj;		/* object to remove */
S3object **prev;	/* pointer to pointer to it */
int hash;
int i;

/* Find object in hash table
 */
hash = S3HashString(name);
for(prev=&tab->tab[hash]; !strequ((*prev)->name,name); prev=&((*prev)->hnext))
	if (!*prev)
		return;	/* not found! */

/* Unlink from hash table
 */
obj = *prev;
*prev = obj->hnext;

/* Find and unlink from display list
 */
for(prev=&tab->head; *prev != obj; prev=&((*prev)->next))
	;
*prev = obj->next;

/* Unlink polygons from list
 */
for(i=0; i<obj->facec; i++)
	S3DelViewFace(tab,obj->face[i]);

S3FreeObject(obj);
}


/* S3LookupObjectByName
 * Lookup an object in a hash table by name. Returns a pointer to it, or
 * NULL if not found
 */
S3object *S3LookupObjectByName(S3objtab *tab, char *name)
{
int hash;
S3object *obj;

hash = S3HashString(name);
for(obj=tab->tab[hash]; obj && !strequ(obj->name,name); obj=obj->hnext)
	;
return obj;
}


/* S3AddObjectByNameAndChildren
 * Add an object and all it's children to a hash table
 */
void S3AddObjectByNameAndChildren(S3objtab *tab, S3object *obj)
{
int i;

S3AddObjectByName(tab,obj);
for(i=0; i<obj->soc; i++)
	S3AddObjectByNameAndChildren(tab,obj->so[i]);
}


/* S3DeleteObjectByNameAndChildren
 * Remove a named object and all it's children from an object table
 */
void S3DeleteObjectByNameAndChildren(S3objtab *tab, char *name)
{
int i;
S3object *obj = S3LookupObjectByName(tab,name);

for(i=0; i<obj->soc; i++)
	S3DeleteObjectByNameAndChildren(tab,obj->so[i]->name);
S3DeleteObjectByName(tab,name);
}


/* S3HashString
 * Hash a string to a position in the hash table
 */
static int S3HashString(char *str)
{
int sum = 0;

while(*str)
	sum += *(str++);
return sum % OBJTAB_SIZE;
}

