%{
#include "s3dt.h"
#include "s3dtint.h"

static int reservedword(char *s);
int yyparse(void);
static int yylex(void);
static void nextchar(void);
static void parseerror(char *s1, char *s2);
static void yyerror(char *s);

static int parse_debug = 0;

#define YYMAXDEPTH 1000
%}

%union {
	double num;
	char string[256];
	}
%token <num> NUM
%token <string> STRING
%token OBJECT,ID,NAME,RADIUS,POS,POINTS,FACE,END,EXTRA
%type <num> expr
%type <num> facetype

%%
file		:	lines END
			{ return 0;
			}
		;

lines		:	lines object
		|
		;

object		:	OBJECT STRING '{'
			{ no = S3CreateObject($2,0);
			  S3AddObject(oh,no);
			}
			objattribs '}'
		;

objattribs	:	objattribs objattrib
		|
		;

objattrib	:	ID expr
			{ no->id = $2;
			}
		|	RADIUS expr
			{ no->radius = DblToReal($2);
			}
		|	POS expr expr expr
			{ no->pos.x = DblToReal($2);
			  no->pos.y = DblToReal($3);
			  no->pos.z = DblToReal($4);
			}
		|	POINTS '{' coords '}'
		|	FACE expr facetype
			{ if (no->facec >= MAX_FACES_PER_OBJECT) {
			    parseerror("Object has too many faces",no->name);
			    return 1;
			    }
			  np = no->face[no->facec] =
			    S3Calloc(1,sizeof(S3polygon));
			  np->colour = $2;
			  np->type   = $2;
			}
			'{' vertices '}'
			{ np->vert = S3MemDup(vert_tmp,np->vertc*sizeof(int));
			  no->facec++;
			}
		|	EXTRA '{'
			{ if (!no->extra_data) {
			    parseerror("No extra data allocated","");
			    return 1;
			    }
			}
			extraattribs '}'
		;

coords		:	coords coord
		|
		;

coord		:	expr expr expr
			{ if (no->ptc >= MAX_POINTS_PER_OBJECT) {
			    parseerror("Object has too many points",no->name);
			    return 1;
			    }
			  no->pt[no->ptc] = S3Malloc(sizeof(S3point));
			  no->pt[no->ptc]->x = $1;
			  no->pt[no->ptc]->y = $2;
			  no->pt[no->ptc]->z = $3;
			  no->ptc++;
			}
		;

vertices	:	vertices vertex
		|
		;

vertex		:	expr
			{ if (np->vertc >= MAX_VERTS_PER_FACE) {
			    parseerror("polygon has too many vertices","");
			    return 1;
			    }
			  if ($1 >= no->ptc) {
			    parseerror("Point does not exist",S3Itoa($1));
			    return 1;
			    }
			  vert_tmp[np->vertc++] = $1;
			}
		;

expr		:	NUM
		;

facetype	:	expr	{ $$ = $1; }
		|	STRING
			{ switch($1[0]) {
			  case 'p':
				$$ = pt_plane; break;
			  case 'w':
				$$ = 0; break;
			  case 'g':
			 	$$ = pt_plane | pt_glass; break;
			  case 's':
				$$ = pt_plane | pt_shaded; break;
			  default:
				$$ = pt_plane;
				}
			}
		;

extraattribs	:	extraattribs extraattrib
		|
		;

extraattrib	:	STRING
			{ evcount = 0;
			}
			extravals ';'
			{ S3eadd eaf = S3GetEAddFunc();
			  int i;

			  if (!eaf) {
			    parseerror("No extra data addition function given",
				       "");
			    return 1;
			    }
			  eaf(no,$1,evlist,evcount);
			  for(i=0; i<evcount; i++)
			    free(evlist[i]);
			}
		;

extravals	:	extravals extraval
		|
		;

extraval	:	STRING
			{ if (evcount >= 30) {
			    parseerror("Too many attributes",$1);
			    return 1;
			    }
			  evlist[evcount] = malloc(strlen($1)+1);
			  strcpy(evlist[evcount],$1);
			  evcount++;
			}
		;

%%
static S3object *no;			/* object currently being read */
static S3polygon *np;			/* polygon currently being read */
static S3objtab *oh;			/* table to add objects to */
static char currch;			/* character being processed */
static char filename[256];		/* file being read */
static int lineno;			/* line number up to in file */
static int vert_tmp[MAX_VERTS_PER_FACE];/* space for new polygon vertices */
static int fd;				/* file descriptor to read from */
static char *evlist[30];		/* temp space for extra data attribs */
static int evcount;

/* yyerror
 * Function called by yacc when an error occurs in parsing
 */
static void yyerror(char *s)
{
parseerror("parse error",s);
}


/* parseerror
 * Function called when a semantic error occurs in the input data
 */
static void parseerror(char *s1, char *s2)
{
if (*s2)
	fprintf(stderr,"%s : %s\nNear line %d in %s\n",s1,s2,lineno,filename);
else
	fprintf(stderr,"%s\nNear line %d in %s\n",s1,s2,lineno,filename);
}


/* nextchar
 * Read one character from the input fd
 */
static void nextchar(void)
{
read(fd,&currch,1);
if (currch == '\n')
	lineno++;
}


/* yylex
 * Read one token
 */
static int yylex(void)
{
char tok[256] = {0};
char *tp = tok;

/* skip whitespace
 */
while(currch == ' ' || currch == '\t' || currch == '\n') {
	nextchar();
	}

if (isdigit(currch) || currch == '.') {
	/* A number
	 */
	double num;

	while(isdigit(currch) || currch == '.') {
		*tp++ = currch;
		nextchar();
		}
	yylval.num = atof(tok);
	if (parse_debug)
		printf("num : %f\n",yylval.num);
	return NUM;
	}
else if (currch == '\"') {
	/* A quoted string
	 */
	nextchar();
	while(currch != '\"') {
		*tp++ = currch;
		nextchar();
		}
	nextchar();
	strcpy(yylval.string,tok);
	if (parse_debug)
		printf("string : %s\n",yylval.string);
	return STRING;
	}
else if (isalpha(currch)) {
	/* An alphanumeric string
	 */
	int rw;

	while(isdigit(currch) || isalpha(currch) || currch == '_') {
		*tp++ = currch;
		nextchar();
		}
	if ((rw = reservedword(tok)) != -1) {
		if (parse_debug)
			printf("reserved : %d = %s\n",rw,tok);
		return rw;
		}
	strcpy(yylval.string,tok);
	if (parse_debug)
		printf("string : %s\n",yylval.string);
	return STRING;
	}
else if (currch == '#') {
	/* cpp file info line
	 */
	char line[256];		/* line starting with '#' */
	char *lnum;		/* real line number */
	char *fnam;		/* real file name */
	int i=0;

	while(currch != '\n') {
		nextchar();
		line[i++] = currch;
		}
	lnum = strtok(line," \n");
	fnam = strtok(NULL," \n");
	lineno = atoi(lnum);
	strcpy(filename,fnam);
	nextchar();
	return yylex();		/* call again to get next token */
	}
else if (currch == -1) {
	/* end of file
	 */
	return END;
	}
else {
	/* some other character.
	 */
	int ch = currch;

	nextchar();
	if (parse_debug)
		printf("char : %c\n",ch);
	return ch;
	}
}


/* reservedword
 * Return the token for reserved word, or -1 if no match
 */
static int reservedword(char *s)
{
int i;
struct {
	char *word;
	int tok;
	} rword[] = {	"object",OBJECT,"id",ID,"name",NAME,"radius",RADIUS,
			"pos",POS,"points",POINTS,"face",FACE,"end",END,
			"extra",EXTRA };

for(i=0; i<sizeof(rword)/sizeof(rword[0]); i++)
	if (!strcmp(rword[i].word,s))
		return rword[i].tok;
return -1;
}


/* S3ParseFd
 * Parse an object description file from a file descriptor.
 * Returns 0 if the parse was successful, or 1 if not.
 */
int S3ParseFd(int filed, S3objtab *ohead)
{
oh = ohead;
fd = filed;
currch = ' ';
strcpy(filename,"unknown");
lineno = 0;
return yyparse();
}


/* S3ParseFile
 * Opens a file and reads s3dt object descriptions from it.
 * Returns 0 if successful, or 1 if an error occurs.
 */
int S3ParseFile(char *fname, S3objtab *ohead)
{
if ((fd = open(fname,O_RDONLY)) == -1)
	return 1;
oh = ohead;
currch = ' ';
strcpy(filename,"fname");
return yyparse();
}

