#include "s3dt.h"
#include "s3dtint.h"

/* S3ScaleObject
 * Scale an object about 0,0,0 in all directions by the given scaling factor
 */
void S3ScaleObject(S3object *ob, double sc)
{
S3ScaleObject3(ob,sc,sc,sc);
}


/* S3ScaleObject3
 * Scale an object about 0,0,0 by a varying amount in along each axis
 */
void S3ScaleObject3(S3object *ob, double x, double y, double z)
{
real rx = DblToReal(x);
real ry = DblToReal(y);
real rz = DblToReal(z);
double px,py,pz;	/* object pos */
int i;

/* Scale points
 */
for(i=0; i<ob->ptc; i++) {
	S3point *pt = ob->pt[i];

	pt->x = MulReal(pt->x,rx);
	pt->y = MulReal(pt->y,ry);
	pt->z = MulReal(pt->z,rz);
	}
for(i=0; i<ob->facec; i++) {
	S3ComputeFace(ob,ob->face[i]);
	}

/* Scale child objects about the centre of this one
 */
S3GetObjectPos(ob,&px,&py,&pz);
for(i=0; i<ob->soc; i++) 
	S3ScaleObjectAbout3(ob->so[i],px,py,pz,x,y,z);
}


/* S3ScaleObjectAbout
 * Scale an object by the same amount in all axis about some point
 */
void S3ScaleObjectAbout(S3object *ob, double cx, double cy, double cz,
				      double sc)
{
S3ScaleObjectAbout3(ob,cx,cy,cz,sc,sc,sc);
}


/* S3ScaleObjectAbout3
 * Scale an object about an arbitrary point in 3d world space
 */
void S3ScaleObjectAbout3(S3object *ob, double cx, double cy, double cz,
				       double x, double y, double z)
{
real rx = DblToReal(x), ry = DblToReal(y), rz = DblToReal(z);
real crx = DblToReal(cx), cry = DblToReal(cy), crz = DblToReal(cz);
int i;

/* Scale points
 */
for(i=0; i<ob->ptc; i++) {
	S3point *pt = ob->pt[i];
	real xx,yy,zz;	/* position relative to cx,cy,cz */

	xx = SubReal(AddReal(ob->pos.x,pt->x),crx);
	yy = SubReal(AddReal(ob->pos.y,pt->y),cry);
	zz = SubReal(AddReal(ob->pos.z,pt->z),crz);
	pt->x = SubReal(AddReal(MulReal(xx,rx),crx),ob->pos.x);
	pt->y = SubReal(AddReal(MulReal(yy,ry),cry),ob->pos.y);
	pt->z = SubReal(AddReal(MulReal(zz,rz),crz),ob->pos.z);
	}
for(i=0; i<ob->facec; i++) {
	S3ComputeFace(ob,ob->face[i]);
	}

/* Scale child objects as well
 */
for(i=0; i<ob->soc; i++)
	S3ScaleObjectAbout3(ob->so[i],cx,cy,cz,x,y,z);
}

