#include "s3dt.h"
#include "s3dtint.h"

static void S3FdPrintf(int fd, char *fmt, ...);
static void S3WriteExtraAttrib(int fd, char *attr, char **vals, int valc);

/* S3FdPrintf
 * Write a formatted string of characters to a file descriptor
 */
static void S3FdPrintf(int fd, char *fmt, ...)
{
va_list ap;
char buf[1024];
int len;

va_start(ap,fmt);
#ifdef ODD_VSPRINTF
vsprintf(buf,fmt,ap);
len = strlen(buf);
#else
len = vsprintf(buf,fmt,ap);
#endif
write(fd,buf,len);
va_end(ap);
}


/* S3WriteFd
 * Output a table of objects to a file descriptor
 */
void S3WriteFd(int fd, S3objtab *oh)
{
S3object *o;

for(o=oh->head; o; o=o->next) {
	S3ewrite ewf;		/* user function to write extra data */
	int i;

	S3FdPrintf(fd,"object \"%s\" {\n",o->name);
	S3FdPrintf(fd,"\tid %d\n",o->id);
	S3FdPrintf(fd,"\tradius %f\n",o->radius);
	S3FdPrintf(fd,"\tpos %f %f %f\n",o->pos.x,o->pos.x,o->pos.z);
	S3FdPrintf(fd,"\tpoints {\n");

	/* Output points
	 */
	for(i=0; i<o->ptc; i++)
		S3FdPrintf(fd,"\t%f %f %f\n",o->pt[i]->x,o->pt[i]->y,
			   o->pt[i]->z);
	S3FdPrintf(fd,"\t}\n");

	/* Output faces
	 */
	for(i=0; i<o->facec; i++) {
		S3polygon *f = o->face[i];
		int j;

		S3FdPrintf(fd,"face %d %d {",f->colour,f->type);
		for(j=0; j<f->vertc; j++)
			S3FdPrintf(fd," %d",f->vert[j]);
		S3FdPrintf(fd," }\n");
		}

	/* Output extra data. As the extra-data format must be written
	 * in the format:
	 * <attribute> <value>* ';'
	 * To enforce this, a user function is called and passed another
	 * function for writing out data in the correct format. :)
	 */
	if (o->extra_data && (ewf = S3GetEWriteFunc())) {
		S3FdPrintf(fd,"extra {\n");
		ewf(fd,o->extra_data,S3WriteExtraAttrib);
		S3FdPrintf(fd,"\t}\n");
		}
	}
S3FdPrintf(fd,"end\n");
}


/* S3WriteExtraAttrib
 * Write an extra_data attribute to a file descriptor.
 * All output of extra data should go through this function, to ensure
 * the correct format.
 */
static void S3WriteExtraAttrib(int fd, char *attr, char **vals, int valc)
{
int i;

S3FdPrintf(fd,"\t\t%s ",attr);
for(i=0; i<valc; i++)
	S3FdPrintf(fd,"%s ",vals[i]);
S3FdPrintf(fd,";\n");
}

