
# line 2 "S3Parser.y"
#include "s3dt.h"
#include "s3dtint.h"

static int reservedword(char *s);
int yyparse(void);
static int yylex(void);
static void nextchar(void);
static void parseerror(char *s1, char *s2);
static void yyerror(char *s);

static int parse_debug = 0;

#define YYMAXDEPTH 1000

# line 17 "S3Parser.y"
typedef union  {
	double num;
	char string[256];
	} YYSTYPE;
#ifdef __cplusplus
#  include <stdio.h>
#  include <yacc.h>
#endif	/* __cplusplus */ 
# define NUM 257
# define STRING 258
# define OBJECT 259
# define ID 260
# define NAME 261
# define RADIUS 262
# define POS 263
# define POINTS 264
# define FACE 265
# define END 266
# define EXTRA 267
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif

/* __YYSCLASS defines the scoping/storage class for global objects
 * that are NOT renamed by the -p option.  By default these names
 * are going to be 'static' so that multi-definition errors
 * will not occur with multiple parsers.
 * If you want (unsupported) access to internal names you need
 * to define this to be null so it implies 'extern' scope.
 * This should not be used in conjunction with -p.
 */
#ifndef __YYSCLASS
# define __YYSCLASS static
#endif
YYSTYPE yylval;
__YYSCLASS YYSTYPE yyval;
typedef int yytabelem;
# define YYERRCODE 256

# line 174 "S3Parser.y"

static S3object *no;			/* object currently being read */
static S3polygon *np;			/* polygon currently being read */
static S3objtab *oh;			/* table to add objects to */
static char currch;			/* character being processed */
static char filename[256];		/* file being read */
static int lineno;			/* line number up to in file */
static int vert_tmp[MAX_VERTS_PER_FACE];/* space for new polygon vertices */
static int fd;				/* file descriptor to read from */
static char *evlist[30];		/* temp space for extra data attribs */
static int evcount;

/* yyerror
 * Function called by yacc when an error occurs in parsing
 */
static void yyerror(char *s)
{
parseerror("parse error",s);
}


/* parseerror
 * Function called when a semantic error occurs in the input data
 */
static void parseerror(char *s1, char *s2)
{
if (*s2)
	fprintf(stderr,"%s : %s\nNear line %d in %s\n",s1,s2,lineno,filename);
else
	fprintf(stderr,"%s\nNear line %d in %s\n",s1,s2,lineno,filename);
}


/* nextchar
 * Read one character from the input fd
 */
static void nextchar(void)
{
read(fd,&currch,1);
if (currch == '\n')
	lineno++;
}


/* yylex
 * Read one token
 */
static int yylex(void)
{
char tok[256] = {0};
char *tp = tok;

/* skip whitespace
 */
while(currch == ' ' || currch == '\t' || currch == '\n') {
	nextchar();
	}

if (isdigit(currch) || currch == '.') {
	/* A number
	 */
	double num;

	while(isdigit(currch) || currch == '.') {
		*tp++ = currch;
		nextchar();
		}
	yylval.num = atof(tok);
	if (parse_debug)
		printf("num : %f\n",yylval.num);
	return NUM;
	}
else if (currch == '\"') {
	/* A quoted string
	 */
	nextchar();
	while(currch != '\"') {
		*tp++ = currch;
		nextchar();
		}
	nextchar();
	strcpy(yylval.string,tok);
	if (parse_debug)
		printf("string : %s\n",yylval.string);
	return STRING;
	}
else if (isalpha(currch)) {
	/* An alphanumeric string
	 */
	int rw;

	while(isdigit(currch) || isalpha(currch) || currch == '_') {
		*tp++ = currch;
		nextchar();
		}
	if ((rw = reservedword(tok)) != -1) {
		if (parse_debug)
			printf("reserved : %d = %s\n",rw,tok);
		return rw;
		}
	strcpy(yylval.string,tok);
	if (parse_debug)
		printf("string : %s\n",yylval.string);
	return STRING;
	}
else if (currch == '#') {
	/* cpp file info line
	 */
	char line[256];		/* line starting with '#' */
	char *lnum;		/* real line number */
	char *fnam;		/* real file name */
	int i=0;

	while(currch != '\n') {
		nextchar();
		line[i++] = currch;
		}
	lnum = strtok(line," \n");
	fnam = strtok(NULL," \n");
	lineno = atoi(lnum);
	strcpy(filename,fnam);
	nextchar();
	return yylex();		/* call again to get next token */
	}
else if (currch == -1) {
	/* end of file
	 */
	return END;
	}
else {
	/* some other character.
	 */
	int ch = currch;

	nextchar();
	if (parse_debug)
		printf("char : %c\n",ch);
	return ch;
	}
}


/* reservedword
 * Return the token for reserved word, or -1 if no match
 */
static int reservedword(char *s)
{
int i;
struct {
	char *word;
	int tok;
	} rword[] = {	"object",OBJECT,"id",ID,"name",NAME,"radius",RADIUS,
			"pos",POS,"points",POINTS,"face",FACE,"end",END,
			"extra",EXTRA };

for(i=0; i<sizeof(rword)/sizeof(rword[0]); i++)
	if (!strcmp(rword[i].word,s))
		return rword[i].tok;
return -1;
}


/* S3ParseFd
 * Parse an object description file from a file descriptor.
 * Returns 0 if the parse was successful, or 1 if not.
 */
int S3ParseFd(int filed, S3objtab *ohead)
{
oh = ohead;
fd = filed;
currch = ' ';
strcpy(filename,"unknown");
lineno = 0;
return yyparse();
}


/* S3ParseFile
 * Opens a file and reads s3dt object descriptions from it.
 * Returns 0 if successful, or 1 if an error occurs.
 */
int S3ParseFile(char *fname, S3objtab *ohead)
{
if ((fd = open(fname,O_RDONLY)) == -1)
	return 1;
oh = ohead;
currch = ' ';
strcpy(filename,"fname");
return yyparse();
}

__YYSCLASS yytabelem yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
	};
# define YYNPROD 32
# define YYLAST 202
__YYSCLASS yytabelem yyact[]={

    10,     5,    49,    39,    19,    29,     6,    19,     3,    45,
    38,    32,    24,    22,     7,    18,    50,    48,    44,    40,
    46,    33,    36,    30,    43,    35,    26,    11,     9,    20,
    21,     8,    23,     4,     2,     1,    27,    25,     0,    28,
     0,    31,    34,     0,     0,     0,     0,     0,     0,     0,
    37,     0,     0,    42,     0,     0,     0,     0,     0,    47,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,    12,    41,    13,    14,    15,
    16,    19,    17,    19,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,    51 };
__YYSCLASS yytabelem yypact[]={

 -3000, -3000,  -258, -3000, -3000,  -252,  -109, -3000, -3000,  -125,
 -3000, -3000,  -250,  -250,  -250,  -110,  -250,  -111, -3000, -3000,
 -3000,  -250, -3000,  -253, -3000,  -250,  -114, -3000, -3000, -3000,
 -3000, -3000, -3000, -3000,  -250,  -113,  -122,  -250, -3000, -3000,
 -3000, -3000, -3000,  -116, -3000, -3000, -3000, -3000,   -57, -3000,
 -3000, -3000 };
__YYSCLASS yytabelem yypgo[]={

     0,    15,    36,    35,    34,    33,    31,    28,    27,    26,
    25,    24,    23,    22,    21,    20,    19,    18,    17,    16 };
__YYSCLASS yytabelem yyr1[]={

     0,     3,     4,     4,     6,     5,     7,     7,     8,     8,
     8,     8,    10,     8,    12,     8,     9,     9,    14,    11,
    11,    15,     1,     2,     2,    13,    13,    17,    16,    18,
    18,    19 };
__YYSCLASS yytabelem yyr2[]={

     0,     5,     4,     0,     1,    12,     4,     0,     5,     5,
     9,     8,     1,    15,     1,    10,     4,     0,     7,     4,
     0,     3,     2,     3,     3,     4,     0,     1,     9,     4,
     0,     3 };
__YYSCLASS yytabelem yychk[]={

 -3000,    -3,    -4,   266,    -5,   259,   258,   123,    -6,    -7,
   125,    -8,   260,   262,   263,   264,   265,   267,    -1,   257,
    -1,    -1,   123,    -1,   123,    -1,    -9,    -2,    -1,   258,
   -12,    -1,   125,   -14,    -1,   -10,   -13,    -1,   123,   125,
   -16,   258,    -1,   -11,   -17,   125,   -15,    -1,   -18,    59,
   -19,   258 };
__YYSCLASS yytabelem yydef[]={

     3,    -2,     0,     1,     2,     0,     0,     4,     7,     0,
     5,     6,     0,     0,     0,     0,     0,     0,     8,    22,
     9,     0,    17,     0,    14,     0,     0,    12,    23,    24,
    26,    10,    11,    16,     0,     0,     0,     0,    20,    15,
    25,    27,    18,     0,    30,    13,    19,    21,     0,    28,
    29,    31 };
typedef struct { char *t_name; int t_val; } yytoktype;
#ifndef YYDEBUG
#	define YYDEBUG	0	/* don't allow debugging */
#endif

#if YYDEBUG

__YYSCLASS yytoktype yytoks[] =
{
	"NUM",	257,
	"STRING",	258,
	"OBJECT",	259,
	"ID",	260,
	"NAME",	261,
	"RADIUS",	262,
	"POS",	263,
	"POINTS",	264,
	"FACE",	265,
	"END",	266,
	"EXTRA",	267,
	"-unknown-",	-1	/* ends search */
};

__YYSCLASS char * yyreds[] =
{
	"-no such reduction-",
	"file : lines END",
	"lines : lines object",
	"lines : /* empty */",
	"object : OBJECT STRING '{'",
	"object : OBJECT STRING '{' objattribs '}'",
	"objattribs : objattribs objattrib",
	"objattribs : /* empty */",
	"objattrib : ID expr",
	"objattrib : RADIUS expr",
	"objattrib : POS expr expr expr",
	"objattrib : POINTS '{' coords '}'",
	"objattrib : FACE expr facetype",
	"objattrib : FACE expr facetype '{' vertices '}'",
	"objattrib : EXTRA '{'",
	"objattrib : EXTRA '{' extraattribs '}'",
	"coords : coords coord",
	"coords : /* empty */",
	"coord : expr expr expr",
	"vertices : vertices vertex",
	"vertices : /* empty */",
	"vertex : expr",
	"expr : NUM",
	"facetype : expr",
	"facetype : STRING",
	"extraattribs : extraattribs extraattrib",
	"extraattribs : /* empty */",
	"extraattrib : STRING",
	"extraattrib : STRING extravals ';'",
	"extravals : extravals extraval",
	"extravals : /* empty */",
	"extraval : STRING",
};
#endif /* YYDEBUG */
#define YYFLAG  (-3000)
/* @(#) $Revision: 70.7 $ */    

/*
** Skeleton parser driver for yacc output
*/

#if defined(NLS) && !defined(NL_SETN)
#include <msgbuf.h>
#endif

#ifndef nl_msg
#define nl_msg(i,s) (s)
#endif

/*
** yacc user known macros and defines
*/
#define YYERROR		goto yyerrlab

#ifndef __RUNTIME_YYMAXDEPTH
#define YYACCEPT	return(0)
#define YYABORT		return(1)
#else
#define YYACCEPT	{free_stacks(); return(0);}
#define YYABORT		{free_stacks(); return(1);}
#endif

#define YYBACKUP( newtoken, newvalue )\
{\
	if ( yychar >= 0 || ( yyr2[ yytmp ] >> 1 ) != 1 )\
	{\
		yyerror( (nl_msg(30001,"syntax error - cannot backup")) );\
		goto yyerrlab;\
	}\
	yychar = newtoken;\
	yystate = *yyps;\
	yylval = newvalue;\
	goto yynewstate;\
}
#define YYRECOVERING()	(!!yyerrflag)
#ifndef YYDEBUG
#	define YYDEBUG	1	/* make debugging available */
#endif

/*
** user known globals
*/

#ifdef YYREENTRANT
__thread int yydebug;		/* set to 1 to get debugging */
#else
int yydebug;			/* set to 1 to get debugging */
#endif

/*
** driver internal defines
*/
/* define for YYFLAG now generated by yacc program. */
/*#define YYFLAG		(FLAGVAL)*/

/*
** global variables used by the parser
*/
# ifndef __RUNTIME_YYMAXDEPTH

#   ifdef YYREENTRANT
__thread __YYSCLASS YYSTYPE yyv[ YYMAXDEPTH ];  /* value stack */
__thread __YYSCLASS int yys[ YYMAXDEPTH ];      /* state stack */
#   else
__YYSCLASS YYSTYPE yyv[ YYMAXDEPTH ];           /* value stack */
__YYSCLASS int yys[ YYMAXDEPTH ];               /* state stack */
#   endif

# else

#  ifdef YYREENTRANT
__thread __YYSCLASS YYSTYPE *yyv;               /* pointer to malloc'ed stack st
value stack */
__thread __YYSCLASS int *yys;                   /* pointer to malloc'ed stack st
ack */
#  else
__YYSCLASS YYSTYPE *yyv;                        /* pointer to malloc'ed value st
ack */
__YYSCLASS int *yys;                            /* pointer to malloc'ed stack stack */
#  endif

#if defined(__STDC__) || defined (__cplusplus)
#include <stdlib.h>
#else
	extern char *malloc();
	extern char *realloc();
	extern void free();
#endif /* __STDC__ or __cplusplus */


static int allocate_stacks(); 
static void free_stacks();
# ifndef YYINCREMENT
# define YYINCREMENT (YYMAXDEPTH/2) + 10
# endif
# endif	/* __RUNTIME_YYMAXDEPTH */
long  yymaxdepth = YYMAXDEPTH;


#ifdef YYREENTRANT

__thread __YYSCLASS YYSTYPE *yypv;      /* top of value stack */
__thread __YYSCLASS int *yyps;          /* top of state stack */

__thread __YYSCLASS int yystate;        /* current state */
__thread __YYSCLASS int yytmp;          /* extra var (lasts between blocks) */

__thread int yynerrs;                   /* number of errors */
__thread __YYSCLASS int yyerrflag;      /* error recovery flag */
__thread int yychar;                    /* current input token number */

__thread extern int yyinit_key;           /* init TLS data once */
extern int TLS_INIT; 

#else

__YYSCLASS YYSTYPE *yypv;               /* top of value stack */
__YYSCLASS int *yyps;                   /* top of state stack */

__YYSCLASS int yystate;                 /* current state */
__YYSCLASS int yytmp;                   /* extra var (lasts between blocks) */

int yynerrs;                            /* number of errors */
__YYSCLASS int yyerrflag;               /* error recovery flag */
int yychar;
#endif


/*
** yyparse - return 0 if worked, 1 if syntax error not recovered from
l*/
int
yyparse()
{
	register YYSTYPE *yypvt;	/* top of value stack for $vars */


        /*
        ** Initialize externals - yyparse may be called more than once
        */

#ifdef YYREENTRANT
        if (yyinit_key != TLS_INIT)
        {
            yyinit_key = TLS_INIT; 
            yyinit_tls();
        }
#endif

	/*
	** Initialize externals - yyparse may be called more than once
	*/
# ifdef __RUNTIME_YYMAXDEPTH
	if (allocate_stacks()) YYABORT;
# endif
	yypv = &yyv[-1];
	yyps = &yys[-1];
	yystate = 0;
	yytmp = 0;
	yynerrs = 0;
	yyerrflag = 0;
	yychar = -1;

	goto yystack;
	{
		register YYSTYPE *yy_pv;	/* top of value stack */
		register int *yy_ps;		/* top of state stack */
		register int yy_state;		/* current state */
		register int  yy_n;		/* internal state number info */

		/*
		** get globals into registers.
		** branch to here only if YYBACKUP was called.
		*/
	yynewstate:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;
		goto yy_newstate;

		/*
		** get globals into registers.
		** either we just started, or we just finished a reduction
		*/
	yystack:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;

		/*
		** top of for (;;) loop while no reductions done
		*/
	yy_stack:
		/*
		** put a state and value onto the stacks
		*/
#if YYDEBUG
		/*
		** if debugging, look up token value in list of value vs.
		** name pairs.  0 and negative (-1) are special values.
		** Note: linear search is used since time is not a real
		** consideration while debugging.
		*/
		if ( yydebug )
		{
			register int yy_i;

			printf( "State %d, token ", yy_state );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ++yy_ps >= &yys[ yymaxdepth ] )	/* room on stack? */
		{
# ifndef __RUNTIME_YYMAXDEPTH
			yyerror( (nl_msg(30002,"yacc stack overflow")) );
			YYABORT;
# else
			/* save old stack bases to recalculate pointers */
			YYSTYPE * yyv_old = yyv;
			int * yys_old = yys;
			yymaxdepth += YYINCREMENT;
			yys = (int *) realloc(yys, yymaxdepth * sizeof(int));
			yyv = (YYSTYPE *) realloc(yyv, yymaxdepth * sizeof(YYSTYPE));
			if (yys==0 || yyv==0) {
			    yyerror( (nl_msg(30002,"yacc stack overflow")) );
			    YYABORT;
			    }
			/* Reset pointers into stack */
			yy_ps = (yy_ps - yys_old) + yys;
			yyps = (yyps - yys_old) + yys;
			yy_pv = (yy_pv - yyv_old) + yyv;
			yypv = (yypv - yyv_old) + yyv;
# endif

		}
		*yy_ps = yy_state;
		*++yy_pv = yyval;

		/*
		** we have a new state - find out what to do
		*/
	yy_newstate:
		if ( ( yy_n = yypact[ yy_state ] ) <= YYFLAG )
			goto yydefault;		/* simple state */
#if YYDEBUG
		/*
		** if debugging, need to mark whether new token grabbed
		*/
		yytmp = yychar < 0;
#endif
		if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
			yychar = 0;		/* reached EOF */
#if YYDEBUG
		if ( yydebug && yytmp )
		{
			register int yy_i;

			printf( "Received token " );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ( ( yy_n += yychar ) < 0 ) || ( yy_n >= YYLAST ) )
			goto yydefault;
		if ( yychk[ yy_n = yyact[ yy_n ] ] == yychar )	/*valid shift*/
		{
			yychar = -1;
			yyval = yylval;
			yy_state = yy_n;
			if ( yyerrflag > 0 )
				yyerrflag--;
			goto yy_stack;
		}

	yydefault:
		if ( ( yy_n = yydef[ yy_state ] ) == -2 )
		{
#if YYDEBUG
			yytmp = yychar < 0;
#endif
			if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
				yychar = 0;		/* reached EOF */
#if YYDEBUG
			if ( yydebug && yytmp )
			{
				register int yy_i;

				printf( "Received token " );
				if ( yychar == 0 )
					printf( "end-of-file\n" );
				else if ( yychar < 0 )
					printf( "-none-\n" );
				else
				{
					for ( yy_i = 0;
						yytoks[yy_i].t_val >= 0;
						yy_i++ )
					{
						if ( yytoks[yy_i].t_val
							== yychar )
						{
							break;
						}
					}
					printf( "%s\n", yytoks[yy_i].t_name );
				}
			}
#endif /* YYDEBUG */
			/*
			** look through exception table
			*/
			{
				register int *yyxi = yyexca;

				while ( ( *yyxi != -1 ) ||
					( yyxi[1] != yy_state ) )
				{
					yyxi += 2;
				}
				while ( ( *(yyxi += 2) >= 0 ) &&
					( *yyxi != yychar ) )
					;
				if ( ( yy_n = yyxi[1] ) < 0 )
					YYACCEPT;
			}
		}

		/*
		** check for syntax error
		*/
		if ( yy_n == 0 )	/* have an error */
		{
			/* no worry about speed here! */
			switch ( yyerrflag )
			{
			case 0:		/* new error */
				yyerror( (nl_msg(30003,"syntax error")) );
				yynerrs++;
				goto skip_init;
			yyerrlab:
				/*
				** get globals into registers.
				** we have a user generated syntax type error
				*/
				yy_pv = yypv;
				yy_ps = yyps;
				yy_state = yystate;
				yynerrs++;
			skip_init:
			case 1:
			case 2:		/* incompletely recovered error */
					/* try again... */
				yyerrflag = 3;
				/*
				** find state where "error" is a legal
				** shift action
				*/
				while ( yy_ps >= yys )
				{
					yy_n = yypact[ *yy_ps ] + YYERRCODE;
					if ( yy_n >= 0 && yy_n < YYLAST &&
						yychk[yyact[yy_n]] == YYERRCODE)					{
						/*
						** simulate shift of "error"
						*/
						yy_state = yyact[ yy_n ];
						goto yy_stack;
					}
					/*
					** current state has no shift on
					** "error", pop stack
					*/
#if YYDEBUG
#	define _POP_ "Error recovery pops state %d, uncovers state %d\n"
					if ( yydebug )
						printf( _POP_, *yy_ps,
							yy_ps[-1] );
#	undef _POP_
#endif
					yy_ps--;
					yy_pv--;
				}
				/*
				** there is no state on stack with "error" as
				** a valid shift.  give up.
				*/
				YYABORT;
			case 3:		/* no shift yet; eat a token */
#if YYDEBUG
				/*
				** if debugging, look up token in list of
				** pairs.  0 and negative shouldn't occur,
				** but since timing doesn't matter when
				** debugging, it doesn't hurt to leave the
				** tests here.
				*/
				if ( yydebug )
				{
					register int yy_i;

					printf( "Error recovery discards " );
					if ( yychar == 0 )
						printf( "token end-of-file\n" );
					else if ( yychar < 0 )
						printf( "token -none-\n" );
					else
					{
						for ( yy_i = 0;
							yytoks[yy_i].t_val >= 0;
							yy_i++ )
						{
							if ( yytoks[yy_i].t_val
								== yychar )
							{
								break;
							}
						}
						printf( "token %s\n",
							yytoks[yy_i].t_name );
					}
				}
#endif /* YYDEBUG */
				if ( yychar == 0 )	/* reached EOF. quit */
					YYABORT;
				yychar = -1;
				goto yy_newstate;
			}
		}/* end if ( yy_n == 0 ) */
		/*
		** reduction by production yy_n
		** put stack tops, etc. so things right after switch
		*/
#if YYDEBUG
		/*
		** if debugging, print the string that is the user's
		** specification of the reduction which is just about
		** to be done.
		*/
		if ( yydebug )
			printf( "Reduce by (%d) \"%s\"\n",
				yy_n, yyreds[ yy_n ] );
#endif
		yytmp = yy_n;			/* value to switch over */
		yypvt = yy_pv;			/* $vars top of value stack */
		/*
		** Look in goto table for next state
		** Sorry about using yy_state here as temporary
		** register variable, but why not, if it works...
		** If yyr2[ yy_n ] doesn't have the low order bit
		** set, then there is no action to be done for
		** this reduction.  So, no saving & unsaving of
		** registers done.  The only difference between the
		** code just after the if and the body of the if is
		** the goto yy_stack in the body.  This way the test
		** can be made before the choice of what to do is needed.
		*/
		{
			/* length of production doubled with extra bit */
			register int yy_len = yyr2[ yy_n ];

			if ( !( yy_len & 01 ) )
			{
				yy_len >>= 1;
				yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
				yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
					*( yy_ps -= yy_len ) + 1;
				if ( yy_state >= YYLAST ||
					yychk[ yy_state =
					yyact[ yy_state ] ] != -yy_n )
				{
					yy_state = yyact[ yypgo[ yy_n ] ];
				}
				goto yy_stack;
			}
			yy_len >>= 1;
			yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
			yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
				*( yy_ps -= yy_len ) + 1;
			if ( yy_state >= YYLAST ||
				yychk[ yy_state = yyact[ yy_state ] ] != -yy_n )
			{
				yy_state = yyact[ yypgo[ yy_n ] ];
			}
		}
					/* save until reenter driver code */
		yystate = yy_state;
		yyps = yy_ps;
		yypv = yy_pv;
	}
	/*
	** code supplied by user is placed in this switch
	*/
	switch( yytmp )
	{
		
case 1:
# line 29 "S3Parser.y"
{ return 0;
			} break;
case 4:
# line 38 "S3Parser.y"
{ no = S3CreateObject(yypvt[-1].string,0);
			  S3AddObject(oh,no);
			} break;
case 8:
# line 49 "S3Parser.y"
{ no->id = yypvt[-0].num;
			} break;
case 9:
# line 52 "S3Parser.y"
{ no->radius = DblToReal(yypvt[-0].num);
			} break;
case 10:
# line 55 "S3Parser.y"
{ no->pos.x = DblToReal(yypvt[-2].num);
			  no->pos.y = DblToReal(yypvt[-1].num);
			  no->pos.z = DblToReal(yypvt[-0].num);
			} break;
case 12:
# line 61 "S3Parser.y"
{ if (no->facec >= MAX_FACES_PER_OBJECT) {
			    parseerror("Object has too many faces",no->name);
			    return 1;
			    }
			  np = no->face[no->facec] =
			    S3Calloc(1,sizeof(S3polygon));
			  np->colour = yypvt[-1].num;
			  np->type   = yypvt[-1].num;
			} break;
case 13:
# line 71 "S3Parser.y"
{ np->vert = S3MemDup(vert_tmp,np->vertc*sizeof(int));
			  no->facec++;
			} break;
case 14:
# line 75 "S3Parser.y"
{ if (!no->extra_data) {
			    parseerror("No extra data allocated","");
			    return 1;
			    }
			} break;
case 18:
# line 88 "S3Parser.y"
{ if (no->ptc >= MAX_POINTS_PER_OBJECT) {
			    parseerror("Object has too many points",no->name);
			    return 1;
			    }
			  no->pt[no->ptc] = S3Malloc(sizeof(S3point));
			  no->pt[no->ptc]->x = yypvt[-2].num;
			  no->pt[no->ptc]->y = yypvt[-1].num;
			  no->pt[no->ptc]->z = yypvt[-0].num;
			  no->ptc++;
			} break;
case 21:
# line 105 "S3Parser.y"
{ if (np->vertc >= MAX_VERTS_PER_FACE) {
			    parseerror("polygon has too many vertices","");
			    return 1;
			    }
			  if (yypvt[-0].num >= no->ptc) {
			    parseerror("Point does not exist",S3Itoa(yypvt[-0].num));
			    return 1;
			    }
			  vert_tmp[np->vertc++] = yypvt[-0].num;
			} break;
case 23:
# line 120 "S3Parser.y"
{ yyval.num = yypvt[-0].num; } break;
case 24:
# line 122 "S3Parser.y"
{ switch(yypvt[-0].string[0]) {
			  case 'p':
				yyval.num = pt_plane; break;
			  case 'w':
				yyval.num = 0; break;
			  case 'g':
			 	yyval.num = pt_plane | pt_glass; break;
			  case 's':
				yyval.num = pt_plane | pt_shaded; break;
			  default:
				yyval.num = pt_plane;
				}
			} break;
case 27:
# line 142 "S3Parser.y"
{ evcount = 0;
			} break;
case 28:
# line 145 "S3Parser.y"
{ S3eadd eaf = S3GetEAddFunc();
			  int i;

			  if (!eaf) {
			    parseerror("No extra data addition function given",
				       "");
			    return 1;
			    }
			  eaf(no,yypvt[-3].string,evlist,evcount);
			  for(i=0; i<evcount; i++)
			    free(evlist[i]);
			} break;
case 31:
# line 164 "S3Parser.y"
{ if (evcount >= 30) {
			    parseerror("Too many attributes",yypvt[-0].string);
			    return 1;
			    }
			  evlist[evcount] = malloc(strlen(yypvt[-0].string)+1);
			  strcpy(evlist[evcount],yypvt[-0].string);
			  evcount++;
			} break;
	}
	goto yystack;		/* reset registers in driver code */
}

# ifdef __RUNTIME_YYMAXDEPTH

static int allocate_stacks() {
	/* allocate the yys and yyv stacks */
	yys = (int *) malloc(yymaxdepth * sizeof(int));
	yyv = (YYSTYPE *) malloc(yymaxdepth * sizeof(YYSTYPE));

	if (yys==0 || yyv==0) {
	   yyerror( (nl_msg(30004,"unable to allocate space for yacc stacks")) );
	   return(1);
	   }
	else return(0);

}


static void free_stacks() {
	if (yys!=0) free((char *) yys);
	if (yyv!=0) free((char *) yyv);
}

# endif  /* defined(__RUNTIME_YYMAXDEPTH) */

