/* texture.c
 *
 * Functions for loading and encoding textures
 */
#include "sim.h"

static char *get_ppm_word(int fd);

/* read_ppm_texture
 * Read raw ppm data from a file descriptor
 */
unsigned char *read_ppm_texture(int fd, int *w, int *h)
{
char fname[256];	/* full path to texture file */
char *magic;		/* P6 at start */
int max;		/* max size of values */
unsigned char *dat;	/* data from file */
int actual;		/* bytes really read */

/* Open file and read header
 */
magic = get_ppm_word(fd);
if (!strequ(magic,"P6")) {
	logerror("Data is not raw ppm");
	return NULL;
	}
*w = atoi(get_ppm_word(fd));
*h = atoi(get_ppm_word(fd));
max = atoi(get_ppm_word(fd));
if (max != 255) {
	logerror("ppm data does not have a max value of 255");
	return NULL;
	}

dat = scalloc(1,(*w)*(*h)*3);
actual = fdread(fd,dat,1,(*w)*(*h)*3);
if (actual != (*w)*(*h)*3) {
	logerror("Error reading ppm data. Only read %d bytes",actual);
	free(dat);
	return NULL;
	}
close(fd);
return dat;
}


/* get_ppm_word
 * Read one word from a ppm file, ignoring ppm comment lines
 */
static char *get_ppm_word(int fd)
{
static char buf[256];

fdgetw(fd,buf,256);
if (buf[0] == '#') {
        /* comment line!! */
        char toss[1024];
        fdgets(fd,toss,1024);
        return get_ppm_word(fd);
        }
return buf;
}


/* encode_rgb
 * 6-bit encodes size bytes from the array in to (4/3)*size bytes in the
 * array out.
 */
void encode_rgb(unsigned char *in, char *out, int size)
{
int i,j;

for(i=0,j=0; i<size; i+=3,j+=4) {
	int k;

	out[j]   = (in[i]>>2);
	out[j+1] = (in[i]<<4) + (in[i+1]>>4);
	out[j+2] = (in[i+1]<<2) + (in[i+2]>>6);
	out[j+3] = (in[i+2]);
	for(k=j; k<j+4; k++)
		out[k] = (out[k]&0x3f)+60;
	}
}


/* decode_rgb
 * 6-bit decodes size bytes of data from the array in to (3/4)*size bytes
 * of data in the array out.
 */
void decode_rgb(char *in, unsigned char *out, int size)
{
int i,j;

for(i=0,j=0; i<size; i+=4,j+=3) {
	int k;

	for(k=i; k<i+4; k++)
		in[k] = in[k]-60;
	out[j]   = (in[i]<<2) + (in[i+1]>>4);
	out[j+1] = (in[i+1]<<4) + (in[i+2]>>2);
	out[j+2] = (in[i+2]<<6) + (in[i+3]);
	}
}


/* write_encoded_texture
 * Output texture data to a file descriptor in 6-bit encoded format
 */
void write_encoded_texture(int fd, unsigned char *data, int w, int h)
{
int tc = w*h*3;
int i;

for(i=0; i<tc; i+=150) {
	char enc[201];		/* encoded data */
	int len;

	if (tc-i >= 150) len = 150;
	else		 len = tc-i;
	encode_rgb(&data[i],enc,len);
	enc[(len/3)*4] = 0;
	fdprintf(fd,"\"%s\"\n",enc);
	}
}

