/* user.c
 *
 * Functions used by client and avatar viewing programs
 */
#include "sim.h"
#include "client.h"

/* current_position
 * Used by graphical clients to interpolate the view position along it's
 * movement path.
 */
void current_position(plview *plv, double *x, double *y, double *z)
{
if (plv->dst < plv->range) {
	double mpr = plv->dst / plv->range;

	*x = plv->x1 + (plv->x2 - plv->x1)*mpr;
	*y = plv->y1 + (plv->y2 - plv->y1)*mpr;
	*z = plv->z1 + (plv->z2 - plv->z1)*mpr;
	}
else {
	*x = plv->x2;
	*y = plv->y2;
	*z = plv->z2;
	}
}


static userprefs currprefs;	/* current preferences state */

/* get_prefs
 * Read the current preferences from the user's .simrc file
 */
userprefs *get_prefs(userprefs *p)
{
static int readprefs = 0;	/* have prefs been read from file? */

if (!readprefs) {
	char prfile[256];
	int fd;

	/* Default prefs, to be overridden by those from file
	 */
	currprefs.default_3d = 2;	/* 3d level (best) */
	currprefs.recved_data = 2;	/* received data action (display) */
	currprefs.move_speed = 2.0;	/* movement speed (2.0m/s) */
	currprefs.password[0] = 0;	/* no password */
	currprefs.display_transfer = 1;	/* do display */
	currprefs.transfer_block = BUFSIZ; /* use a sane block size */
	currprefs.w = 500;
	currprefs.h = 510;
	currprefs.x = -1;
	currprefs.y = -1;

	sprintf(prfile,"%s/.simrc",get_home_dir());
	if ((fd = open(prfile,O_RDONLY)) != -1) {
		char line[1024], *pr, *vl;

		while(fdgets(fd,line,1024)) {
			if (!(pr = strtok(line,"\t")) ||
			    !(vl = strtok(NULL,"\n")))
				break;
			if (strequ(pr,"default_3d:"))
				currprefs.default_3d = atoi(vl);
			if (strequ(pr,"recved_data:"))
				currprefs.recved_data = atoi(vl);
			if (strequ(pr,"move_speed:"))
				currprefs.move_speed = atof(vl);
			if (strequ(pr,"password:"))
				strcpy(currprefs.password,vl);
			if (strequ(pr,"display_transfer:"))
				currprefs.display_transfer = atoi(vl);
			if (strequ(pr,"transfer_block:"))
				currprefs.transfer_block = atoi(vl);
			}
		close(fd);
		}
	readprefs = 1;
	}
if (p) *p = currprefs;
return &currprefs;
}


/* put_prefs
 * Store a set of preferences in memory
 */
void put_prefs(userprefs *p)
{
currprefs = *p;
}


/* save_prefs
 * Write the current preferences to the user's .simrc file
 */
int save_prefs(void)
{
char prfile[256];
int fd;

sprintf(prfile,"%s/.simrc",get_home_dir());
if ((fd = creat(prfile,0700)) == -1)
	return 0;
fdprintf(fd,"default_3d:\t%d\n",currprefs.default_3d);
fdprintf(fd,"recved_data:\t%d\n",currprefs.recved_data);
fdprintf(fd,"move_speed:\t%f\n",currprefs.move_speed);
fdprintf(fd,"password:\t%s\n",currprefs.password);
fdprintf(fd,"display_transfer:\t%d\n",currprefs.display_transfer);
fdprintf(fd,"transfer_block:\t%d\n",currprefs.transfer_block);
close(fd);
return 1;
}


static userservprefs currsprefs;	/* current server preferences state */

/* get_sprefs
 * Get the user's current server preferences 
 */
userservprefs *get_sprefs(userservprefs *p)
{
static int readsprefs = 0;

if (!readsprefs) {
	char sprfile[256];
	int fd;

	/* Default server
	 */
	strcpy(currsprefs.server,"indy14.cs.monash.edu.au");
	currsprefs.port = SIM_PORT;

	/* Read from file
	 */
	sprintf(sprfile,"%s/.simrc_server",get_home_dir());
	if ((fd = open(sprfile,O_RDONLY)) != -1) {
		char line[1024], *pr, *vl;

		while(fdgets(fd,line,1024)) {
			if (!(pr = strtok(line,"\t")) ||
			    !(vl = strtok(NULL,"\n")))
				break;
			if (strequ(pr,"server:"))
				strcpy(currsprefs.server,vl);
			if (strequ(pr,"port:"))
				currsprefs.port = atoi(vl);
			}
		close(fd);
		}
	readsprefs = 1;
	}
if (p) *p = currsprefs;
return &currsprefs;
}


/* put_sprefs
 * Store the user's server preferences in memory
 */
void put_sprefs(userservprefs *p)
{
currsprefs = *p;
}


/* save_sprefs
 * Write user's current server preferences to the .simrc_server file
 */
int save_sprefs(void)
{
char sprfile[256];
int fd;

sprintf(sprfile,"%s/.simrc_server",get_home_dir());
if ((fd = creat(sprfile,0700)) == -1)
	return 0;
fdprintf(fd,"server:\t%s\n",currsprefs.server);
fdprintf(fd,"port:\t%d\n",currsprefs.port);
close(fd);
return 1;
}

